# Dealing with frealix .ali files
class FrealixAli
	NAME = "Frealix alignment"

	require_relative "FrealixWaypoints.rb"

	def initialize(filename="waypoints.ali")
		@filename = filename
		@file_last_read = Time.new(1900)	# We pretend the file was last read in 1900
		@units = 'P'
		@header = "!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Pixels   par_vers. 4\n"
		if  File.exist?(@filename)
			readFromDisk()
		else
			$LOG.warn() {"File does not exist: #{@filename}"}
		end
	end

	# Read file contents from disk
	def readFromDisk
		if (File.exists?(@filename))
			@file_last_read = Time.now
			@str = File.read(@filename)
			@lines = @str.split(/\n/)
		else
			# File doesn't exist
		end
	end
	
	# Write to disk
	def writeToDisk
	  File.write(@filename,@str)
	end

	# Return an array of waypoints from the contents of the file
	def getWaypoints
		wps = []
		@lines.each do |line|
			next if line.match(/^[!#]/)?true:false
			wps << FrealixWaypoint.new(:string=>line)
			$LOG.debug("getWaypoints") {print wps[0]}
		end
		return wps
	end
	
	# Replace the string content with information from the array of waypoints
	def putWaypoints(wps)
	  @str = @header
	  wps.each{|wp|
	    @str += wp.toString
	  }
	end
end
