# Filament class
class FrealixFilament
	NAME = "Frealix filament"

	attr_accessor :score_wps
	attr_accessor :curvature, :overtwist, :stretch, :changeOfDirection
	attr_accessor	:log_filename
	attr_accessor	:num
  attr_accessor :wps, :wps_changes, :film_num
  attr_accessor :polarity_flipped
  attr_accessor :score_no_restraints
  attr_accessor :num_subunits
  attr_accessor :polarity_dscore
  attr_reader   :wpdist_mean, :wpdist_mean_nofal
  attr_reader   :wpdist_sd, :wpdist_sd_nofal

	def initialize(options={})
		if options[:wps]
			@wps = options[:wps].dclone
		else
			@wps = []
		end
		if (options[:num])
			@num = options[:num]
		else
			@num = 0
			@num = wps[0].fil_num if (@wps.size > 0)
		end
		if (options[:wps_changes])
		  @wps_changes = options[:wps_changes]
		else
		  @wps_changes = []
		end
		@log_filename = ""
		@polarity_flipped = false
		@num_subunits = 0 #Number of subunits contributing to the 3D reconstruction
		@polarity_dscore = 0.0  #Difference in score between the two polarities
		@score_wps = []
	end

	# Return the array of path lengths
	def getWaypointPathLengths
		pl = []
		@wps.each { |wp| pl << wp.path_length }
		return pl
	end
	
	# Return the average waypoint-to-waypoint distance
	def updateWpToWpDistStats
    distances = []
    distances_nofal = []
	  @wps.each_index{|i|
      if (i < @wps.size-1)
        dist = @wps[i+1].path_length - @wps[i].path_length
        distances << dist
        if (i > 0 && i < @wps.size-2)
          distances_nofal << dist
        end
      end
    }
    (@wpdist_mean,@wpdist_sd) = distances.sample_stats
    (@wpdist_mean_nofal,@wpdist_sd_nofal) = distances_nofal.sample_stats
  end

	def removeFalWps!
		@wps.delete_at(0)
		@wps.delete_at(-1)
	end

	# Return the arc/chord length ratio
	def getArcChordLengthRatio
	  FrealixWaypoint.arcLength(@wps[1],@wps[-1])/FrealixWaypoint.chordLength(@wps[1],@wps[-1])
	end
	def getArcChordLengthRatioNoFAL
	  FrealixWaypoint.arcLength(@wps[2],@wps[-2])/FrealixWaypoint.chordLength(@wps[2],@wps[-2])
	end
	  

	# Return the array of scores
	def getWaypointScores
		s = []
		@wps.each { |wp| s << wp.score }
		return s
	end
	
	#Zero most properties except XY
	def keepXYOnly!
	  @wps.each {|wp| wp.keepXYOnly!}
	end
	
	# Fix the last WP's phi value
	def fixLastWpPhi!
	  #if ( ((@wps[-1].phi > @wps[-2].phi) && (@wps[3].phi < @wps[2].phi)) || ((@wps[-1].phi < @wps[-2].phi) && (@wps[3].phi > @wps[2].phi)) ) then
	    @wps[-1].phi = @wps[-2].phi + ( @wps[-1].path_length - @wps[-2].path_length ) / ( @wps[-2].path_length - @wps[-3].path_length ) * (@wps[-2].phi - @wps[-3].phi)
	  #end
	end
	
	# Ensure that subunit_num is always increasing
	def fixSubunitNum!
	  @wps.each_with_index {|wp,i|
	    if (i > 0) then
	      if (@wps[i].subunit_num <= @wps[i-1].subunit_num) then
	              
	        # Guess a new value for subunit_num by linear interpolation
	        if (i < @wps.length-1) then
            # Find the next waypoints which has a greater subunit_num
            j = i+1
            while @wps[j].subunit_num <= @wps[i].subunit_num do
              j += 1
            end  
	          if (@wps[i].subunit_num < @wps[j].subunit_num) then
	            @wps[i].subunit_num = @wps[i-1].subunit_num + (@wps[j].subunit_num - @wps[i-1].subunit_num) / (@wps[j].path_length - @wps[i-1].path_length) * (@wps[i].path_length - @wps[i-1].path_length)
	            puts "Info: fixed subunit_num of waypoints #{i+1} of filament #{@num}"
            else
	            puts "ERROR: waypoint #{i+1} of filament #{@num} cannot be handled"
	            exit -1
	          end 
	        else
	          # We're on the last waypoint
	          @wps[i].subunit_num = @wps[i-1].subunit_num + (@wps[i].path_length - @wps[i-1].path_length)/1.403
	          puts "Warning: assuming TMV symmetry for waypoint #{i+1} of filament #{@num}"
	        end
	      end
	    end
    }
  end

  def getScoreWaypointXs
    t = []
    @score_wps.each { |wp| t << wp.x }
    return t
  end
  def getScoreWaypointYs
    t = []
    @score_wps.each { |wp| t << wp.y }
    return t
  end
  def getWaypointXs
    t = []
    @wps.each { |wp| t << wp.x }
    return t
  end
  def getWaypointYs
    t = []
    @wps.each { |wp| t << wp.y }
    return t
  end
  def getWaypointZs
    t = []
    @wps.each { |wp| t << wp.z }
    return t
  end
  def getWaypointThetas
		t = []
		@wps.each { |wp| t << wp.theta }
		return t
	end
	def getWaypointPsis
		p = []
		@wps.each { |wp| p << wp.psi }
		return p
	end
	
	def addToPhi(val)
	  @wps.map! {|p| p.addToPhi!(val);  p}
	end

	def addWaypoint(wp)
		self.wps << wp
	end

	def score
		self.wps[0].score.to_f
	end
	
	def score_std
	  if (score_wps.count > 1)
	    (mean,std) = self.score_wps.collect{|wp| wp.score}.sample_stats
	    std.round(4)
	  else
	    0.0
	  end
	end
	  

	def length
	  (self.wps[self.wps.size-1].path_length.to_f - self.wps[0].path_length.to_f).round(1)
	end
	
	def lengthNoFal
    (self.wps[self.wps.size-2].path_length.to_f - self.wps[1].path_length.to_f).round(1)
	end

	def film_num
	  self.wps[0].film_num
  end

  def average_z
    a = 0.0
    @wps.each{|wp| a += wp.z}
    a = a / @wps.size.to_f
  end
  def average_x
    a = 0.0
    @wps.each{|wp| a += wp.x}
    a = a / @wps.size.to_f
  end
  def average_y
    a = 0.0
    @wps.each{|wp| a += wp.y}
    a = a / @wps.size.to_f
  end
	
	# Make sure psi and theta are as smooth as possible, even
	# if it means breaking the convention that -90<=psi<=90
	def psiThetaSmooth
		require_relative "rubylix.rb"
		keep_theta_positive = @wps[0].theta > 0.0
		@wps.each { |wp|
			theta_positive = wp.theta > 0.0
			if ( (theta_positive && !keep_theta_positive) || (!theta_positive && keep_theta_positive))
				wp.theta = -1.0 * wp.theta
				wp.psi   = wp.psi.m180to180
			end
		}
	end

	def psiMeanSD
		require_relative "rubylix.rb"
		psi = []
		@wps.each { |wp| psi << wp.psi.to_f }
		psi.sample_stats
	end
	def thetaMeanSD
		require_relative "rubylix.rb"
		theta = []
		@wps.each { |wp| theta << wp.theta.to_f }
		theta.sample_stats
	end
	def thetaAbsMeanSD
		require_relative "rubylix.rb"
		theta = []
		@wps.each { |wp| theta << wp.theta.to_f.abs }
		theta.sample_stats
	end
	
	# Split a filament into several filaments, each corresponding to one of its protofilaments
	def getProtofilaments
	  pfs = Array.new(2,FrealixFilament)
	  pfs[0] = FrealixFilament.new(:wps=>self.wps.dclone,:num=>self.num)
    pfs[1] = FrealixFilament.new(:wps=>self.wps.dclone,:num=>self.num)
	  pfs[1].wps.each {|wp| wp.addToPhi(180.0)}
	  pfs
	end

	# Return an array of filaments, given an array of waypoints
	def self.filamentsFromWaypoints(wps)
		fils = []
		wps.each do |wp|
			#$LOG.debug("filamentsFromWaypoints") {"Looking for home for WP with score #{wp.score}"}
			found_existing_fil = false
			# If the current waypoint does belong to any of the existing
			# filaments, we'll need to create a new filament
			# So let's check
			if (fils.size >0)
				fils.each_with_index do |fil,i|
					# For each filament, check if the filament number matches wp's fil_num.
					#$LOG.debug("filamentsFromWaypoints") {"#{wp.fil_num.to_i} #{fil.num.to_i}"}
					if (wp.fil_num.to_i == fil.num.to_i)
						fils[i].addWaypoint(wp)
						$LOG.debug("filamentsFromWaypoints") {"WP with score #{wp.score} added to filament index #{i}"}
						found_existing_fil = true
						break # We're done searching
					end
				end
			end
			# If we didn't find an existing filament, add it to the array
			if (!found_existing_fil)
				fils[fils.size] = FrealixFilament.new(:wps=>[wp])
				$LOG.debug("filamentsFromWaypoints") {"Starting new filament score #{wp.score}"}
			end
		end
		return fils
	end

	# Write out array of filaments to disk as a waypoints.ali file
	def self.filamentsToAliFile(fils,filename)
    # Open the file
    ali_file = File.open(filename,"w")
    # Write out header line
    printf(ali_file,"!Fil.#   WP # Film #           X           Y           Z  PathLength          Phi        Theta          Psi   Subunit #    Score Refine? Units Angstr   par_vers. 4\n")
    # Write out waypoints
    fils.each {|fil|
      fil.wps.each_with_index {|wp,i|
        wp.fil_num = fil.num
        wp.num = i + 1
        printf(ali_file,wp.toString)
      }
    }
    ali_file.close
  end
  
  def changePixelSize(old_pixel_size,new_pixel_size)
    # By default, coordinates are stored in Angstroms, so we first convert them to pixel coordinates, then convert back
    # using the new pixel size
    @wps.each {|wp|
      wp.convertAngToPix(old_pixel_size)
      wp.convertPixToAng(new_pixel_size)
    }
  end
  
  def addToNum(val)
    @num += val
  end
  def mulNum(val)
    @num *= val
  end
end
