# Dealing with directory where frealix refinement happens
class FrealixRefinementDirectory
  NAME = "Frealix refinement directory"

  # Initialiser method with default parameter
  def initialize(directory=ENV['PWD'])
    @directory = directory
    # Check directory exists!
    if (!Dir.exists?(@directory))
      puts "Directory does not exist: #{@directory}"
    end
    @round_dir_root = "frealix_round_"
  end
  
  
  # The root name for round subdirectories
  attr_reader :round_dir_root
  attr_reader :directory
  
  # Create subdirectory for the next round and return its name
  def mkdirCurrentRound()
    dirname = roundSubDir(lastCompletedRound()+1)
    Dir.mkdir(dirname)
    return dirname+"/"
  end
  
  # Return the round number we're on
  def currentRound()
    self.lastCompletedRound() + 1
  end
  
  # Return the directory for the round we're currently processing
  def currentRoundDir()
    self.roundSubDir(currentRound())
  end
  
  def previousRoundDir()
    self.roundSubDir(currentRound()-1)
  end
  
  # Work out latest completed round
  def lastCompletedRound()
    lastCompletedRound = 0
    while (roundComplete?(lastCompletedRound)) do
      lastCompletedRound += 1
    end
    lastCompletedRound -= 1
    return lastCompletedRound
  end
  
  def roundComplete?(i)
    dir = roundSubDir(i)
    File.exists?(dir+"/3drec.mrc") && File.exists?(dir+"/waypoints.ali")
  end
  
  # Return the subdir name for the round given
  def roundSubDir(round)
    sprintf("%s%03i",@round_dir_root,round)
  end
  
  # Return the round number given the subdirname
  def self.roundNumFromSubDir(subdir)
    pat = /#{@round_dir_root}_(\d\d\d)/
    subdir.match(pat)[1].to_i
  end
  
end
