#  FrealixWaypoints.rb
#
#  Copyright 2014 Howard Hughes Medical Institute.
#  All rights reserved.
#  Use is subject to Janelia Farm Research Campus Software Copyright 1.1
#  license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
#  
#
#
class FrealixWaypoint
	NAME = "Frealix waypoint"

	attr_reader :fil_num, :film_num, :num
	attr_accessor :fil_num, :num, :refine
	attr_accessor :psi, :theta, :phi
	attr_accessor :x,:y,:z
	attr_accessor :subunit_num
	attr_reader :path_length, :score

	def initialize(options = {})
		# Default values for the options
		defaults = {	:fil_num		=>	0,
						:num			=>	0,
						:film_num		=>	0,
						:x				=>	0.0,
						:y				=>	0.0,
						:z				=>	0.0,
						:phi			=>	0.0,
						:theta			=>	0.0,
						:psi			=>	0.0,
						:path_length	=>	0.0,
						:subunit_num	=>	0.0,
						:score			=> 	0.0,
						:string			=>	"",
						:refine			=>	"T",
						:curvature		=>	0.0,
						:torsion		=>	0.0,
						:stretch		=>	0.0
					}

		# Apply defaults (?)
		@refine = defaults[:refine]
					
		# If string was supplied, we parse it assuming it was
		# from a waypoints.ali file
		if (options[:string] != defaults[:string] && options[:string])
			populateFromString(options[:string])
		end

		# The other optional arguments override the string
		@fil_num 		= options[:fil_num]		if (options[:fil_num] 		)#&& options[:fil_num] 		!= defaults[:fil_num])
		@film_num 				= options[:film_num] 	if (options[:film_num] 		)#&& options[:film_num] 		!= defaults[:film_num])
		@num 				= options[:num] 		if (options[:num] 			)#&& options[:num] 			!= defaults[:num])
		@x 				= options[:x] 			if (options[:x] 			)#&& options[:x] 				!= defaults[:x])
		@y 				= options[:y] 			if (options[:y] 			)#&& options[:y] 				!= defaults[:y])
		@z 				= options[:z] 			if (options[:z] 			)#&& options[:z] 				!= defaults[:z])
		@path_length 	= options[:path_length]	if (options[:path_length]	)#&& options[:path_length] 	!= defaults[:path_length])
		@phi 			= options[:phi] 		if (options[:phi] 			)#&& options[:phi] 			!= defaults[:phi])
		@theta 			= options[:theta] 		if (options[:theta] 		)#&& options[:theta] 			!= defaults[:theta])
		@psi			= options[:psi] 		if (options[:psi] 			)#&& options[:psi] 			!= defaults[:psi])
		@subunit_num	= options[:subunit_num]	if (options[:subunit_num] 	)#&& options[:subunit_num] 	!= defaults[:subunit_num])
		@score 			= options[:score] 		if (options[:score] 		)#&& options[:score] 			!= defaults[:score])
		@refine 		= options[:refine] 		if (options[:refine] 		)#&& options[:refine] 		!= defaults[:refine])
		@curvature 		= options[:curvature] 	if (options[:curvature] 	)#&& options[:curvature] 		!= defaults[:curvature])
		@torsion 		= options[:torsion] 	if (options[:torsion] 		)#&& options[:torsion] 		!= defaults[:torsion])
		@stretch 		= options[:stretch] 	if (options[:stretch] 		)#&& options[:stretch] 		!= defaults[:stretch])
	end

	def score
		@score
	end
	
	# Everything is re-initialised except XY, number, fil number etc
	def keepXYOnly!
    #@fil_num    = 
    #@film_num        = 
    #@num        = 
    #@x        = 
    #@y        = 
    @z        = 0.0 
    @path_length  = 0.0 
    @phi      = 0.0 
    @theta      = 0.0 
    @psi      = 0.0 
    @subunit_num  = 0.0 
    @score      = 0.0 
    #@refine     = 
    @curvature    = 0.0 
    @torsion    = 0.0 
    @stretch    = 0.0  
	end

	# Print out waypoint information to string, formatted like a line from an ali file
	def toString
	  sprintf("%6i %6i %6i %11.3f %11.3f %11.3f %11.3f %12.3f %12.3f %12.3f %11.3f %8.3f %7s \n",
	          @fil_num,@num,@film_num,@x,@y,@z,@path_length,@phi,@theta,@psi,@subunit_num,@score,@refine)
	end

	# Populate a waypoint with values taken from a string from a waypoints.ali file
	def populateFromString(string)
		(@fil_num,@num,@film_num,@x,@y,@z,@path_length,
		@phi,@theta,@psi,@subunit_num,@score,@refine) = string.split
		#
		@fil_num = @fil_num.to_i
		@num = @num.to_i
		@x = @x.to_f
		@y = @y.to_f
		@z = @z.to_f
		@path_length = @path_length.to_f
		@phi = @phi.to_f
		@theta = @theta.to_f
		@psi = @psi.to_f
		@subunit_num = @subunit_num.to_f
		@score = @score.to_f
		$LOG.debug() {"New waypoint has score #{@score}"}
	end
	
	def convertPixToAng(pixel_size)
	  @x = (@x-1.0) * pixel_size
	  @y = (@y-1.0) * pixel_size
	  @z = (@z-1.0) * pixel_size
	  @path_length = @path_length * pixel_size
	end
	
	def convertAngToPix(pixel_size)
    @x = (@x / pixel_size) + 1.0
    @y = (@y / pixel_size) + 1.0
    @z = (@z / pixel_size) + 1.0
    @path_length = (@path_length / pixel_size)
	end
	
	def addToPhi!(val)
	  @phi += val
	end
	
	def self.arcLength(one,two)
	  two.path_length - one.path_length
	end
	def self.chordLength(one,two)
	  Math.sqrt((two.x-one.x)**2+(two.y-one.y)**2+(two.z-one.z)**2)
	end
end
