#!/usr/bin/env ruby
#  cat_slices.rb
#
#  Concatenate central slices of individual filament reconstructions
#
#  Copyright 2014 Howard Hughes Medical Institute.
#  All rights reserved.
#  Use is subject to Janelia Farm Research Campus Software Copyright 1.1
#  license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
#  

# We need to sort the files in ascending order of filament, so let's define a function to extract the filament number from the filename
def getFilamentNum(filename)
  pat=/_(\d\d\d\d\d\d).mrc/
  m = filename.match(pat)
  m[1].to_i
end

require_relative "rubylix.rb"
require_relative "FrealixFilament.rb"
require_relative "FrealixAli.rb"

require 'optparse'

options = {}
defaults = {}

optparse = OptionParser.new do |opts|
  opts.banner = "Usage: cat_slices.rb [options] subdirectory"

  # Help message
  opts.on_tail("-h","--help","Show this message") do
    puts opts
    exit
  end

  options[:sort_by_score] = false
  opts.on("-s","--sort-by-score", "Sort filaments by score (descending)") do |b|
    options[:sort_by_score] = b
  end
  
  #options[:output_filename] = nil
  #opts.on("-o FILENAME","--output-filename FILENAME", "Output filename") do |b|
  #  options[:output_filename] = b
  #end
  
  #options[:pattern] = nil
  #  opts.on("-p PATTERN","--pattern PATTERN", "Glob pattern for slice filenames") do |b|
  #  options[:pattern] = b
  #end

end

optparse.parse!

if (ARGV.count < 1) then
  printf("Usage: cat_slices.rb [options] subdirectory\n")
  exit 1
end
@subdir = ARGV[0]

#if (options[:output_filename]) then
#  @output_filename = options[:output_filename]
#else
#  @output_filename = "#{@subdir}/3drec_slices.img"
#end


$LOG.info() {"Will concatenate central slices from individual filament reconstructions in #{@subdir} into #{@output_filename}"}
  
# Let's find all the files
#if (options[:pattern]) then
#  wild = options[:pattern]
#else
#  wild = "#{@subdir}/3drec_???_slice_??????.mrc"
#end

#
["xy","yz"].each { |plane|
	wild = "#{@subdir}/3drec_???_slice_#{plane}_??????.mrc"
	output_filename = "#{@subdir}/3drec_slices_#{plane}.img"
	$LOG.info() {"Using pattern: #{wild}"}
	files = Dir.glob(wild)
	
	# Let's also grab the filament information from the waypoints.ali file
	if (options[:sort_by_score]) then
	ali_in = FrealixAli.new(filename="#{@subdir}/waypoints.ali")
	fils = FrealixFilament.filamentsFromWaypoints(ali_in.getWaypoints)
	end
	
	# Sort the filaments
	if (options[:sort_by_score]) then
	files.sort!{|f1,f2| 
	num1 = getFilamentNum(f1)
	num2 = getFilamentNum(f2)
	# Find the filament in the array, based on its number
	ind1 = fils.index{|f| f.num == num1}
	ind2 = fils.index{|f| f.num == num2}
	score1 = fils[ind1].score
	score2 = fils[ind2].score
	score2 <=> score1 # To get descending score sorting
	}
	$LOG.info() {"Filaments were sorted by score (descending order)"}
	else
	# Let's sort them according to filament number
	files.sort!{|f1,f2| getFilamentNum(f1) <=> getFilamentNum(f2)}
	$LOG.info() {"Filaments were sorted by id number (ascending order)"}
	end
	
	
	
	# Let's write the sorted list to a text file
	str = files.join("\n")
	File.write("#{@subdir}/slices.txt",str)
	
	# Let's call Imagic to do the conversion
	#puts "#{ENV['IMAGIC_ROOT']}/stand/em2em.e"
	IO.popen("#{ENV['IMAGIC_ROOT']}/stand/em2em.e",mode="r+") { |io|
	io.puts "2D"
	io.puts "MRC"
	io.puts "MRC"
	io.puts "SET"
	io.puts "IMAGIC"
  io.puts "FILE"	
  io.puts "#{@subdir}/slices.txt"
  io.puts "#{output_filename}"
	io.puts "1.0"
	io.puts "NO"
	io.puts "NAME_OF_IMPORT_FILES"
	io.puts "NO"
	io.readlines
	}
	$LOG.info() {"Output filename: #{output_filename}"}
}

