MODULE DATES_AND_TIMES
    INTEGER,            PARAMETER               ::  SECONDS_IN_A_YEAR       =   31536000
    INTEGER,            PARAMETER               ::  SECONDS_IN_A_MONTH      =   2592000
    INTEGER,            PARAMETER               ::  SECONDS_IN_A_DAY        =   86400
    INTEGER,            PARAMETER               ::  SECONDS_IN_AN_HOUR      =   3600
    INTEGER,            PARAMETER               ::  SECONDS_IN_A_MINUTE     =   60
    INTEGER,            PARAMETER               ::  MILLISECONDS_IN_A_SECOND=   1000

CONTAINS
    !>    \brief    Returns an integer array with date & time information (useful when you want to compute a duration)
    FUNCTION GET_DATE_AND_TIME()
        IMPLICIT NONE
        ! Arguments
        ! Return value
        INTEGER    ::    GET_DATE_AND_TIME(8)
        ! Private variable
        INTEGER    ::    TEMP(8)
        ! Start work
        CALL DATE_AND_TIME(VALUES=TEMP)
        GET_DATE_AND_TIME = TEMP
    END FUNCTION GET_DATE_AND_TIME

    !>  \brief  Return the number of milliseconds between two dates & times
    PURE INTEGER FUNCTION MILLISECONDS_BTW_DATES(DATE_FIRST,DATE_SECOND)
        IMPLICIT NONE
        ! Arguments
        INTEGER,    INTENT(IN)  ::  DATE_FIRST(8), DATE_SECOND(8)
        ! Private variables
        INTEGER ::  DATE_DIFF(8)
        ! Start work
        DATE_DIFF = DATE_SECOND - DATE_FIRST

        MILLISECONDS_BTW_DATES =    DATE_DIFF(8) + MILLISECONDS_IN_A_SECOND * (DATE_DIFF(7) + DATE_DIFF(1) * SECONDS_IN_A_YEAR + &
                                    DATE_DIFF(2) * SECONDS_IN_A_MONTH + DATE_DIFF(3) * SECONDS_IN_A_DAY + &
                                    DATE_DIFF(5) * SECONDS_IN_AN_HOUR + DATE_DIFF(6) * SECONDS_IN_A_MINUTE)
    END FUNCTION MILLISECONDS_BTW_DATES

    !>  \brief  Return the number of seconds between two dates & times
    PURE INTEGER FUNCTION SECONDS_BTW_DATES(DATE_FIRST,DATE_SECOND)
        IMPLICIT NONE
        ! Arguments
        INTEGER,    INTENT(IN)  ::  DATE_FIRST(8), DATE_SECOND(8)
        ! Private variables

        ! Start work
        SECONDS_BTW_DATES = INT(MILLISECONDS_BTW_DATES(DATE_FIRST,DATE_SECOND)/MILLISECONDS_IN_A_SECOND)
    END FUNCTION SECONDS_BTW_DATES



    !>    \brief    Returns the date & time printed out in a nice format
    FUNCTION NICE_DATE_AND_TIME()
        IMPLICIT NONE
        CHARACTER(LEN=19)    ::    NICE_DATE_AND_TIME
        ! Private variables
        CHARACTER(LEN=8)    ::    CURRENT_DATE
        CHARACTER(LEN=10)    ::    CURRENT_TIME

        CALL DATE_AND_TIME(CURRENT_DATE,CURRENT_TIME)    ! This is an intrinsic function

        NICE_DATE_AND_TIME =   CURRENT_DATE(1:4) // '-' // CURRENT_DATE(5:6) // '-' // CURRENT_DATE(7:8) // ' ' // &
                               CURRENT_TIME(1:2) // ':' // CURRENT_TIME(3:4) // ':' // CURRENT_TIME(5:6) // ':' // CURRENT_TIME(7:8)

    END FUNCTION NICE_DATE_AND_TIME

    !>    \brief    Convert a duration in seconds to a nice character string
    FUNCTION NICE_TIME_FROM_SECONDS(SECONDS)
        IMPLICIT NONE
        ! Arguments
        REAL    ::    SECONDS
        ! Result
        CHARACTER(LEN=:),    ALLOCATABLE    ::    NICE_TIME_FROM_SECONDS
        ! Private variables
        CHARACTER(LEN=200)    ::    STRING
        INTEGER    ::    DURATION(4)
        ! Start work
        DURATION = NICE_DURATION(SECONDS)
        STRING = ''
        IF (DURATION(1) .NE. 0) THEN
            WRITE(STRING,'(I0,1X,A)') DURATION(1), 'day'
            IF (DURATION(1) .GT. 1) THEN
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), 's,'
            ELSE
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), ','
            ENDIF
        ENDIF
        IF (DURATION(2) .NE. 0) THEN
            WRITE(STRING,'(A,1X,I0,1X,A)') TRIM(ADJUSTL(STRING)), DURATION(2), 'hour'
            IF (DURATION(2) .GT. 1) THEN
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), 's,'
            ELSE
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), ','
            ENDIF
        ENDIF
        IF (DURATION(3) .NE. 0) THEN
            WRITE(STRING,'(A,1X,I0,1X,A)') TRIM(ADJUSTL(STRING)), DURATION(3), 'minute'
            IF (DURATION(3) .GT. 1) THEN
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), 's and'
            ELSE
                WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), ' and'
            ENDIF
        ENDIF
        WRITE(STRING,'(A,1X,I0,1X,A)') TRIM(ADJUSTL(STRING)), DURATION(4), 'second'
        IF (DURATION(4) .NE. 1) THEN
            WRITE(STRING,'(2A)') TRIM(ADJUSTL(STRING)), 's'
        ENDIF
        ! Copy the result over
        STRING = ADJUSTL(STRING)
		
		! I much prefer the first line, but gfortran has a bug which means 
		! I have to use the second line instead, which I think is equivalent
        !ALLOCATE(CHARACTER(LEN=LEN_TRIM(STRING)) :: NICE_TIME_FROM_SECONDS)
		ALLOCATE(NICE_TIME_FROM_SECONDS, SOURCE=(STRING(:LEN_TRIM(STRING))))
        
		NICE_TIME_FROM_SECONDS = TRIM(STRING)
    END FUNCTION NICE_TIME_FROM_SECONDS

    !>    \brief    Convert a duration in seconds to a duration in days, hours, minutes and seconds
    FUNCTION NICE_DURATION(SECONDS)
        IMPLICIT NONE
        ! Arguments
        REAL    ::    SECONDS
        ! Result
        INTEGER    ::    NICE_DURATION(4)    !<    1. days, 2. hours, 3. minutes, 4. seconds
        ! Private variables

        INTEGER    ::    ISECONDS
        ! Start work
        ISECONDS = NINT(SECONDS)
        NICE_DURATION(1) = ISECONDS/SECONDS_IN_A_DAY
        NICE_DURATION(2) = (ISECONDS-NICE_DURATION(1)*SECONDS_IN_A_DAY)/SECONDS_IN_AN_HOUR
        NICE_DURATION(3) = (ISECONDS-NICE_DURATION(1)*SECONDS_IN_A_DAY-NICE_DURATION(2)*SECONDS_IN_AN_HOUR)/SECONDS_IN_A_MINUTE
        NICE_DURATION(4) = (ISECONDS-NICE_DURATION(1)*SECONDS_IN_A_DAY-NICE_DURATION(2)*SECONDS_IN_AN_HOUR - &
                            NICE_DURATION(3)*SECONDS_IN_A_MINUTE)
    END FUNCTION NICE_DURATION
END MODULE DATES_AND_TIMES
