!>  \brief  Functions to generate figures & plots from filament objects
MODULE FILAMENTS_PLOTS


CONTAINS

    !>  \brief  Generate a simple figure of the path of a filament
    SUBROUTINE FILAMENT_THREED_PLOT(SELF,FILENAME)
        USE FILAMENTS_PATH
        USE PLOTS
        IMPLICIT NONE
        ! Arguments
        TYPE(FILAMENT),     INTENT(INOUT)  ::  SELF        !<  Filament object
        CHARACTER(LEN=*),   INTENT(IN)  ::  FILENAME    !<  Filename to write to
        ! Private variables
        LOGICAL,    PARAMETER    ::  DEBUG   =   .TRUE.
        TYPE(WAYPOINT), ALLOCATABLE ::  PATH(:)
        TYPE(PLOT)  ::FIG
        REAL,   ALLOCATABLE ::  X(:), Y(:), Z(:)
        ! Start work

        ! Obtain path of filament (every subunit)
        CALL FILAMENT_GET_PATH_SUBUNIT_STEPS(SELF,PATH,STEP=1)
        ! Set up plot
        CALL PLOT_INIT(FIG,DIMENSIONALITY=3,FILENAME=FILENAME,   &
                        ALTITUDE=20.0,AZIMUTH=45.0)
        ! Copy coordinates over to explicit arrays
        X = PATH%COO(1)
        Y = PATH%COO(2)
        Z = PATH%COO(3)
        ! Add series to the plot
        CALL PLOT_ADD_XYZ(FIG,X,Y,Z,DRAW_SHADOW=.TRUE.)
        ! Draw the plot
        CALL PLOT_DRAW(FIG)
    END SUBROUTINE FILAMENT_THREED_PLOT

END MODULE FILAMENTS_PLOTS
