!>  \brief  Safely compare float
!!
!!  See for example discussion here: http://computer-programming-forum.com/49-fortran/6bd3f309b12a9632.htm
!!
MODULE FLOAT_COMPARISONS
    IMPLICIT NONE

    INTEGER, PARAMETER  ::  NULP = 1

    CONTAINS

    !>  \brief  Check whether two floats are equal within numerical precision
    !!
    !!  Apparently, this is an F95 implementation of the scheme described in:
    !!  Knuth, Art of Computer Programming, vol 2, 3rd ed (1998), p. 233
    !!  but I haven't checked this myself
    !!  Also of interest may be http://fcmp.sourceforge.net/, which I also haven't checked out
    pure logical function doubles_are_equal(x,y)
        real(kind=8),   intent(in)    ::  x,y
        doubles_are_equal = abs(x-y) < nulp*2*spacing(max(abs(x),abs(y)))
    end function doubles_are_equal

    !>  \brief  Check whether two floats are equal within numerical precision
    !!
    !!  This alternative doesn't require the setting of a NULP parameter, and is due to
    !!  Paul van Dels (see http://computer-programming-forum.com/49-fortran/6bd3f309b12a9632.htm)
    pure logical function f_equal_alt(x,y)
        real(kind=8),   intent(in)    ::  x,y
        ! private variables
        real(kind=8)                ::  epsilon_exponent, epsilon_delta
        real(kind=8),   parameter   ::  one = 1.0d0
        real(kind=8),   parameter   ::  two = 2.0d0
        ! start work
        epsilon_exponent = exponent( max( abs(x), abs(y) ) )
        epsilon_delta    = max( epsilon(one) * two**epsilon_exponent, epsilon(one) )
        f_equal_alt = abs(x-y) < epsilon_delta
    end function f_equal_alt


!
! The routines below should be deprecated
!

    !>    \brief    Test whether two reals are significantly (i.e. above precision) different
    LOGICAL FUNCTION REALS_ARE_DIFFERENT(A,B)
        IMPLICIT NONE
        ! Arguments
        REAL(KIND=4),    INTENT(IN)    ::    A,B
        ! Private variables

        ! Start work
        REALS_ARE_DIFFERENT = ABS(A-B) .GT. SPACING(A)
    END FUNCTION REALS_ARE_DIFFERENT

    !>    \brief    Test whether two reals are significantly (i.e. above precision) different
    LOGICAL FUNCTION DBLES_ARE_DIFFERENT(A,B)
        IMPLICIT NONE
        ! Arguments
        REAL(KIND=8),    INTENT(IN)    ::    A,B
        ! Private variables

        ! Start work
        DBLES_ARE_DIFFERENT = ABS(A-B) .GT. SPACING(A)
    END FUNCTION DBLES_ARE_DIFFERENT

END MODULE FLOAT_COMPARISONS
