PROGRAM FREALIX

    USE RUNTIME_PARAMETERS
    use DatesAndTimes

    IMPLICIT NONE

    type(DateAndTime)   ::  myTime
    ! Current instance of the program

    ! Start work
    WRITE(*,'(A)') '<?xml version="1.0"?>'
    WRITE(*,'(A)') '<frealix>'
    DATE_START = myTime%DateAndTimeAsIntegers()

    ! Register signal handlers
    CALL REGISTER_SIGNAL_HANDLERS()


    ! Call the main subroutine
    CALL MAIN()
    DATE_FINISH = myTime%DateAndTimeAsIntegers()

    WRITE(*,'(/4A/)') myTime%DateAndTimeAsString(), ': FREALIX says goodbye (total execution time: ', &
                      myTime%DurationAsString(REAL(myTime%SecondsBetweenDates(DATE_START,DATE_FINISH))), ')'
    WRITE(*,'(A)') '</frealix>'

    CONTAINS

    ! The actual subroutine
    SUBROUTINE MAIN()

        ! Modules
        USE RUNTIME_PARAMETERS, ONLY : PRINT_WELCOME
        USE PARSERS                 ! This gives us access to a bunch of subroutines to parse files, command-line options etc.
        USE UI                      ! This gives us access to user-interaction subroutines

        ! Mode modules
        USE MODE_COMBINE_3DS
        USE MODE_FINALISE_3DS
        USE MODE_CALC_FSC
        USE MODE_CALC_FRSC
        USE MODE_RPS
        USE MODE_STACK_FROM_PAR
        USE MODE_SYNTH_CTF
        USE MODE_TEST_ROTATE
        USE MODE_TEST
        USE MODE_TEST_EXTRACT_SLICE
        USE MODE_TEST_INSERT_SLICE
        USE MODE_ORIGIN_TO_CENTER
        USE MODE_APPLY_FILTER
        USE MODE_TEST_SPLINE
        USE MODE_FREALIGN_STACK_TO_THREED
        USE MODE_FREALIGN_STACK_TO_FREALIX_ALI
        USE MODE_PAD
        USE MODE_FILAMENT_STARTUP
        USE MODE_FILAMENT_REFINE
        USE MODE_HEXTEND
        USE MODE_HMEASURE
        USE MODE_BIN
        USE MODE_ALIGN
        USE MODE_STACK_ALIGN_TO_SUM
        USE MODE_MASK
        USE MODE_HMASK
        USE MODE_CONVOLVE
        USE MODE_BIN_PAR
        USE MODE_EXTRACT
        USE MODE_FIND_FILAMENTS
        USE MODE_SPLIT_PAR
        USE MODE_SWAPXY
        USE MODE_REBUILD_SEAM
        USE MODE_SYNTH_MICROGRAPH
        USE MODE_ROTATE
        USE MODE_BFACTOR
        USE MODE_DENSITY_HISTOGRAM
        USE MODE_AMYLOID_LAYER_LINE
        USE MODE_UNIT_TESTS
        USE MODE_RANDOMIZE_WAYPOINTS
        USE MODE_FREALIX_TO_FREALIGN
        USE MODE_APPLY_CTF
        !USE MODE_FILAMENT_DISPLAY
        USE MODE_NORMALISE
        USE MODE_ESTIMATE_PERSISTENCE_LENGTH
        USE MODE_MOVIE_ALIGN

        IMPLICIT NONE

        ! Check the command-line - get the control file name
        CALL PARSE_COMMAND_LINE_ARGUMENTS

        ! Get global run-time parameters from the control file
        CALL GET_GLOBAL_RTPS()

        ! Welcome message - make this a subroutine in runtime_parameters.f90
        CALL PRINT_WELCOME()

        ! Check which mode the user wanted us to run
        CALL GET_FREALIX_MODE

        ! Call the subroutine corresponding to the mode the user asked for
        SELECT CASE(UPPER_CASE(FREALIX_MODE))
            CASE('COMBINE_3DS')
                WRITE(*,'(A)') 'Frealix will run its COMBINE_3DS mode'
                CALL MODE_COMBINE_3DS_INIT_AND_RUN()
            CASE ('FINALISE_3DS')
                WRITE(*,'(A)') 'Frealix will run its FINALISE_3DS mode'
                CALL MODE_FINALISE_3DS_INIT_AND_RUN()
            CASE('CALC_FSC')
                WRITE(*,'(A)') 'Frealix will run its CALC_FSC mode'
                CALL MODE_CALC_FSC_INIT_AND_RUN
            CASE('CALC_FRSC')
                WRITE(*,'(A)') 'Frealix will run its CALC_FRSC mode'
                CALL MODE_CALC_FRSC_INIT_AND_RUN
            CASE('RPS')
                WRITE(*,'(A)') 'Frealix will run its RPS mode'
                CALL MODE_RPS_INIT_AND_RUN
            CASE('STACK_FROM_PAR')
                WRITE(*,'(A)') 'Frealix will run its STACK_FROM_PAR mode'
                CALL MODE_STACK_FROM_PAR_INIT_AND_RUN
            CASE('SYNTH_CTF')
                WRITE(*,'(A)') 'Frealix will run its SYNTH_CTF mode'
                CALL MODE_SYNTH_CTF_INIT_AND_RUN
            CASE('TEST_ROTATE')
                WRITE(*,'(A)') 'Frealix will run its TEST_ROTATE mode'
                CALL MODE_TEST_ROTATE_INIT_AND_RUN
            CASE('TEST')
                WRITE(*,'(A)') 'Frealix will run its TEST mode'
                CALL MODE_TEST_INIT_AND_RUN
            CASE('TEST_EXTRACT_SLICE')
                WRITE(*,'(A)')  'Frealix will run its TEST_EXTRACT_SLICE mode'
                CALL MODE_TEST_EXTRACT_SLICE_INIT_AND_RUN
            CASE('TEST_INSERT_SLICE')
                WRITE(*,'(A)')  'Frealix will run its TEST_INSERT_SLICE mode'
                CALL MODE_TEST_INSERT_SLICE_INIT_AND_RUN
            CASE('ORIGIN_TO_CENTER')
                WRITE(*,'(A)')  'Frealix will run its ORIGIN_TO_CENTER mode'
                CALL MODE_ORIGIN_TO_CENTER_INIT_AND_RUN
            CASE('APPLY_FILTER')
                WRITE(*,'(A)')  'Frealix will run its APPLY_FILTER mode'
                CALL MODE_APPLY_FILTER_INIT_AND_RUN
            CASE('TEST_SPLINE')
                WRITE(*,'(A)')  'Frealix will run its TEST_SPLINE mode'
                CALL MODE_TEST_SPLINE_INIT_AND_RUN
            CASE('FREALIGN_STACK_TO_THREED')
                WRITE(*,'(A)')  'Frealix will run its FREALIGN_STACK_TO_THREED mode'
                CALL MODE_FREALIGN_STACK_TO_THREED_INIT_AND_RUN
            CASE('FREALIGN_STACK_TO_FREALIX_ALI')
                WRITE(*,'(A)')  'Frealix will run its FREALIGN_STACK_TO_FREALIX_ALI mode'
                CALL MODE_FREALIGN_STACK_TO_FREALIX_ALI_INIT_AND_RUN
            CASE('PAD')
                WRITE(*,'(A)')  'Frealix will run its PAD/CROP/RESIZE mode'
                CALL MODE_PAD_INIT_AND_RUN
            CASE('CROP')
                WRITE(*,'(A)')  'Frealix will run its PAD/CROP/RESIZE mode'
                CALL MODE_PAD_INIT_AND_RUN
            CASE('RESIZE')
                WRITE(*,'(A)')  'Frealix will run its PAD/CROP/RESIZE mode'
                CALL MODE_PAD_INIT_AND_RUN
            !CASE('FILAMENT_STARTUP_OLD')
            !   WRITE(*,'(A)')  'Frealix will run its FILAMENT_STARTUP_OLD mode'
            !   CALL MODE_FILAMENT_STARTUP_OLD
            CASE('FILAMENT_STARTUP')
                WRITE(*,'(A)')  'Frealix will run its FILAMENT_STARTUP mode'
                CALL MODE_FILAMENT_STARTUP_INIT_AND_RUN
            CASE('FILAMENT_REFINE')
                WRITE(*,'(A)')  'Frealix will run its FILAMENT_REFINE mode'
                CALL MODE_FILAMENT_REFINE_INIT_AND_RUN
            CASE('HEXTEND')
                WRITE(*,'(A)')  'Frealix will run its HEXTEND mode'
                CALL MODE_HEXTEND_INIT_AND_RUN
            CASE('HMEASURE')
                WRITE(*,'(A)')  'Frealix will run its HMEASURE mode'
                CALL MODE_HMEASURE_INIT_AND_RUN
            CASE('BIN')
                !WRITE(*,'(A)')  'Frealix will run its BIN mode'
                CALL MODE_BIN_INIT_AND_RUN
            CASE('CONVOLVE')
                WRITE(*,'(A)')  'Frealix will run its CONVOLVE mode'
                CALL MODE_CONVOLVE_INIT_AND_RUN
            CASE('BIN_PAR')
                WRITE(*,'(A)')  'Frealix will run its BIN_PAR mode'
                CALL MODE_BIN_PAR_INIT_AND_RUN
            CASE('ALIGN')
                WRITE(*,'(A)') 'Frealix will run its ALIGN mode'
                CALL MODE_ALIGN_INIT_AND_RUN
            CASE('STACK_ALIGN_TO_SUM')
                WRITE(*,'(A)') 'Frealix will run its STACK_ALIGN_TO_SUM mode'
                CALL MODE_STACK_ALIGN_TO_SUM_INIT_AND_RUN
            CASE ('MASK')
                WRITE(*,'(A)') 'Frealix will run its MASK mode'
                CALL MODE_MASK_INIT_AND_RUN
            CASE ('HMASK')
                WRITE(*,'(A)') 'Frealix will run its HMASK mode'
                CALL MODE_HMASK_INIT_AND_RUN
            CASE ('EXTRACT')
                WRITE(*,'(A)') 'Frealix will run its EXTRACT mode'
                CALL MODE_EXTRACT_INIT_AND_RUN
            CASE ('FIND_FILAMENTS')
                WRITE(*,'(A)') 'Frealix will run its FIND_FILAMENTS mode'
                CALL MODE_FIND_FILAMENTS_INIT_AND_RUN
            CASE ('SPLIT_PAR')
                WRITE(*,'(A)') 'Frealix will run its SPLIT_PAR mode'
                CALL MODE_SPLIT_PAR_INIT_AND_RUN
            CASE ('SWAPXY')
                WRITE(*,'(A)') 'Frealix will run its SWAPXY mode'
                CALL MODE_SWAPXY_INIT_AND_RUN
            CASE ('REBUILD_SEAM')
                WRITE(*,'(A)') 'Frealix will run its REBUILD_SEAM mode'
                CALL MODE_REBUILD_SEAM_INIT_AND_RUN
            CASE ('SYNTH_MICROGRAPH')
                WRITE(*,'(A)') 'Frealix will run its SYNTH_MICROGRAPH mode'
                CALL MODE_SYNTH_MICROGRAPH_INIT_AND_RUN
            CASE ('ROTATE')
                WRITE(*,'(A)') 'Frealix will run its ROTATE mode'
                CALL MODE_ROTATE_INIT_AND_RUN
            CASE ('BFACTOR')
                WRITE(*,'(A)') 'Frealix will run its BFACTOR mode'
                CALL MODE_BFACTOR_INIT_AND_RUN
            !CASE ('UI_TEST')
            !    WRITE(*,'(A)') 'Frealix will run its UI_TEST mode'
            !    CALL MODE_UI_TEST_INIT_AND_RUN
            CASE ('DENSITY_HISTOGRAM')
                WRITE(*,'(A)') 'Frealix will run my CREATE_HISTOGRAM mode'
                CALL MODE_DENSITY_HISTOGRAM_INIT_AND_RUN
            !CASE ('HIST_FIT')
            !    WRITE(*,'(A)') 'Frealix will run my HIST_FIT mode'
            !    CALL MODE_HIST_FIT
            CASE ('AMYLOID_LAYER_LINE')
                WRITE(*,'(A)') 'Frealix will run its AMYLOID_LAYER_LINE mode'
                CALL MODE_AMYLOID_LAYER_LINE_INIT_AND_RUN
            CASE ('UNIT_TESTS')
                WRITE(*,'(A)') 'Frealix will run its UNIT_TESTS mode'
                CALL MODE_UNIT_TESTS_INIT_AND_RUN
            CASE ('RANDOMIZE_WAYPOINTS')
                WRITE(*,'(A)') 'Frealix will run its RANDOMIZE_WAYPOINTS mode'
                CALL MODE_RANDOMIZE_WAYPOINTS_INIT_AND_RUN
            CASE ('FREALIX_TO_FREALIGN')
                WRITE(*,'(A)') 'Frealix will run its FREALIX_TO_FREALIGN mode'
                CALL MODE_FREALIX_TO_FREALIGN_INIT_AND_RUN
            CASE ('APPLY_CTF')
                WRITE(*,'(A)') 'Frealix will run its APPLY_CTF mode'
                CALL MODE_APPLY_CTF_INIT_AND_RUN
            !CASE('FILAMENT_DISPLAY')
            !    WRITE(*,'(A)') 'Frealix will run its FILAMENT_DISPLAY mode'
            !    CALL MODE_FILAMENT_DISPLAY_INIT_AND_RUN
            CASE('NORMALISE','NORMALIZE')
                WRITE(*,'(A)') 'Frealix will run its NORMALISE mode'
                CALL MODE_NORMALISE_INIT_AND_RUN
            CASE('ESTIMATE_PERSISTENCE_LENGTH')
                WRITE(*,'(A)') 'Frealix will run its ESTIMATE_PERSISTENCE_LENGTH mode'
                CALL MODE_ESTIMATE_PL_INIT_AND_RUN
            CASE('MOVIE_ALIGN')
                WRITE(*,'(A)') 'Frealix will run its ESTIMATE_PERSISTENCE_LENGTH mode'
                CALL MODE_MOVIE_ALIGN_INIT_AND_RUN
            CASE DEFAULT
                WRITE(*,'(A,A)') '**ERROR: this Frealix mode is not recognised: ', TRIM(FREALIX_MODE)
                CALL TERMINATE('Fatal error in Frealix')
        END SELECT

        !WRITE(*,'(2A)') 'End date & time = ', NICE_DATE_AND_TIME()

    END SUBROUTINE MAIN

END PROGRAM FREALIX
