!>  \todo   Consider merging this into the IMAGE_COMPARISON module
MODULE FSCS
    USE IMAGES


CONTAINS

SUBROUTINE FSC_UNIT_TESTS()
    IMPLICIT NONE
    CALL FSC_UNIT_TEST_1()
END SUBROUTINE FSC_UNIT_TESTS

!>  \brief  Check that an image has FSC of 1.0 with itself
SUBROUTINE FSC_UNIT_TEST_1()
    USE IMAGES
    USE RUNTIME_PARAMETERS, ONLY : TERMINATE
    IMPLICIT NONE
    ! Private variables
    TYPE(IMAGE) ::  SELF, SELF_COPY
    ! Start work
    WRITE(*,'(A)') '**info(FSC_UNIT_TEST_1): entering'
    SELF%logical_dimensions = [13,13,13]
    CALL IMAGE_ALLOCATE(SELF)
    CALL RANDOM_NUMBER(SELF%real_values)
    SELF_COPY = SELF
    !CALL IMAGE_ALLOCATE(SELF_COPY,MOULD=SELF)
    !SELF_COPY%real_values = SELF%real_values

    !
    !CALL COMPUTE_FSC(SELF,SELF_COPY,FSC_VALUES)
    !ALLOCATE(FSC_VALUES(7))
    !FSC_VALUES = 1.0
    !
    !IF (ANY(FSC_VALUES .LT. 0.999)) THEN
    !    WRITE(*,'(A)') '**ERROR(FSC_UNIT_TEST_1): not all FSC values are 1.0'
    !    WRITE(*,*) FSC_VALUES
    !    CALL TERMINATE('Failed unit test')
    !ELSE
    !    WRITE(*,'(A)') '**info(FSC_UNIT_TEST_1): passed unit test'
    !ENDIF
    WRITE(*,'(A)') '**info(FSC_UNIT_TEST_1): exiting'
END SUBROUTINE FSC_UNIT_TEST_1

!>  \brief  Check that an image has FSC of 1.0 with itself
SUBROUTINE FSC_UNIT_TEST_2()
    USE IMAGES
    USE RUNTIME_PARAMETERS, ONLY : TERMINATE
    IMPLICIT NONE
    ! Private variables
    TYPE(IMAGE) ::  SELF, SELF_COPY
    REAL, ALLOCATABLE :: FSC_VALUES(:)
    ! Start work
    WRITE(*,'(A)') '**info(FSC_UNIT_TEST_2): entering'
    SELF%logical_dimensions = [13,13,13]
    CALL IMAGE_ALLOCATE(SELF)
    CALL RANDOM_NUMBER(SELF%real_values)
    SELF_COPY = SELF
    !
    CALL COMPUTE_FSC(SELF,SELF_COPY,FSC_VALUES)
    !
    IF (ANY(FSC_VALUES .LT. 0.999)) THEN
        WRITE(*,'(A)') '**ERROR(FSC_UNIT_TEST_2): not all FSC values are 1.0'
        WRITE(*,*) FSC_VALUES
        CALL TERMINATE('Failed unit test')
    ELSE
        WRITE(*,'(A)') '**info(FSC_UNIT_TEST_2): passed unit test'
    ENDIF
    WRITE(*,'(A)') '**info(FSC_UNIT_TEST_2): exiting'
END SUBROUTINE FSC_UNIT_TEST_2

!>  \brief  Return the normalised cross-correlation between shells of volumes
SUBROUTINE COMPUTE_FSC(IMG1,IMG2,FSC_OUT)
    USE IMAGES
    USE RUNTIME_PARAMETERS, ONLY  : TERMINATE
    IMPLICIT NONE
    ! Arguments
    TYPE(IMAGE),        INTENT(INOUT)   ::  IMG1    !< Volume to be compared
    TYPE(IMAGE),        INTENT(INOUT)   ::  IMG2    !< Volume to be compared
    REAL, ALLOCATABLE,  INTENT(OUT) ::  FSC_OUT(:)  !< Output FSC. One correlation coefficient per shell.
    ! Private variables
    INTEGER                     ::  I,J,K,IRAD,IERR
    REAL(KIND=8),   ALLOCATABLE ::  FSC_NUM(:)
    REAL(KIND=8),   ALLOCATABLE ::  FSC_DENOM1(:)
    REAL(KIND=8),   ALLOCATABLE ::  FSC_DENOM2(:)
    LOGICAL                     ::  DO_NOT_FOLD(3) = (/.TRUE.,.FALSE.,.FALSE./)
    LOGICAL                     ::  REAL_INPUT(2)
    LOGICAL,    PARAMETER       ::  IIGNORE_FRIEDEL_MATES   =   .TRUE.  ! Some Friedel mates are described explicitely by the FFTW array, we should not count them twice!


    ! Allocate memory
    ALLOCATE(   FSC_NUM   (HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))),   &
                FSC_DENOM1(HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))),   &
                FSC_DENOM2(HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1)))  )

    ! Check type
    REAL_INPUT(1) = IMG1%is_in_real_space
    REAL_INPUT(2) = IMG2%is_in_real_space
    IF (REAL_INPUT(1)) CALL FWD_FT(IMG1)
    IF (REAL_INPUT(2)) CALL FWD_FT(IMG2)

    !
    IF ( IMG1%is_in_real_space .OR. IMG2%is_in_real_space) THEN
        CALL TERMINATE('Fatal error in FSC')
    ENDIF

    ! Check dims
    IF (.NOT. img1%HasSameDimsAs(img2)) THEN
        WRITE(*,'(2(A,3(I0,1X)))') '**ERROR(FSC): images dimensions do not match: ', &
                                    IMG1%logical_dimensions, ' & ', IMG2%logical_dimensions
        CALL TERMINATE('Fatal error in FSC')
    ENDIF

    ! For now, FSC only works for cubes or squares
    IF (.NOT. img1%IsSquare()) THEN
        WRITE(*,'(A,3I0)') '**ERROR(FSC): images are not square / cubic: ', IMG1%logical_dimensions
        CALL TERMINATE('Fatal error in FSC')
    ENDIF

    ! Allocate output array
    IF (ALLOCATED(FSC_OUT)) DEALLOCATE(FSC_OUT)
    ALLOCATE(FSC_OUT(HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))),STAT=IERR)
    IF (IERR .NE. 0) THEN
        WRITE(*,'(A)') '**ERROR(FSC): Allocation of output array failed'
        CALL TERMINATE('Fatal error in FSC')
    ENDIF

    ! Initialise output array
    FSC_OUT = 0.0
    FSC_NUM = 0.0D0
    FSC_DENOM1 = 0.0D0
    FSC_DENOM2 = 0.0D0

    ! Loop over voxels
    DO I=1,HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))
        DO J=1,IMG1%logical_dimensions(2)
            IF (    IIGNORE_FRIEDEL_MATES   .AND. I == 1    &
                                            .AND. J .GT. 1  &
                                            .AND. J .LT. LOWEST_INDEX_FOR_NEGATIVE_FREQUENCIES(IMG1%logical_dimensions(2))) THEN
                CYCLE    ! Skip voxels whose Friedel mates are explicit in the FFTW description
            ENDIF
            DO K=1,IMG1%logical_dimensions(3)
                IF (IIGNORE_FRIEDEL_MATES   .AND. I == 1    &
                                            .AND. J == 1    &
                                            .AND. K .GT. 1  &
                                            .AND. K .LT. LOWEST_INDEX_FOR_NEGATIVE_FREQUENCIES(IMG1%logical_dimensions(3))) THEN
                    CYCLE
                ENDIF
                irad = img1%ComputeShellNumberFromPhysicalAddress(i,j,k,do_not_fold) + 1 ! Shell 0 (the origin voxel) needs index 1 (Fortran arrays are addressed from 1)
                IF (IRAD .LE. HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))) THEN
                    FSC_NUM(IRAD)    = FSC_NUM(IRAD)    + DBLE(IMG1%complex_values(I,J,K) * CONJG(IMG2%complex_values(I,J,K)))
                    FSC_DENOM1(IRAD) = FSC_DENOM1(IRAD) + DBLE(ABS(IMG1%complex_values(I,J,K)))**2
                    FSC_DENOM2(IRAD) = FSC_DENOM2(IRAD) + DBLE(ABS(IMG2%complex_values(I,J,K)))**2
                ENDIF
            ENDDO
        ENDDO
    ENDDO

    IF (ANY(FSC_DENOM1 .LE. 0.0D0) .OR. ANY(FSC_DENOM2 .LE. 0.0D0)) THEN
        WRITE(*,'(A)') '**warning(COMPUTE_FSC): at least one of the shells appears to be empty!'
    ENDIF

    ! Loop over shells
    DO IRAD=1, HIGHEST_INDEX_DIM1(IMG1%logical_dimensions(1))
        IF (FSC_DENOM1(IRAD) .GT. 0.0D0 .AND. FSC_DENOM2(IRAD) .GT. 0.0D0) THEN
            FSC_OUT(IRAD) = REAL( FSC_NUM(IRAD)/SQRT(FSC_DENOM1(IRAD)*FSC_DENOM2(IRAD)) )
        ELSE
            FSC_OUT(IRAD) = 0.0D0
        ENDIF
    ENDDO

    ! Return the images in the same space as they were given
    IF (REAL_INPUT(1)) CALL BWD_FT(IMG1)
    IF (REAL_INPUT(2)) CALL BWD_FT(IMG2)
END SUBROUTINE COMPUTE_FSC

!>  \brief  Compute the p-value, which quantifies the statistical significance with which estimated SSNR > target SSNR
!!
!!  The code largely follows the notations of Bershad & Rockmore (1974)
SUBROUTINE FSC_EVAL_P(FSC,N,REC_DIM,OBJ_DIAM,N_ASYM,SSNR_TARGET,P)
    USE NETLIB_ERF, ONLY : DERF
    IMPLICIT NONE
    ! Arguments
    REAL,       ALLOCATABLE,    INTENT(IN)      ::  FSC(:)      !<  FSC values, 1 per shell
    INTEGER,    ALLOCATABLE,    INTENT(IN)      ::  N(:)        !<  For each shell, the number of values that were used to
                                                                !!  compute the FSC (not counting Friedel mates)
    INTEGER,                    INTENT(IN)      ::  REC_DIM     !<  Dimension of the reconstruction volume (pixels)
    REAL,                       INTENT(IN)      ::  OBJ_DIAM    !<  Average diameter of the object (pixels)
    INTEGER,                    INTENT(IN)      ::  N_ASYM      !<  Number of asymmetric units that contributed to the reconstruction
    REAL,                       INTENT(IN)      ::  SSNR_TARGET !<  Target SSNR
    REAL,       ALLOCATABLE,    INTENT(INOUT)   ::  P(:)        !<  P-value
    ! Private variables
    INTEGER ::  CURRENT_SHELL
    REAL    ::  CURRENT_FSC
    REAL    ::  SSNR_EST_BIASED, SSNR_EST_UNBIASED
    REAL    ::  SSNR_TARGET_UNBIASED_VAR
    REAL    ::  N_IND
    REAL    ::  SIG2_Z_EST, SIG2_Z_TARGET
    REAL    ::  TERM_1, TERM_2, TERM_3, A, B
    REAL    ::  FSC_TARGET
    ! Start work

    ! Allocate p-value array
    IF (ALLOCATED(P)) DEALLOCATE(P)
    ALLOCATE(P(SIZE(FSC)))

    ! Work out the target FSC. E.g. 0.143
    ! According to the equation below, FSC_TARGET=0.143 <=> SSNR_TARGET=0.333,
    ! which seems to contradict Table A1 in Rosenthal & Henderson (2003)
    FSC_TARGET = SSNR_TARGET / (2.0 + SSNR_TARGET)

    ! Loop over the shells
    DO CURRENT_SHELL=1,SIZE(FSC)
        CURRENT_FSC = MIN(FSC(CURRENT_SHELL),0.999)

        ! Equation 7 in B&R
        SSNR_EST_BIASED = 2.0 * CURRENT_FSC / (1-CURRENT_FSC)

        ! Number of independent Fourier voxels in the shell
        N_IND = N(CURRENT_SHELL) / REAL(N_ASYM) * (OBJ_DIAM / REAL(REC_DIM))**2

        ! The variance of the Fisher transform of the FSC estimator at the target SSNR (Eq11 in B&R):
        TERM_1 = 1.0 / (N_IND-1.0)
        TERM_2 = (4.0-FSC_TARGET**2)/(2.0*(N_IND-1.0))
        TERM_3 = (-21.0*FSC_TARGET**2-21.0*FSC_TARGET+176) / (48.0*(N_IND-1.0)**2)

        SIG2_Z_TARGET = TERM_1 * ( 1.0 + TERM_2 + TERM_3 )

        ! The variance of the Fisher transform of the FSC estimator at the estimated SSNR (Eq11 in B&R):
        TERM_2 = (4.0-CURRENT_FSC**2)/(2.0*(N_IND-1.0))
        TERM_3 = (-21.0*CURRENT_FSC**2-21.0*CURRENT_FSC+176) / (48.0*(N_IND-1.0)**2)

        SIG2_Z_EST = TERM_1 * ( 1.0 + TERM_2 + TERM_3 )

        ! The unbiased estimate of SSNR
        A      = EXP(-2.0*SIG2_Z_EST)
        B      = -0.5 * (1.0-EXP(-2.0*SIG2_Z_EST))
        SSNR_EST_UNBIASED = A * SSNR_EST_BIASED + B

        ! The error of the unbiased estimator (Eq 14 in B&R) at the target SSNR
        SSNR_TARGET_UNBIASED_VAR = (EXP(4.0*SIG2_Z_TARGET)-1.0) * (SSNR_TARGET+0.5)**2

        ! We call the Netlib double-precision version of ERF, since not all compiler support the F2008 ERF intrinsic yet
        P(CURRENT_SHELL) = 0.5*(1.0-DERF(   &
                                REAL( (SSNR_EST_UNBIASED-SSNR_TARGET)/SQRT(2.0*SSNR_TARGET_UNBIASED_VAR),KIND=8))    &
                                )

    ENDDO ! End of loop over shells
END SUBROUTINE FSC_EVAL_P

END MODULE FSCS
