!>	\brief	Histrogram fitting
!!
!!	\author Rachel Ruskin
MODULE HISTOGRAM_FIT

CONTAINS

    FUNCTION FIT_FUNCTION(VALUES, PARAMS) BIND(C)
    !Let's make a test placeholder function.
        USE, INTRINSIC :: ISO_C_BINDING
        USE FGSL
        USE RUNTIME_PARAMETERS
        USE HISTOGRAM_FIT_PARAMS
        IMPLICIT NONE
        INTEGER                 ::      STATUS
        REAL(KIND=C_DOUBLE)     ::      FIT_FUNCTION
        TYPE(C_PTR), VALUE      ::      PARAMS
        TYPE(HIST_INFO), POINTER        ::  PARAMETERS
        TYPE(C_PTR), VALUE      ::      VALUES
        TYPE(FGSL_VECTOR)       ::  VAL_VEC_FGSL
        REAL(FGSL_DOUBLE), POINTER  ::  VAL_FGSL(:)
        REAL(KIND=8), ALLOCATABLE   ::  V(:), RHO(:), PDF(:), FIT_FUN_ARRAY(:)
        REAL(KIND=8), POINTER   ::MODEL(:)


        CALL FGSL_OBJ_C_PTR(VAL_VEC_FGSL,VALUES)
        STATUS = FGSL_VECTOR_ALIGN(VAL_FGSL,VAL_VEC_FGSL)
        IF (STATUS .NE. FGSL_SUCCESS) CALL TERMINATE('FGSL_VECTOR_ALIGN failed')
        ALLOCATE(V(SIZE(VAL_FGSL)), STAT=STATUS)
        IF (STATUS .NE. 0) CALL TERMINATE('Memory allocation failed')
        V=VAL_FGSL

        !PARAMETERS is the usable one.
        CALL C_F_POINTER(PARAMS, PARAMETERS)

        !calling this rho to make code more legible.
        ALLOCATE(RHO(SIZE(PARAMETERS%BIN_BOUNDS)), PDF(SIZE(PARAMETERS%HIST)), FIT_FUN_ARRAY(SIZE(PARAMETERS%HIST)))
        RHO=PARAMETERS%BIN_BOUNDS


        MODEL=>MAIN_PIECEWISE(V, RHO, PARAMETERS)

        FIT_FUNCTION=SUM((MODEL-PARAMETERS%HIST)**2)

        !--------------------------------------------------------------------------------------------
        !       This part is the non-squared fitting function. Keep it just in case
        !--------------------------------------------------------------------------------------------

        !Values: V(1)=p_m, V(2)=2sigma^2, V(3)=A, V(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
        !And here's another very iffy fix for a divide by zero problem:
!       IF (V(4) .EQ. V(5)) V(5)=V(4)+0.01
!       IF (V(2) .LT. 0.001)    V(2)=0.001
!
!       Y1=EXP(-((V(4)-V(1)) ** 2)/V(2))
!       Y2=-(V(3)/V(5)) * (ABS(LOG(ABS(V(6)/V(5)))) ** (0.5))
!       S1=-((V(4)-V(1))/(0.5*V(2)))*EXP(-((V(4)-V(1))**2)/V(2))
!       S2=-V(3)/(V(5)**2) * (ABS(LOG(ABS(V(6)/V(5)))) ** (0.5) + (V(6)/(2*V(5))) * ABS(LOG(ABS(V(6)/V(5)))) ** (-0.5))
!       S0=(Y1-Y2)/(V(4)-V(5))
!       A=(S1+S2-2*S0)/((V(4)-V(5))**2)
!       B=(S0-S2)/(V(4)-V(5))-(V(4)+2*V(5))*A
!       C=S2-3*A*V(5)**2-2*B*V(5)
!       D=Y2-A*V(5)**3-B*V(5)**2-C*V(5)
!
!       !Values: V(1)=p_m, V(2)=2sigma^2, V(3)=A, V(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
!       DO I=1, SIZE(PARAMETERS%HIST)
!           ! for r < p1
!           IF (RHO(I) .LE. V(4)) THEN
!               PDF(I)=V(7)*V(7)*EXP(-((RHO(I)-V(1))**2)/(V(2)))
!       !   !for p1<r<p2
!           ELSE IF ((RHO(I) .LE. V(5)) .AND. (RHO(I) .GT. V(4))) THEN
!               PDF(I)=V(7)*(A*RHO(I)**3+B*RHO(I)**2+C*RHO(I)+D)
!       !   !for r>p2
!           ELSE IF (RHO(I) .GT. V(5)) THEN
!
!               PDF(I)=V(7)*(V(3)/RHO(I))*ABS(LOG(ABS(V(6)/RHO(I))))**(0.5)
!           ENDIF
!       ENDDO
!
!       !FIT_FUNCTION=SUM((PDF-PARAMETERS%HIST)**2)
!       !--------------------------------------------------------------------------------
!       !       This part is the squared fitting function. Keep it just in case
!       !-------------------------------------------------------------------------------
!       !Values: V(1)=p_m, V(2)=2sigma^2, V(3)=A, V(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
!
!       DO I=1, SIZE(PARAMETERS%HIST)
!
!           ! for r < p1
!           IF (RHO(I) .LE. V(4)) THEN
!               PDF(I)=V(7)*EXP(-((RHO(I)-V(1))**2)/(V(2)/2))
!           !for p1<r<p2
!           !Put in TEMPORARY absolute value statement.
!           ELSE IF ((RHO(I) .LE. V(5)) .AND. (RHO(I) .GT. V(4))) THEN
!               PDF(I)=(V(7)/(V(4)-V(5)))*((1/(V(2)/2))*EXP(-((V(4)-V(1))**2)/(V(2)/2))*((V(5)-RHO(I))**2) * &
!               (2*V(4)**3-V(5)*(V(2)/2)-2*RHO(I)*(V(5)*V(1)+V(2)/2)-2*(V(4)**2)*(V(5)+V(1)+RHO(I))+ &
!               V(4)*(3*(V(2)/2)+2*V(1)*RHO(I)+2*V(5)*(V(1)+RHO(I))))+((V(3)**2)*((V(4)-RHO(I))**2)* &
!               ((V(4)-V(5))*(V(5)-RHO(I))+(3*V(4)*V(5)-5*V(5)**2-2*V(4)*RHO(I)+4*V(5)*RHO(I))*LOG(ABS(V(6)/V(5)))) &
!               /(V(5)**3)))
!           !for r>p2
!           ELSE IF (RHO(I) .GT. V(5)) THEN
!               PDF(I)=V(7)*((V(3)/RHO(I))**2)*LOG(ABS(V(6)/RHO(I)))
!           ENDIF
!       ENDDO
!       ----------------------------------------------------------------------------------

!       TEMP=MAXLOC(PARAMETERS%HIST)
!       DO I=1, SIZE(FIT_FUN_ARRAY)
!           IF (I .LE. TEMP(1)) THEN
!               FIT_FUN_ARRAY(I)=((SQRT(ABS(PDF(I)))-PARAMETERS%HIST(I))**2)*0.5
!           ELSE
!               FIT_FUN_ARRAY(I)=(SQRT(ABS(PDF(I)))-PARAMETERS%HIST(I))**2
!           ENDIF
!       !ENDDO
!       !FIT_FUNCTION=SUM(FIT_FUN_ARRAY)
!       RIDER=0.0
!       IF (V(1) .LT. 0) RIDER= RIDER+10**8
!       IF (V(4) .LT. 0) RIDER= RIDER+10**8
!       IF (V(5) .LT. V(4)) RIDER= RIDER+10**8
!       IF (V(4) .LT. V(1)) RIDER= RIDER+10**8
!       IF (V(6) .LT. 0) RIDER= RIDER+10**8
!       !IF (V(4) .GT. 0.3) FIT_FUNCTION = FIT_FUNCTION+10**8
!       IF (V(2) .LT. 0.1) RIDER= RIDER+10**8
!
!       FIT_FUNCTION=SUM((PDF-PARAMETERS%HIST)**2)+RIDER
!
!       PRINT *, 'rider', RIDER
!       PRINT *, 'ff', FIT_FUNCTION
        !----------------------------------------------------------------------------------------------
        ! Here is a model fitting function that fits a Gaussian.
        !DO I=1,300
        !   XVALS(I)=EXP((-(-1+(I-1)/150.0)**2)/(2*SIGMA**2))
        !ENDDO

        !DO I=1,300
        !   MODELVALS(I)=EXP((-((-1+(I-1)/150.0)-VAL_VEC(1))**2)/(2*VAL_VEC(2)**2))
        !ENDDO

        !FIT_FUNCTION=SUM((XVALS-MODELVALS)**2)
        !------------------------------------------------------------------------------------------------


    END FUNCTION FIT_FUNCTION


END MODULE HISTOGRAM_FIT
