!>	\brief	Used for histogram fitting
!!
!!	\author	Rachel Ruskin
MODULE HISTOGRAM_FIT_PARAMS
    ! Contains the parameter type that needs to get shuttled back and forth
    ! Also contains the piecewise function
    IMPLICIT NONE

    TYPE HIST_INFO

    REAL, ALLOCATABLE   ::  BIN_BOUNDS(:)
    REAL, ALLOCATABLE   ::  HIST(:)
    REAL, DIMENSION (7, 2)  ::  PTM !(stands for parameters to minimize) This vector has 0 on parameters to be held fixed.
    !For parameters to be varied, contains the index of the VALUES vector that V should correspond to.
    !For parameters to be fixed, contains the value of the parameter
    !For example, (.13 0.034 0 0.04 0.14 0 0// 0 0 1 0 0 2 3) means that V(3)=VALUES(1), and V(1)=0.13

    END TYPE

    CONTAINS

    FUNCTION MAIN_PIECEWISE(VALUES, RHO, PARAMS)

        IMPLICIT NONE

        REAL(KIND=8)                        ::  V(7)
        REAL(KIND=8)                        ::  VALUES(:)
        REAL(KIND=8)                        ::  RHO(:)
        REAL(KIND=8), ALLOCATABLE, TARGET   ::  PDF(:)
        REAL(KIND=8), POINTER               ::  MAIN_PIECEWISE(:)
        TYPE(HIST_INFO), POINTER            ::  PARAMS
        INTEGER                             ::  I

        ALLOCATE(PDF(SIZE(RHO)))

        DO I=1, SIZE(V)
            IF (PARAMS%PTM(I, 2) .GT. 0.5) THEN
                V(I)=VALUES(INT(PARAMS%PTM(I, 2)))
            ELSE
                V(I)=PARAMS%PTM(I, 1)
            ENDIF
        ENDDO

        !V(1)=0.13
        !V(2)=0.034
        !V(3)=VALUES(1)
        !V(4)=0.04
        !V(5)=0.14
        !V(6)=VALUES(2)
        !V(7)=VALUES(3)

!       Y1=EXP(-((V(4)-V(1)) ** 2)/V(2))
!       Y2=-(V(3)/V(5)) * (ABS(LOG(ABS(V(6)/V(5)))) ** (0.5))
!       S1=-((V(4)-V(1))/(0.5*V(2)))*EXP(-((V(4)-V(1))**2)/V(2))
!       S2=-V(3)/(V(5)**2) * ((LOG(ABS(V(6)/V(5)))) ** (0.5) + (V(6)/(2*V(5))) * ABS(LOG(ABS(V(6)/V(5)))) ** (-0.5))
!       S0=(Y1-Y2)/(V(4)-V(5))
!       A=(S1+S2-2*S0)/((V(4)-V(5))**2)
!       B=(S0-S2)/(V(4)-V(5))-(V(4)+2*V(5))*A
!       C=S2-3*A*V(5)**2-2*B*V(5)
!       D=Y2-A*V(5)**3-B*V(5)**2-C*V(5)

        !Values: V(1)=p_m, V(2)=2sigma^2, V(3)=A, V(4)=p1, v(5)=p2, v(6)=p0, v(7)=N
        !---------------------------------------------------------------------------
        !       unsquared version
        !---------------------------------------------------------------------------
!       DO I=1, SIZE(RHO)
!           ! for r < p1
!           IF (RHO(I) .LE. V(4)) THEN
!               PDF(I)=V(7)*EXP(-((RHO(I)-V(1))**2)/(V(2)))
!       !   !for p1<r<p2
!           ELSE IF ((RHO(I) .LT. V(5)) .AND. (RHO(I) .GT. V(4))) THEN
!               PDF(I)=V(7)*(A*RHO(I)**3+B*RHO(I)**2+C*RHO(I)+D)
!       !   !for r>p2
!           ELSE IF (RHO(I) .GE. V(5)) THEN
!
!               PDF(I)=V(7)*(V(3)/RHO(I))*ABS(LOG(ABS(V(6)/RHO(I))))**(0.5)
!           ENDIF
!       ENDDO

!---------------------------------------------------------------------------------
!               Squared Version
!---------------------------------------------------------------------------------

        DO I=1, SIZE(RHO)
            ! for r < p1
            IF (RHO(I) .LE. V(4)) THEN
            !to prevent floating point underflow (23=10 log 10)
                IF (((RHO(I)-V(1))**2)/(ABS(V(2))/2) .GE. 23) THEN
                    PDF(I)=0
                ELSE
                    PDF(I)=(V(7)**2)*EXP(-((RHO(I)-V(1))**2)/(ABS(V(2))/2))
                ENDIF
            !for p1<r<p2
            !Put in TEMPORARY absolute value statement.
            ELSE IF ((RHO(I) .LT. V(5)) .AND. (RHO(I) .GT. V(4))) THEN
                PDF(I)=((V(7)**2)/((V(4)-V(5))**3))*(((1/(V(2)/2))*EXP(-((V(4)-V(1))**2)/(ABS(V(2))/2))*((V(5)-RHO(I))**2) * &
                (2*V(4)**3-V(5)*(V(2)/2)-2*RHO(I)*(V(5)*V(1)+V(2)/2)-2*(V(4)**2)*(V(5)+V(1)+RHO(I))+ &
                V(4)*(3*(V(2)/2)+2*V(1)*RHO(I)+2*V(5)*(V(1)+RHO(I))))+(((V(3)**2)/(V(5)**3))*((V(4)-RHO(I))**2)* &
                ((V(4)-V(5))*(V(5)-RHO(I))+(3*V(4)*V(5)-5*V(5)**2-2*V(4)*RHO(I)+4*V(5)*RHO(I))*LOG(ABS(V(6)/V(5)))))))
            !for r>p2
            ELSE IF (RHO(I) .GE. V(5)) THEN
                PDF(I)=(V(7)**2)*((V(3)/RHO(I))**2)*LOG(ABS(V(6)/RHO(I)))
            ENDIF
        ENDDO

        PDF=SQRT(ABS(PDF))
        MAIN_PIECEWISE=>PDF

    END FUNCTION MAIN_PIECEWISE

END MODULE HISTOGRAM_FIT_PARAMS
