!>  \brief  Mode to align two images
MODULE MODE_ALIGN
    ! Modules
    USE MODES
    USE RTPS


    IMPLICIT NONE
    TYPE(MODE) :: ALIGN

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  REFERENCE_FILENAME  =               &
        RTP_CHAR(   VALUE='reference.mrc',                  &
                    KEYWORD='REFERENCE_FILENAME',           &
                    DESCRIPTION='')

    TYPE(RTP_LOGI), TARGET  ::  HELICAL_VOLUME_ALIGNMENT  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='HELICAL_VOLUME_ALIGNMENT',           &
                    DESCRIPTION='The input and reference have helical symmetry around the Z axis')

CONTAINS

SUBROUTINE MODE_ALIGN_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL ALIGN%INIT(COMMAND='ALIGN',SUBROUTINE=MODE_ALIGN_SLAVE,SHORT_DESCRIPTION='Align an image or volume to a reference')

    ! Add RTPs to the mode
    CALL ALIGN%ADD_RTP(INPUT_FILENAME)
    CALL ALIGN%ADD_RTP(REFERENCE_FILENAME)
    CALL ALIGN%ADD_RTP(HELICAL_VOLUME_ALIGNMENT)

    ! Run the mode
    CALL ALIGN%RUN()
END SUBROUTINE MODE_ALIGN_INIT_AND_RUN

SUBROUTINE MODE_ALIGN_SLAVE
    ! Modules
    USE IMAGES
    USE IMAGEFILES
    USE IMAGES_COMPARISON, ONLY : FIND_SHIFT_EULER
    USE EULERANGLES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGEFILE) ::  INPUT_FILE, REF_FILE, OUTPUT_FILE
    TYPE(IMAGE)     ::  VOLIN,VOLREF
    REAL            ::  SHIFT(3)
    TYPE(EULER)     ::  ROTATION, ROTATION_HALFRANGE, ROTATION_SAMPLING


    ! Start work

    ! Initialise input and output files
    CALL INPUT_FILE%INIT(INPUT_FILENAME%VALUE)
    CALL REF_FILE%INIT(REFERENCE_FILENAME%VALUE)
    CALL OUTPUT_FILE%INIT('aligned.mrc')

    ! Read data from disk
    call volin%ReadFromDisk(input_file)
    call volref%ReadFromDisk(ref_file)

    ! Close input files
    CALL INPUT_FILE%CLOSE()
    CALL REF_FILE%CLOSE()

    IF (HELICAL_VOLUME_ALIGNMENT%VALUE) THEN


    ELSE
        ! Set rotation exhaustive search parameters
        ROTATION_HALFRANGE%PSI = 30.0
        ROTATION_SAMPLING%PSI = 5.0
        PRINT *, '**debug(MODE_ALIGN): setting rotation search half range / sampling = ',  &
                     ROTATION_HALFRANGE%PSI, ROTATION_SAMPLING%PSI


        ! Find alignment parameters
        CALL FIND_SHIFT_EULER(VOLIN,VOLREF,SHIFT,ROTATION,  &
                                ROTATION_SAMPLING=ROTATION_SAMPLING,ROTATION_HALF_RANGE=ROTATION_HALFRANGE)

        PRINT '(A,3(F8.3,1X))', '**info(MODE_ALIGN): shift from input to reference = ', SHIFT
        PRINT '(A,F8.3)', '**info(MODE_ALIGN): rotation from input to reference = ', ROTATION%PSI
    ENDIF

    ! Add step to shift / rotate input -> output


END SUBROUTINE MODE_ALIGN_SLAVE

END MODULE MODE_ALIGN
