!>  \brief  Mode to apply an arbitrary filter to an image. There is only one filter image/volume, but there can be a stack of input images/volumes.
!!  \todo   Rewrite with modern I/O routines.
MODULE MODE_APPLY_FILTER
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: APPLY_FILTER

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILE  =               &
        RTP_CHAR(   VALUE='input',                  &
                    KEYWORD='INPUT_FILE',           &
                    DESCRIPTION='Input image. Real, with origin at center pixel.')


    TYPE(RTP_CHAR), TARGET  ::  MASK_FILE  =               &
        RTP_CHAR(   VALUE='mask',                  &
                    KEYWORD='MASK_FILE',           &
                    DESCRIPTION='Mask/filter image. Real, from 0.0 to 1.0, with origin at center pixel.')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILE  =               &
        RTP_CHAR(   VALUE='output',                  &
                    KEYWORD='OUTPUT_FILE',           &
                    DESCRIPTION='Output image.Real, with origin at center pixel.')

CONTAINS

SUBROUTINE MODE_APPLY_FILTER_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL APPLY_FILTER%INIT(COMMAND='APPLY_FILTER',SUBROUTINE=MODE_APPLY_FILTER_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL APPLY_FILTER%ADD_RTP(INPUT_FILE)
    CALL APPLY_FILTER%ADD_RTP(MASK_FILE)
    CALL APPLY_FILTER%ADD_RTP(OUTPUT_FILE)

    ! Run the mode
    CALL APPLY_FILTER%RUN()
END SUBROUTINE MODE_APPLY_FILTER_INIT_AND_RUN


SUBROUTINE MODE_APPLY_FILTER_SLAVE
    USE RUNTIME_PARAMETERS
    USE PARSERS
    USE IMAGES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE) ::  INPUT_IMAGE,OUTPUT_IMAGE,MASK_IMAGE
    LOGICAL ::  THREED
    INTEGER ::  NUM_IN_STACK, LOC_IN_STACK
    INTEGER ::  FIRST_SLICE, LAST_SLICE

    ! Get runtime parameters


    ! Start work
    CALL READ_FROM_DISK(INPUT_IMAGE,INPUT_FILE%VALUE)
    CALL READ_FROM_DISK(MASK_IMAGE,MASK_FILE%VALUE)

    ! Work out whether we are 2D or 3D case
    IF (MASK_IMAGE%logical_dimensions(3) .EQ. 1) THEN
        THREED = .FALSE.
    ELSE
        THREED = .TRUE.
    ENDIF

    ! Allocate intermediate image accordingly
    CALL IMAGE_ALLOCATE(OUTPUT_IMAGE,MASK_IMAGE)

    ! Work out number of images in stack
    NUM_IN_STACK = INPUT_IMAGE%logical_dimensions(3) / MASK_IMAGE%logical_dimensions(3)
    WRITE(*,'(A,I4,A)') '**debug(MODE_APPLY_FILTER): input stack of ',NUM_IN_STACK,' images'


    DO LOC_IN_STACK = 1, NUM_IN_STACK
        FIRST_SLICE = 1 + (LOC_IN_STACK-1)*MASK_IMAGE%logical_dimensions(3)
        LAST_SLICE  = FIRST_SLICE + MASK_IMAGE%logical_dimensions(3) - 1
        OUTPUT_IMAGE%real_values(:,:,1:MASK_IMAGE%logical_dimensions(3)) = INPUT_IMAGE%real_values(:,:,FIRST_SLICE:LAST_SLICE)
        CALL FWD_FT(OUTPUT_IMAGE)
        CALL APPLY_FILTER_IMG(OUTPUT_IMAGE,MASK_IMAGE)
        CALL BWD_FT(OUTPUT_IMAGE)
        CALL OUTPUT_IMAGE%WriteToDisk(OUTPUT_FILE%VALUE,wanted_location=LOC_IN_STACK)
    ENDDO
END SUBROUTINE MODE_APPLY_FILTER_SLAVE

END MODULE MODE_APPLY_FILTER
