!>  \brief  Mode to bin or unbin images, volumes, frealix2 paramter files or boxer coordinates files.
MODULE MODE_BFACTOR
    ! Modules
    USE MODES
    USE RTPS


    IMPLICIT NONE
    TYPE(MODE) :: BFACTOR

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='output.mrc',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  B_FACTOR  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='BFACTOR',           &
                    DESCRIPTION='B-factor, in A^2, following the crystallographic, aka Henderson, aka Grigorieff convention')


    TYPE(RTP_LOGI), TARGET  ::  LOW_PASS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='LOW_PASS',           &
                    DESCRIPTION='Whether to also apply a cosine-edge low-pass filter.')


    TYPE(RTP_REAL), TARGET  ::  LOW_PASS_RESOLUTION  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='LOW_PASS_RESOLUTION',           &
                    DESCRIPTION='Low-pass filter resolution, in Angstroms. This is the spacing at which the filter reaches 0.')


    TYPE(RTP_REAL), TARGET  ::  LOW_PASS_FALLOFF  =               &
        RTP_REAL(   VALUE=40.0,                  &
                    KEYWORD='LOW_PASS_FALLOFF',           &
                    DESCRIPTION='Low-pass filter falloff width, in Angstroms.')


    TYPE(RTP_LOGI), TARGET  ::  HIGH_PASS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='HIGH_PASS',           &
                    DESCRIPTION='Whether to also apply a cosine-edge high-pass filter')


    TYPE(RTP_REAL), TARGET  ::  HIGH_PASS_RESOLUTION  =               &
        RTP_REAL(   VALUE=250.0,                  &
                    KEYWORD='HIGH_PASS_RESOLUTION',           &
                    DESCRIPTION='High-pass filter resolution, in Angstroms. This is the spacing at which the filter reaches 1.0.')


    TYPE(RTP_REAL), TARGET  ::  HIGH_PASS_FALLOFF  =               &
        RTP_REAL(   VALUE=250.0,                  &
                    KEYWORD='HIGH_PASS_FALLOFF',           &
                    DESCRIPTION='High-pass filter falloff width, in Angstroms.')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='In Angstroms.')

    TYPE(RTP_LOGI), TARGET  ::  TWO_D  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='TWO_D',           &
                    DESCRIPTION='Process 2D slices rather than the 3D volume')

CONTAINS

SUBROUTINE MODE_BFACTOR_INIT_AND_RUN
    ! Intialise the mode
    CALL BFACTOR%INIT(COMMAND='BFACTOR',SUBROUTINE=MODE_BFACTOR_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL BFACTOR%ADD_RTP(INPUT_FILENAME)
    CALL BFACTOR%ADD_RTP(OUTPUT_FILENAME)
    CALL BFACTOR%ADD_RTP(B_FACTOR)
    CALL BFACTOR%ADD_RTP(LOW_PASS)
    CALL BFACTOR%ADD_RTP(LOW_PASS_RESOLUTION)
    CALL BFACTOR%ADD_RTP(LOW_PASS_FALLOFF)
    CALL BFACTOR%ADD_RTP(HIGH_PASS)
    CALL BFACTOR%ADD_RTP(HIGH_PASS_RESOLUTION)
    CALL BFACTOR%ADD_RTP(HIGH_PASS_FALLOFF)
    CALL BFACTOR%ADD_RTP(PIXEL_SIZE)
    CALL BFACTOR%ADD_RTP(TWO_D)

    ! Run the mode
    CALL BFACTOR%RUN()
END SUBROUTINE MODE_BFACTOR_INIT_AND_RUN

SUBROUTINE MODE_BFACTOR_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    USE IMAGEFILES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)     ::  VOL, STACK
    TYPE(IMAGEFILE) ::  INPUT_FILE, OUTPUT_FILE
    INTEGER         ::  I


    ! Start work
    CALL INPUT_FILE%INIT(INPUT_FILENAME%VALUE)


    ! We assume the image is square/cube
    IF (TWO_D%VALUE) THEN
        IF (VOL%logical_dimensions(1) .NE. VOL%logical_dimensions(2)) THEN
            WRITE(*,'(A,2(I0,1X))') '**ERROR(MODE_BFACTOR): input images are not square: ', VOL%logical_dimensions(1:2)
            CALL TERMINATE('Fatal error in MODE_BFACTOR')
        ENDIF
    ELSE
        IF (.NOT. vol%IsSquare()) THEN
            WRITE(*,'(A,3(I0,1X))') '**ERROR(MODE_BFACTOR): input volume is not cubic: ', VOL%logical_dimensions
            CALL TERMINATE('Fatal error in MODE_BFACTOR')
        ENDIF
        call stack%ReadFromDisk(INPUT_FILE,read_volume_not_slice=.true.)
        VOL = STACK
    ENDIF

    CALL OUTPUT_FILE%INIT(OUTPUT_FILENAME%VALUE,MOULD=INPUT_FILE,PIXEL_SIZE=PIXEL_SIZE%VALUE)

    IF (TWO_D%VALUE) THEN
        DO I=1,INPUT_FILE%GetStackSize()
            call VOL%ReadFromDisk(INPUT_FILE,wanted_location=I)
            CALL FWD_FT(VOL)
            CALL VOL%ApplyBFactor(BFACTOR=B_FACTOR%VALUE/PIXEL_SIZE%VALUE**2,   &
                                LP_COSINE=LOW_PASS%VALUE,                  &
                                LP_COSINE_FREQ=PIXEL_SIZE%VALUE/LOW_PASS_RESOLUTION%VALUE,  &
                                LP_COSINE_FALLOFF=PIXEL_SIZE%VALUE/LOW_PASS_FALLOFF%VALUE,  &
                                HP_COSINE=HIGH_PASS%VALUE,                  &
                                HP_COSINE_FREQ=PIXEL_SIZE%VALUE/HIGH_PASS_RESOLUTION%VALUE,  &
                                HP_COSINE_FALLOFF=PIXEL_SIZE%VALUE/HIGH_PASS_FALLOFF%VALUE,  &
                                PRINT_FILTER_SHAPE=.TRUE.)
            CALL BWD_FT(VOL)
            call vol%WriteToDisk(output_file,wanted_location=i)
        ENDDO
    ELSE
        CALL FWD_FT(VOL)
        CALL VOL%ApplyBFactor(BFACTOR=B_FACTOR%VALUE/PIXEL_SIZE%VALUE**2,   &
                            LP_COSINE=LOW_PASS%VALUE,                  &
                            LP_COSINE_FREQ=PIXEL_SIZE%VALUE/LOW_PASS_RESOLUTION%VALUE,  &
                            LP_COSINE_FALLOFF=PIXEL_SIZE%VALUE/LOW_PASS_FALLOFF%VALUE,  &
                            HP_COSINE=HIGH_PASS%VALUE,                  &
                            HP_COSINE_FREQ=PIXEL_SIZE%VALUE/HIGH_PASS_RESOLUTION%VALUE,  &
                            HP_COSINE_FALLOFF=PIXEL_SIZE%VALUE/HIGH_PASS_FALLOFF%VALUE,  &
                            PRINT_FILTER_SHAPE=.TRUE.)
        CALL BWD_FT(VOL)
        call vol%WriteToDisk(OUTPUT_FILE)
    ENDIF

    CALL INPUT_FILE%CLOSE()
    CALL OUTPUT_FILE%PrintInfo()
    CALL OUTPUT_FILE%CLOSE()

END SUBROUTINE MODE_BFACTOR_SLAVE

END MODULE MODE_BFACTOR
