!>  \brief  Frealix mode which calculates the FRSC between two volumes
MODULE MODE_CALC_FRSC
    USE MODES
    USE RTPS

    TYPE(RTP_CHAR), TARGET  ::  INPUT_VOLUME_1  =               &
        RTP_CHAR(   VALUE='vol1',                  &
                    KEYWORD='INPUT_VOLUME_1',           &
                    DESCRIPTION='Input volume 1')
  
  
    TYPE(RTP_CHAR), TARGET  ::  INPUT_VOLUME_2  =               &
        RTP_CHAR(   VALUE='vol2',                  &
                    KEYWORD='INPUT_VOLUME_2',           &
                    DESCRIPTION='Input volume 2')

CONTAINS
SUBROUTINE MODE_CALC_FRSC_INIT_AND_RUN()
    IMPLICIT NONE
    TYPE(MODE) :: CALC_FRSC

    ! Intialise the mode
    CALL CALC_FRSC%INIT(COMMAND='CALC_FRSC',SUBROUTINE=MODE_CALC_FRSC_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL CALC_FRSC%ADD_RTP(INPUT_VOLUME_1)
    CALL CALC_FRSC%ADD_RTP(INPUT_VOLUME_2)

    ! Run the mode
    CALL CALC_FRSC%RUN()
END SUBROUTINE MODE_CALC_FRSC_INIT_AND_RUN

SUBROUTINE MODE_CALC_FRSC_SLAVE()
    USE RUNTIME_PARAMETERS
    USE PARSERS
    USE IMAGES
    USE IMAGEFILES
    USE IMAGES_COMPARISON
    USE FSCS
    USE FRSCS, ONLY : FRSC
    IMPLICIT NONE
    !
    LOGICAL, PARAMETER   ::  PPRINT = .TRUE.
    TYPE(IMAGE) ::  VOL1,VOL2
    REAL, ALLOCATABLE :: FSC_ARRAY(:)

    ! Read volumes from disk
    CALL READ_FROM_DISK(VOL1,INPUT_VOLUME_1%VALUE)
    CALL READ_FROM_DISK(VOL2,INPUT_VOLUME_2%VALUE)

    ! Compute the FRSC
    CALL FRSC(VOL1,VOL2)

    ! As debug, we also do the FSC
    CALL FWD_FT(VOL1)
    CALL FWD_FT(VOL2)
    CALL COMPUTE_FSC(VOL1,VOL2,FSC_ARRAY)
    CALL PRINT_FSC(FSC_ARRAY)

END SUBROUTINE MODE_CALC_FRSC_SLAVE

END MODULE MODE_CALC_FRSC
