!>  \brief  Frealix mode which calculates the FSC between two volumes
MODULE MODE_CALC_FSC
    USE MODES
    USE RTPS
    IMPLICIT NONE

    TYPE(RTP_CHAR), TARGET  ::  INPUT_VOLUME_1  =               &
        RTP_CHAR(   VALUE='vol1',                  &
                    KEYWORD='INPUT_VOLUME_1',           &
                    DESCRIPTION='Input volume 1')


    TYPE(RTP_CHAR), TARGET  ::  INPUT_VOLUME_2  =               &
        RTP_CHAR(   VALUE='vol2',                  &
                    KEYWORD='INPUT_VOLUME_2',           &
                    DESCRIPTION='Input volume 2')


CONTAINS
SUBROUTINE MODE_CALC_FSC_INIT_AND_RUN()
    IMPLICIT NONE
    TYPE(MODE) :: CALC_FSC

    ! Intialise the mode
    CALL CALC_FSC%INIT(COMMAND='CALC_FSC',SUBROUTINE=MODE_CALC_FSC_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL CALC_FSC%ADD_RTP(INPUT_VOLUME_1)
    CALL CALC_FSC%ADD_RTP(INPUT_VOLUME_2)

    ! Run the mode
    CALL CALC_FSC%RUN()


END SUBROUTINE MODE_CALC_FSC_INIT_AND_RUN

SUBROUTINE MODE_CALC_FSC_SLAVE()
    USE IMAGES
    USE IMAGEFILES
    USE IMAGES_COMPARISON
    USE FSCS

    IMPLICIT NONE

    ! Variables
    TYPE(IMAGE) ::  VOL1,VOL2,MASK
    REAL, ALLOCATABLE :: FSC_ARRAY(:)
    LOGICAL ::  PPRINT

    ! We will print out the results
    PPRINT = .TRUE.

    ! Get runtime parameters from control file


    ! Read volumes from disk
    call vol1%ReadFromDisk(input_volume_1%value,read_volume_not_slice=.true.)
    call vol2%ReadFromDisk(input_volume_2%value,read_volume_not_slice=.true.)

    ! Compute the normalised cross-correlation
    MASK%logical_dimensions = VOL1%logical_dimensions
    CALL IMAGE_ALLOCATE(MASK)
    MASK%real_values = 1.0
    WRITE(*,'(A,F0.4)') '**info(MODE_CALC_FSC): normalised cross-correlation coefficient: ',    &
                                                CORRELATION(VOL1,VOL2,MASK,NORMALISE=.TRUE.)

    ! Forward FT
    CALL FWD_FT(VOL1)
    CALL FWD_FT(VOL2)

    ! Compute the FSC
    CALL COMPUTE_FSC(VOL1,VOL2,FSC_ARRAY)

    ! Print the FSC to screen
    IF (PPRINT) THEN
        CALL PRINT_FSC(FSC_ARRAY)
    ENDIF

END SUBROUTINE MODE_CALC_FSC_SLAVE

END MODULE MODE_CALC_FSC
