MODULE MODE_EXTRACT

    !
    ! Modules we'll need
    !
    USE RUNTIME_PARAMETERS
    USE MODES
    USE RTPS

    IMPLICIT NONE
    TYPE(MODE) :: EXTRACT

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='Input image / volume')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='threed.mrc',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='Output image / volume')


    TYPE(RTP_CHAR), TARGET  ::  SELECTION  =               &
        RTP_CHAR(   VALUE='1-2,4',                  &
                    KEYWORD='SELECTION',           &
                    DESCRIPTION='The image locations to be extracted')

CONTAINS

SUBROUTINE MODE_EXTRACT_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL EXTRACT%INIT(COMMAND='EXTRACT',SUBROUTINE=MODE_EXTRACT_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL EXTRACT%ADD_RTP(INPUT_FILENAME)
    CALL EXTRACT%ADD_RTP(OUTPUT_FILENAME)
    CALL EXTRACT%ADD_RTP(SELECTION)

    ! Run the mode
    CALL EXTRACT%RUN()
END SUBROUTINE MODE_EXTRACT_INIT_AND_RUN

SUBROUTINE MODE_EXTRACT_SLAVE
    USE IMAGES
    USE IMAGEFILES
    USE PARSERS
    IMPLICIT NONE
    !
    ! Private variables
    !
    TYPE(IMAGE)             ::  INPUT_IMAGE
    TYPE(IMAGE)             ::  TMP_IMAGE
    TYPE(IMAGEFILE)         ::  OUTPUT_FILE
    INTEGER,    ALLOCATABLE ::  LIST_OF_SLICES(:)
    INTEGER                 ::  I

    !
    ! Get variable values from control file
    !


    !
    ! Start work
    !

    ! Read input image
    CALL READ_FROM_DISK(INPUT_IMAGE,INPUT_FILENAME%VALUE)

    ! Work out which slices we're selecting
    LIST_OF_SLICES = SET_TO_LIST(SELECTION%VALUE)

    ! Temporary 2D image
    TMP_IMAGE%logical_dimensions(1:2) = INPUT_IMAGE%logical_dimensions(1:2)
    TMP_IMAGE%is_in_real_space = INPUT_IMAGE%is_in_real_space
    TMP_IMAGE%logical_dimensions(3) = 1
    CALL IMAGE_ALLOCATE(TMP_IMAGE)

    ! Output file
    CALL OUTPUT_FILE%INIT(OUTPUT_FILENAME%VALUE)

    ! Loop over list of slices
    DO I=1,SIZE(LIST_OF_SLICES)
        TMP_IMAGE%real_values(:,:,1) = INPUT_IMAGE%real_values(:,:,LIST_OF_SLICES(I))
        call tmp_image%WriteToDisk(output_file,wanted_location=i)
    ENDDO

    CALL OUTPUT_FILE%PrintInfo()
    CALL OUTPUT_FILE%CLOSE()



END SUBROUTINE MODE_EXTRACT_SLAVE

END MODULE MODE_EXTRACT
