!>  \brief  Given a set of boxer files, film files and CTFTILT/FIND files, bootstrap frealix analysis
!!          by generating of parameter file. If approximate helical parameters are known, a 3D reconstruction
!!          can also be generated.
!!
!!
!!
MODULE MODE_FILAMENT_STARTUP
!dir$ OPTIMIZE:0
    !
    ! Modules we'll need
    !
    USE RUNTIME_PARAMETERS
    USE MODES
    USE RTPS


    IMPLICIT NONE
    TYPE(MODE) :: FILAMENT_STARTUP

    TYPE(RTP_CHAR), TARGET  ::  INPUT_BOX_FILES  =               &
        RTP_CHAR(   VALUE='input_?????_?????.box',                  &
                    KEYWORD='INPUT_BOX_FILES',           &
                    DESCRIPTION='The first segment of ? is the film identifier, the second one is the filament identifier. '//&
                                'This second identifier must start at 1 and increment without breaks.')

    TYPE(RTP_CHAR), TARGET  ::  INPUT_IMOD_POINTS_FILENAME  =               &
        RTP_CHAR(   VALUE='filaments.txt',                  &
                    KEYWORD='INPUT_IMOD_POINTS_FILENAME',           &
                    DESCRIPTION='Output of IMOD''s model2point command (with -contour option). '//&
                                'Coordinates along selected filaments.')

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_PARAMETER_FILE  =               &
        RTP_CHAR(   VALUE='waypoints.ali',                  &
                    KEYWORD='OUTPUT_PARAMETER_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFTILT_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFTILT_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFFIND_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFFIND_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  FILM_FILES  =               &
        RTP_CHAR(   VALUE='film_?????.mrc',                  &
                    KEYWORD='FILM_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_3D_FILENAME  =               &
        RTP_CHAR(   VALUE='model3d_out.mrc',                  &
                    KEYWORD='OUTPUT_3D_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  FILAMENT_TRACE_FILES  =               &
        RTP_CHAR(   VALUE='trace_?????_?????.box',                  &
                    KEYWORD='FILAMENT_TRACE_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  FILM_SELECTION  =               &
        RTP_CHAR(   VALUE='1-120,122,125-128',                  &
                    KEYWORD='FILM_SELECTION',           &
                    DESCRIPTION='Which films we''re going to analyse - the string here would take all films from '//&
                                '1 to 120, film 122 and films 125 to 128')


    TYPE(RTP_LOGI), TARGET  ::  REVERSE_FILM_CONTRAST  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='REVERSE_FILM_CONTRAST',           &
                    DESCRIPTION='It is assumed that the protein density is darker than the background. If not and you'//&
                                ' want the contrast to be reversed internally, set this to T')


    TYPE(RTP_LOGI), TARGET  ::  CALC_3D_RECONSTRUCTION  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='CALC_3D_RECONSTRUCTION',           &
                    DESCRIPTION='Whether we should attempt a 3D reconstruction (requires known helical parameters)')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=9.99,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  ADD_EXTRA_FIRST_LAST_WPS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='ADD_EXTRA_FIRST_LAST_WPS',           &
                    DESCRIPTION='Whether to add extra way points before & after the first & last waypoints '//  &
                                'described by the boxer files')


    TYPE(RTP_REAL), TARGET  ::  TWIST  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  RISE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  WPS_ARE_CROSSOVERS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WPS_ARE_CROSSOVERS',           &
                    DESCRIPTION='Whether waypoints (WPs) are crossovers (e.g. Amyloid beta fibrils)')


    TYPE(RTP_LOGI), TARGET  ::  FAL_WPS_ARE_CROSSOVERS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='FAL_WPS_ARE_CROSSOVERS',           &
                    DESCRIPTION='Whether the first and last waypoints are also crossovers. Typically, it''s recommended '// &
                                'extra FAL waypoints are picked before and after the first and last waypoint. '//&
                                'Therefore, this would typically false.')


    TYPE(RTP_LOGI), TARGET  ::  FAL_WPS_IN_THREED  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='FAL_WPS_IN_THREED',           &
                    DESCRIPTION='If false, the parts of the filament between the first and the second as well as between'//&
                                ' the penultimate and last waypoints will not be included in the 3D reconstruction')


    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='HELIX_MAX_RADIUS',           &
                    DESCRIPTION='Maximum outer radius of the helix, in Angstroms')


    TYPE(RTP_LOGI), TARGET  ::  MICROTUBULE  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='MICROTUBULE',           &
                    DESCRIPTION='Whether the filaments are microtubules')


    TYPE(RTP_INTG), TARGET  ::  MT_PF_NUMBER  =               &
        RTP_INTG(   VALUE=0,                  &
                    KEYWORD='MT_PF_NUMBER',           &
                    DESCRIPTION='For microtubules only: number of protofilaments (e.g. 13)')


    TYPE(RTP_INTG), TARGET  ::  MT_RISE  =               &
        RTP_INTG(   VALUE=0,                  &
                    KEYWORD='MT_RISE',           &
                    DESCRIPTION='For microtubules only: how much further along the filament (distance measured in '//   &
                                'tubulin subunits) the end of a MT_PF_NUMBER-element "washer" is compared to its '//&
                                'beginning. The 13-pf microtobules have a value of 3.')


    TYPE(RTP_INTG), TARGET  ::  NUM_HELICAL_STARTS  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='NUM_HELICAL_STARTS',           &
                    DESCRIPTION='Number of helical starts. The helical parameters given above must describeone of '//&
                                'these starts. Not including symmetry (e.g. could be describing a 1-start helix with '//&
                                'an additional 2-fold axial symmetry).')


    TYPE(RTP_LOGI), TARGET  ::  WRITE_FILAMENT_TRACES  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WRITE_FILAMENT_TRACES',           &
                    DESCRIPTION='Whether to write coordinates (boxer) files showing waypoint positions and filament traces')

CONTAINS

SUBROUTINE MODE_FILAMENT_STARTUP_INIT_AND_RUN
    ! Intialise the mode
    CALL FILAMENT_STARTUP%INIT(COMMAND='FILAMENT_STARTUP',SUBROUTINE=MODE_FILAMENT_STARTUP_SLAVE,&
                                SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL FILAMENT_STARTUP%ADD_RTP(INPUT_BOX_FILES)
    CALL FILAMENT_STARTUP%ADD_RTP(INPUT_IMOD_POINTS_FILENAME)
    CALL FILAMENT_STARTUP%ADD_RTP(OUTPUT_PARAMETER_FILE)
    CALL FILAMENT_STARTUP%ADD_RTP(CTFTILT_FILES)
    CALL FILAMENT_STARTUP%ADD_RTP(CTFFIND_FILES)
    CALL FILAMENT_STARTUP%ADD_RTP(FILM_FILES)
    CALL FILAMENT_STARTUP%ADD_RTP(OUTPUT_3D_FILENAME)
    CALL FILAMENT_STARTUP%ADD_RTP(FILAMENT_TRACE_FILES)
    CALL FILAMENT_STARTUP%ADD_RTP(FILM_SELECTION)
    CALL FILAMENT_STARTUP%ADD_RTP(REVERSE_FILM_CONTRAST)
    CALL FILAMENT_STARTUP%ADD_RTP(CALC_3D_RECONSTRUCTION)
    CALL FILAMENT_STARTUP%ADD_RTP(PIXEL_SIZE)
    CALL FILAMENT_STARTUP%ADD_RTP(ADD_EXTRA_FIRST_LAST_WPS)
    CALL FILAMENT_STARTUP%ADD_RTP(TWIST)
    CALL FILAMENT_STARTUP%ADD_RTP(RISE)
    CALL FILAMENT_STARTUP%ADD_RTP(SYMMETRY_AXIAL)
    CALL FILAMENT_STARTUP%ADD_RTP(SYMMETRY_PERP)
    CALL FILAMENT_STARTUP%ADD_RTP(WPS_ARE_CROSSOVERS)
    CALL FILAMENT_STARTUP%ADD_RTP(FAL_WPS_ARE_CROSSOVERS)
    CALL FILAMENT_STARTUP%ADD_RTP(FAL_WPS_IN_THREED)
    CALL FILAMENT_STARTUP%ADD_RTP(HELIX_MAX_RADIUS)
    CALL FILAMENT_STARTUP%ADD_RTP(MICROTUBULE)
    CALL FILAMENT_STARTUP%ADD_RTP(MT_PF_NUMBER)
    CALL FILAMENT_STARTUP%ADD_RTP(MT_RISE)
    CALL FILAMENT_STARTUP%ADD_RTP(NUM_HELICAL_STARTS)
    CALL FILAMENT_STARTUP%ADD_RTP(WRITE_FILAMENT_TRACES)

    ! Run the mode
    CALL FILAMENT_STARTUP%RUN()

END SUBROUTINE MODE_FILAMENT_STARTUP_INIT_AND_RUN

SUBROUTINE MODE_FILAMENT_STARTUP_SLAVE
    USE FILAMENTS_MODEL_BASE, ONLY : FILAMENT_PRINT_INFO
    USE COOS
    USE FILMS
    USE PARSERS
    USE FILAMENTS
    USE HELICES
    USE FREALIX_PARAMETERS
    USE THREEDS
    USE FMCS
    USE INTERPOLATION
    use DatesAndTimes
    use ImageFiles

    IMPLICIT NONE
    !
    ! Private variables
    !
    INTEGER,    ALLOCATABLE         ::  LIST_OF_FILMS(:), NUM_OF_FILAMENTS(:)
    INTEGER                         ::  I, J, K, FILM_NUM, FILAMENT_NUM, IERR, CURRENT_POINT
    CHARACTER(LEN=FILENAME_LEN)     ::  FILM_FILENAME
    CHARACTER(LEN=FILENAME_LEN)     ::  CTF_FILENAME
    CHARACTER(LEN=FILENAME_LEN)     ::  BOX_FILENAME_ROOT, BOX_FILENAME
    CHARACTER(LEN=FILENAME_LEN)     ::  FILAMENT_TRACE_FILENAME, TEMP_FILENAME
    LOGICAL                         ::  USE_CTFTILT     ! Whether we'll be using CTFTILT (rather than CTFFIND)
    TYPE(FILM)                      ::  CURRENT_FILM
    TYPE(BOXER_DB)                  ::  CURRENT_BOXER_DB
    !TYPE(POINT)                     ::  POINTS
    TYPE(FILAMENT), ALLOCATABLE     ::  FILS(:)
    TYPE(HELICAL_SYMMETRY)          ::  HELIX
    LOGICAL                         ::  NEW_FILM, NEW_FILAMENT
    TYPE(IMAGE)                     ::  CURRENT_FILM_IMAGE
    TYPE(IMAGE)                     ::  NEW_3DVOL
    TYPE(THREED)                    ::  NEW_3DREC
    REAL                            ::  FILM_HP_PARAM
    TYPE(IMAGEFILE)                 ::  FILM_FILE
    TYPE(IMAGEFILE)                 ::  THREED_FILE
    TYPE(INTERPOLATION_SCHEME)      ::  INTERPOL_SCHEME
    TYPE(IMOD_POINT), ALLOCATABLE   ::  IMOD_POINTS(:)
    LOGICAL, ALLOCATABLE            ::  WHICH_FILM(:)
    TYPE(POINT), ALLOCATABLE        ::  CURRENT_FIL_POINTS(:)
    INTEGER                         ::  FIRST_POINT, LAST_POINT
    LOGICAL                         ::  END_OF_FILAMENT
    LOGICAL,    PARAMETER           ::  DEBUG    =  .FALSE.
    CHARACTER(LEN=6)                ::  NUM_FILMS_C
    type(DateAndTime)               ::  myTime

    !
    ! Get variable values from control file
    !



    !
    ! Start work
    !

    ! Are we given CTFFIND files or CTFTILT files?
    USE_CTFTILT = .FALSE.
    IF (STRING_EQUAL(CTFTILT_FILES%VALUE,'none')) THEN
        IF (STRING_EQUAL(CTFFIND_FILES%VALUE,'none')) THEN
            WRITE(*,'(A)') '**ERROR(MODE_FILAMENT_STARTUP): neither CTFFIND nor CTFTILT files were supplied'
            CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
        ELSE
            USE_CTFTILT = .FALSE.
        ENDIF
    ELSE
        USE_CTFTILT = .TRUE.
    ENDIF

    ! Diagnostic - are using CTFTILT?
    IF (.NOT. USE_CTFTILT) THEN
        PRINT '(A)', '**info(MODE_FILAMENT_STARTUP): No CTFTILT files given, will use CTFFIND info'
    ENDIF

    ! We'll use either EMAN boxes or IMOD points
    IF (INPUT_BOX_FILES%SET_BY_USER .AND. INPUT_IMOD_POINTS_FILENAME%SET_BY_USER) THEN
        WRITE(*,'(A)') '**ERROR(MODE_FILAMENT_STARTUP): you cannot give both an IMOD points file and box files as input.'
        CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
    ENDIF

    ! Check the IMOD points file exists
    IF (INPUT_IMOD_POINTS_FILENAME%SET_BY_USER) THEN
        IF (.NOT. FILE_EXISTS(INPUT_IMOD_POINTS_FILENAME%VALUE)) THEN
            WRITE(*,'(2A)') '**ERROR(MODE_FILAMENT_STARTUP): no such file: ', INPUT_IMOD_POINTS_FILENAME%VALUE
            CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
        ENDIF
    ENDIF


    ! Check
    IF (ADD_EXTRA_FIRST_LAST_WPS%VALUE .AND. .NOT. FAL_WPS_ARE_CROSSOVERS%VALUE) THEN
        WRITE(*,'(2A)') '**ERROR: you asked for extra waypoints to be added AND said that the first and ',   &
                        'last waypoints supplied were not crossovers.'
        CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
    ENDIF

    ! Get list of films we'll be analysing from the user
    LIST_OF_FILMS = SET_TO_LIST(FILM_SELECTION%VALUE)
    WRITE(NUM_FILMS_C,'(I6)') SIZE(LIST_OF_FILMS)
    IF (SIZE(LIST_OF_FILMS) .GT. 100) THEN
        WRITE(*,'(A,I0,A)') '**info(MODE_FILAMENT_STARTUP): ', SIZE(LIST_OF_FILMS), ' films are selected'
    ELSE
        PRINT '(A,I3,A,'//NUM_FILMS_C//'(I0,1X))', '**info(MODE_FILAMENT_STARTUP): The following ', &
                                                    SIZE(LIST_OF_FILMS), ' films are selected: ', LIST_OF_FILMS
    ENDIF

    ! Allocate memory so we can track the number of filaments per film
    ALLOCATE(NUM_OF_FILAMENTS(SIZE(LIST_OF_FILMS)),STAT=IERR)
    IF (IERR .NE. 0) THEN
        WRITE(*,'(A)') '**ERROR(MODE_FILAMENT_STARTUP): memory allocation for count of filaments failed with error ', IERR
        CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
    ENDIF

    ! Initialise the helical symmetry object
    CALL HELICAL_SYMMETRY_INIT( HELIX,RISE%VALUE,TWIST%VALUE,    &
                                SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE,NUM_STARTS=NUM_HELICAL_STARTS%VALUE)

    ! Also, if we specify that the points are crossovers, PHI will be set to 0/180
    IF (WPS_ARE_CROSSOVERS%VALUE) THEN
        WRITE(*,'(A)') '**info(MODE_FILAMENT_STARTUP): supplied coordinates are crossovers & will be assigned phi 0 or 180'
        IF (.NOT. FAL_WPS_ARE_CROSSOVERS%VALUE) THEN
            WRITE(*,'(A)') '**info(MODE_FILAMENT_STARTUP): first and last waypoints will not be treated as crossovers'
        ENDIF
    ENDIF

    ! Check that we have the film, CTF and box files available
    DO I=1,SIZE(LIST_OF_FILMS)
        FILM_NUM = LIST_OF_FILMS(I)

        ! Check the relevant film file exists
        CALL FILENAME_FROM_WILD(FILM_FILES%VALUE,FILM_NUM,FILM_FILENAME)
        IF (.NOT. FILE_EXISTS(FILM_FILENAME)) THEN
            WRITE(*,'(A,A)') '**ERROR(MODE_FILAMENT_STARTUP): file does not exist: ', TRIM(ADJUSTL(FILM_FILENAME))
            CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
        ENDIF

        ! Check the relevant CTFFIND/CTFTILT file exists
        IF (USE_CTFTILT) THEN
            CALL FILENAME_FROM_WILD(CTFTILT_FILES%VALUE,FILM_NUM,CTF_FILENAME)
        ELSE
            CALL FILENAME_FROM_WILD(CTFFIND_FILES%VALUE,FILM_NUM,CTF_FILENAME)
        ENDIF
        IF (.NOT. FILE_EXISTS(CTF_FILENAME)) THEN
            WRITE(*,'(A,A)') '**ERROR(MODE_FILAMENT_STARTUP): file does not exist: ', TRIM(ADJUSTL(CTF_FILENAME))
            CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
        ENDIF

        ! Check the boxer file exists
        IF (INPUT_BOX_FILES%SET_BY_USER) THEN
            ! Find out the root filename for the box files associated with this film
            ! (FILENAME_FROM_WILD) only acts on the first set of question marks, leaves the rest of the filename alone
            CALL FILENAME_FROM_WILD(INPUT_BOX_FILES%VALUE,FILM_NUM,BOX_FILENAME_ROOT)

            ! Check the first box file exists
            CALL FILENAME_FROM_WILD(BOX_FILENAME_ROOT,1,BOX_FILENAME)
            IF (.NOT. FILE_EXISTS(BOX_FILENAME)) THEN
                WRITE(*,'(A,A)') '**warning(MODE_FILAMENT_STARTUP): file does not exist: ', TRIM(BOX_FILENAME)
                WRITE(*,'(A)') 'Note that the .box files must be numbered first with the film identifier, ',    &
                                'then in a sequence starting at 1'
                !CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
            ENDIF
        ENDIF
    ENDDO


    !
    ! Find out how many filaments we'll be dealing with, store that in the NUM_OF_FILAMENTS array (indexed by film)
    !
    NUM_OF_FILAMENTS = 0
    IF (INPUT_BOX_FILES%SET_BY_USER) THEN
        ! We were given .box files as input
        ! The outer loop is over the films, the inner loop over filaments
        DO I=1,SIZE(LIST_OF_FILMS)
            FILM_NUM = LIST_OF_FILMS(I)
            CALL FILENAME_FROM_WILD(INPUT_BOX_FILES%VALUE,FILM_NUM,BOX_FILENAME_ROOT)
            ! Loop over the box files
            J = 0
            DO
                J = J+1
                ! Check box file exists
                CALL FILENAME_FROM_WILD(BOX_FILENAME_ROOT,J,BOX_FILENAME)
                IF (FILE_EXISTS(BOX_FILENAME)) THEN
                    !
                ELSE
                    ! This file didn't exist
                    NUM_OF_FILAMENTS(I) = J-1
                    WRITE(*,'(A,I0,A,I0,A)') '**info(MODE_FILAMENT_STARTUP): film ', FILM_NUM, ' has ', &
                                                NUM_OF_FILAMENTS(I), ' associated filaments'
                    EXIT
                ENDIF
            ENDDO
        ENDDO
    ELSE IF (INPUT_IMOD_POINTS_FILENAME%SET_BY_USER) THEN
        ! We were given IMOD points as input, let's load them all into memory
        CALL IMOD_POINTS_READ_FROM_DISK(IMOD_POINTS,INPUT_IMOD_POINTS_FILENAME%VALUE)
        IF (DEBUG) WRITE(*,'(A)') '**debug(MODE_FILAMENT_STARTUP): below are the imod points:'
        ! Loop over the IMOD points
        DO CURRENT_POINT=1,SIZE(IMOD_POINTS)
            IF (DEBUG) CALL IMOD_POINT_PRINT_INFO(IMOD_POINTS(CURRENT_POINT))
            ! Are we on a new filament?
            NEW_FILAMENT = .FALSE.
            IF (CURRENT_POINT .EQ. 1) THEN
                NEW_FILAMENT = .TRUE.
            ELSE
                IF (IMOD_POINTS(CURRENT_POINT)%CONTOUR_NUMBER .NE. IMOD_POINTS(CURRENT_POINT-1)%CONTOUR_NUMBER) THEN
                    NEW_FILAMENT = .TRUE.
                ENDIF
            ENDIF
            ! If we're on a new filament and it belongs to a film we're interested in, we'll need to keep track
            IF (NEW_FILAMENT) THEN
                WHICH_FILM = IMOD_POINTS(CURRENT_POINT)%SECTION_NUMBER .EQ. LIST_OF_FILMS
                IF (ANY(WHICH_FILM)) THEN
                    DO I=1,SIZE(WHICH_FILM)
                        IF (WHICH_FILM(I)) THEN
                            NUM_OF_FILAMENTS(I) = NUM_OF_FILAMENTS(I)+1
                            EXIT
                        ENDIF
                    ENDDO
                ENDIF
            ENDIF
        ENDDO
    ENDIF

    ! We can now allocate memory for all the filament objects
    !PRINT *, '**info(MODE_FILAMENT_STARTUP): Total number of filaments = ', SUM(NUM_OF_FILAMENTS)
    ALLOCATE(FILS(SUM(NUM_OF_FILAMENTS)),STAT=IERR)
    IF (IERR .NE. 0) THEN
        WRITE(*,'(A)') '**ERROR(MODE_FILAMENT_STARTUP): memory allocation for filaments failed with error ', IERR
        CALL TERMINATE('Fatal error in MODE_FILAMENT_STARTUP')
    ENDIF

    !
    ! Now we can loop again, but we don't have to check files exists, and we know how many filaments we're expecting
    !
    IF (INPUT_IMOD_POINTS_FILENAME%SET_BY_USER) THEN
        FILAMENT_NUM = 0
        FIRST_POINT = 1
        DO CURRENT_POINT=1,SIZE(IMOD_POINTS)
            ! Are we at the end of the current filament?
            END_OF_FILAMENT = .FALSE.
            IF (CURRENT_POINT .EQ. SIZE(IMOD_POINTS)) THEN
                END_OF_FILAMENT = .TRUE.
            ELSE
                IF (IMOD_POINTS(CURRENT_POINT)%CONTOUR_NUMBER .NE. IMOD_POINTS(CURRENT_POINT+1)%CONTOUR_NUMBER) THEN
                    END_OF_FILAMENT = .TRUE.
                ENDIF
            ENDIF
            ! If we're at the end of the current filament and it belongs to a film we're interested in, we'll need to keep track
            IF (END_OF_FILAMENT) THEN
                LAST_POINT = CURRENT_POINT
                WHICH_FILM = IMOD_POINTS(CURRENT_POINT)%SECTION_NUMBER .EQ. LIST_OF_FILMS
                IF (ANY(WHICH_FILM)) THEN
                    ! We're about to initialise a new filament
                    FILAMENT_NUM = FILAMENT_NUM + 1
                    ! Which film is this filament on?
                    DO I=1,SIZE(WHICH_FILM)
                        IF (WHICH_FILM(I)) THEN
                            FILM_NUM = LIST_OF_FILMS(I)
                            EXIT
                        ENDIF
                    ENDDO
                    ! Output for the user
                    WRITE(*,'(4(A,I0),A)') '**info(MODE_FILAMENT_STARTUP): filament ', FILAMENT_NUM, ' of ', SIZE(FILS), &
                                            ' is on micrograph ', FILM_NUM, ' and has ', LAST_POINT-FIRST_POINT+1 , ' waypoints'
                    ! Get the film filename, the CTF information and initialise the film object
                    CALL FILENAME_FROM_WILD(FILM_FILES%VALUE,FILM_NUM,FILM_FILENAME)
                    IF (USE_CTFTILT) THEN
                        CALL FILENAME_FROM_WILD(CTFTILT_FILES%VALUE,FILM_NUM,CTF_FILENAME)
                        CALL FILM_INIT(CURRENT_FILM,FILM_NUM,FILM_FILENAME,PIXEL_SIZE%VALUE,CTFTILT_FILENAME=CTF_FILENAME)
                    ELSE
                        CALL FILENAME_FROM_WILD(CTFFIND_FILES%VALUE,FILM_NUM,CTF_FILENAME)
                        CALL FILM_INIT(CURRENT_FILM,FILM_NUM,FILM_FILENAME,PIXEL_SIZE%VALUE,CTFFIND_FILENAME=CTF_FILENAME)
                    ENDIF
                    ! Get the points for the current filament
                    IF (ALLOCATED(CURRENT_FIL_POINTS)) DEALLOCATE(CURRENT_FIL_POINTS)
                    ALLOCATE(CURRENT_FIL_POINTS(LAST_POINT-FIRST_POINT+1))
                    DO I=1,SIZE(CURRENT_FIL_POINTS)
                        CURRENT_FIL_POINTS(I)%COO(1:2) = IMOD_POINTS(FIRST_POINT+I-1)%COO(1:2)
                    ENDDO
                    ! Initialise a filament - this includes a spline fit - since we're not giving way points (just points),
                    ! this will be a parabolic spline, and the PSI and THETA angles will be worked out from it.
                    CALL FILAMENT_INIT( FILS(FILAMENT_NUM),CURRENT_FIL_POINTS,  &
                                        FILM_IN=CURRENT_FILM,CROSSOVERS=WPS_ARE_CROSSOVERS%VALUE,   &
                                        HELIX=HELIX,ID=FILAMENT_NUM, &
                                        ADD_EXTRA_FAL_WPS=ADD_EXTRA_FIRST_LAST_WPS%VALUE,   &
                                        FAL_WPS_ARE_CROSSOVERS=FAL_WPS_ARE_CROSSOVERS%VALUE, &
                                        MAX_RADIUS=HELIX_MAX_RADIUS%VALUE, &
                                        FAL_WPS_IN_THREED=FAL_WPS_IN_THREED%VALUE, &
                                        MICROTUBULE=MICROTUBULE%VALUE, MT_PF_NUMBER=MT_PF_NUMBER%VALUE,MT_RISE=MT_RISE%VALUE)

                    ! Diagnostic - print filament info
                    CALL FILAMENT_PRINT_INFO(FILS(FILAMENT_NUM))

                    ! Diagnostic - write out waypoint boxer files
                    IF (WRITE_FILAMENT_TRACES%VALUE) THEN
                        CALL FILENAME_FROM_WILD(FILAMENT_TRACE_FILES%VALUE,FILM_NUM,FILAMENT_TRACE_FILENAME)
                        CALL FILENAME_FROM_WILD(FILAMENT_TRACE_FILENAME,FILS(FILAMENT_NUM)%ID,FILAMENT_TRACE_FILENAME)
                        CALL FILENAME_REMOVE_EXTENSION(FILAMENT_TRACE_FILENAME,TEMP_FILENAME)
                        FILAMENT_TRACE_FILENAME = TRIM(ADJUSTL(TEMP_FILENAME))//'_wp.box'
                        CALL FILAMENT_BOXER_DB_TO_DISK(FILS(FILAMENT_NUM),FILAMENT_TRACE_FILENAME,WITH_TRACE=.FALSE.)
                    ENDIF

                ENDIF ! End of: filament was on one of the selected films
                ! Ready for the next filament
                FIRST_POINT = CURRENT_POINT+1
                LAST_POINT  = CURRENT_POINT+1
            ENDIF ! End of: we were on the last point of a filament
        ENDDO
    ELSE IF (INPUT_BOX_FILES%SET_BY_USER) THEN
        FILAMENT_NUM = 0
        DO I=1,SIZE(LIST_OF_FILMS)
            FILM_NUM = LIST_OF_FILMS(I)

            ! Find out the root filename for the box files associated with this film
            ! (FILENAME_FROM_WILD) only acts on the first set of question marks, leaves the rest of the filename alone
            CALL FILENAME_FROM_WILD(INPUT_BOX_FILES%VALUE,FILM_NUM,BOX_FILENAME_ROOT)

            ! The filename for the current film
            CALL FILENAME_FROM_WILD(FILM_FILES%VALUE,FILM_NUM,FILM_FILENAME)

            ! Before we loop over the filaments associated with this film, let's initialise a film object
            IF (USE_CTFTILT) THEN
                CALL FILENAME_FROM_WILD(CTFTILT_FILES%VALUE,FILM_NUM,CTF_FILENAME)
                CALL FILM_INIT(CURRENT_FILM,FILM_NUM,FILM_FILENAME,PIXEL_SIZE%VALUE,CTFTILT_FILENAME=CTF_FILENAME)
            ELSE
                CALL FILENAME_FROM_WILD(CTFFIND_FILES%VALUE,FILM_NUM,CTF_FILENAME)
                CALL FILM_INIT(CURRENT_FILM,FILM_NUM,FILM_FILENAME,PIXEL_SIZE%VALUE,CTFFIND_FILENAME=CTF_FILENAME)
            ENDIF

            !
            ! Loop over the box files
            !
            IF (NUM_OF_FILAMENTS(I) .GT. 0) THEN
            DO J=1, NUM_OF_FILAMENTS(I)

                ! Increment the count of filaments
                FILAMENT_NUM = FILAMENT_NUM + 1

                ! Check box file exists
                CALL FILENAME_FROM_WILD(BOX_FILENAME_ROOT,J,BOX_FILENAME)

                ! Output for the user
                WRITE(*,'(A,I0,A,I0,A,I0)') '**info(MODE_FILAMENT_STARTUP): on filament ', J, ' of ', NUM_OF_FILAMENTS(I), &
                                            ' on micrograph ', FILM_NUM

                ! Read the coordinates in
                CALL BOXER_DB_READ_FROM_DISK(CURRENT_BOXER_DB,BOX_FILENAME)

                ! Get the points for the current filament
                IF (ALLOCATED(CURRENT_FIL_POINTS)) DEALLOCATE(CURRENT_FIL_POINTS)
                ALLOCATE(CURRENT_FIL_POINTS(SIZE(CURRENT_BOXER_DB%POINT)))
                DO K=1,SIZE(CURRENT_FIL_POINTS)
                    CURRENT_FIL_POINTS(K)%COO(1:2) = CURRENT_BOXER_DB%POINT(K)%COO(1:2)
                ENDDO


                ! Initialise a filament - this includes a spline fit - since we're not giving way points (just points),
                ! this will be a parabolic spline, and the PSI and THETA angles will be worked out from it.
                CALL FILAMENT_INIT( FILS(FILAMENT_NUM),CURRENT_FIL_POINTS,FILM_IN=CURRENT_FILM,CROSSOVERS=WPS_ARE_CROSSOVERS%VALUE,   &
                                    HELIX=HELIX,ID=FILAMENT_NUM, &
                                    ADD_EXTRA_FAL_WPS=ADD_EXTRA_FIRST_LAST_WPS%VALUE,   &
                                    FAL_WPS_ARE_CROSSOVERS=FAL_WPS_ARE_CROSSOVERS%VALUE, &
                                    MAX_RADIUS=HELIX_MAX_RADIUS%VALUE, &
                                    FAL_WPS_IN_THREED=FAL_WPS_IN_THREED%VALUE, &
                                    MICROTUBULE=MICROTUBULE%VALUE, MT_PF_NUMBER=MT_PF_NUMBER%VALUE,MT_RISE=MT_RISE%VALUE)


                ! Diagnostic - print filament info
                CALL FILAMENT_PRINT_INFO(FILS(FILAMENT_NUM))

                ! Diagnostic - write out waypoint boxer files
                IF (WRITE_FILAMENT_TRACES%VALUE) THEN
                    CALL FILENAME_FROM_WILD(FILAMENT_TRACE_FILES%VALUE,FILM_NUM,FILAMENT_TRACE_FILENAME)
                    CALL FILENAME_FROM_WILD(FILAMENT_TRACE_FILENAME,FILS(FILAMENT_NUM)%ID,FILAMENT_TRACE_FILENAME)
                    CALL FILENAME_REMOVE_EXTENSION(FILAMENT_TRACE_FILENAME,TEMP_FILENAME)
                    FILAMENT_TRACE_FILENAME = TRIM(ADJUSTL(TEMP_FILENAME))//'_wp.box'
                    CALL FILAMENT_BOXER_DB_TO_DISK(FILS(FILAMENT_NUM),FILAMENT_TRACE_FILENAME,WITH_TRACE=.FALSE.)
                ENDIF

            ENDDO
            ENDIF

        ENDDO
    ENDIF

    ! Diagnostic message
    PRINT '(A,I0,A)', '**info(MODE_FILAMENT_STARTUP): a total of ', FILAMENT_NUM, ' filaments were initialised'


    !
    ! Now that we have an array of initialised filaments, we can write a parameter file to disk
    !
    CALL FILAMENTS_TO_FREALIX_PAR(FILS,OUTPUT_PARAMETER_FILE%VALUE)


    !
    !   Here, start work on calculating a 3D reconstruction
    !
    IF (CALC_3D_RECONSTRUCTION%VALUE) THEN

        !
        ! Initialise the interpolation scheme
        !
        CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'LINEAR','RECT',2)

        ! Initialise a 3D reconstruction - we set its dimensions as a cube of same dimension as the box size given by the user.
        !! TODO: think carefully about how to set the mask parameters. In test images with defocus 15000A, anything larger than 0.7
        !!       was dangerous because it left visible artefacts from the non-recovery of PSF-delocalised information.
        NEW_3DREC%logical_dimensions(1:3) = FMC_THREED_SIZE(INTERPOL_SCHEME,HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE)
        WRITE(*,'(A,3(I4,1X))') '**info(MODE_FILAMENT_STARTUP): setting 3D reconstruction box size to ', &
                                NEW_3DREC%logical_dimensions
        CALL THREED_INIT(   NEW_3DREC,                                          &
                            MASK_RAD            =   NEW_3DREC%logical_dimensions(1)/2.0*0.99,  &
                            MASK_FALLOFF        =   NEW_3DREC%logical_dimensions(1)/2.0*0.1,   &
                            INTERPOL_SCHEME     =   INTERPOL_SCHEME)

        DO I=1,SIZE(FILS)
            !
            ! Message for user
            !
            WRITE(*,'(/3A,I4,A,I4,A,I4,A)') '**info(MODE_FILAMENT_STARTUP): ', myTime%DateAndTimeAsString(), &
                                            ' Reconstructing filament ', FILS(I)%ID, ' (', I, ' of ', SIZE(FILS), ')'
            !
            ! Will we need to read a new film image from disk?
            !
            IF (I .EQ. 1) THEN
                NEW_FILM    =   .TRUE.
            ELSE IF (FILS(I)%FILM%FILENAME .NE. FILS(I-1)%FILM%FILENAME) THEN
                NEW_FILM    =   .TRUE.
            ELSE
                NEW_FILM    =   .FALSE.
            ENDIF

            !
            ! If necessary, read a new film image from disk
            !
            IF (NEW_FILM) THEN
                CALL FILM_FILE%INIT(FILS(I)%FILM%FILENAME)
                CALL CURRENT_FILM_IMAGE%ReadFromDisk(FILM_FILE)
                CALL FILM_FILE%CLOSE()
                ! Flip contrast if necessary
                IF (REVERSE_FILM_CONTRAST%VALUE) THEN
                    CURRENT_FILM_IMAGE%real_values = CURRENT_FILM_IMAGE%real_values * (-1.0)
                ENDIF
                ! High-pass filter the film image to bring average density to 0.0 and remove density ramps
                FILM_HP_PARAM =   PIXEL_SIZE%VALUE / (3.0 * HELIX_MAX_RADIUS%VALUE)
                CALL BAND_PASS(CURRENT_FILM_IMAGE,FILM_HP_PARAM,0.0,FILM_HP_PARAM/2.0)
            ENDIF

            CALL FILAMENT_CALC_THREED(FILS(I),CURRENT_FILM_IMAGE,NEW_3DREC,FINALISE=.FALSE.)
        ENDDO
        CALL THREED_EVAL_RECONSTRUCTION(NEW_3DREC,NEW_3DVOL)
        ! Write to disk
        CALL THREED_WRITE_TO_DISK(NEW_3DREC,OUTPUT_3D_FILENAME%VALUE)
        CALL THREED_FILE%INIT(  OUTPUT_3D_FILENAME%VALUE,    &
                                DIM_X=NEW_3DVOL%logical_dimensions(1),  &
                                DIM_Y=NEW_3DVOL%logical_dimensions(2),  &
                                DIM_Z=NEW_3DVOL%logical_dimensions(3))
        CALL NEW_3DVOL%WriteToDisk(THREED_FILE)
        CALL THREED_FILE%CLOSE()
        !CALL WRITE_TO_DISK(NEW_3DVOL,OUTPUT_3D_FILENAME%VALUE)
    ENDIF

    WRITE(*,'(/2A/)') '**info(MODE_FILAMENT_STARTUP): all done. Output parameter file: ', TRIM(ADJUSTL(OUTPUT_PARAMETER_FILE%VALUE))


END SUBROUTINE MODE_FILAMENT_STARTUP_SLAVE

END MODULE MODE_FILAMENT_STARTUP

