MODULE MODE_FREALIGN_STACK_TO_THREED
    ! Modules
    USE MODES
    USE RTPS

    IMPLICIT NONE
    TYPE(MODE) :: FREALIGN_STACK_TO_THREED

    TYPE(RTP_CHAR), TARGET  ::  FREALIGN_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='input.par',                  &
                    KEYWORD='FREALIGN_PAR_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  STACK_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='STACK_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  THREED_FILENAME  =               &
        RTP_CHAR(   VALUE='output.mrc',                  &
                    KEYWORD='THREED_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  MODEL_THREED_FILENAME  =               &
        RTP_CHAR(   VALUE='model.mrc',                  &
                    KEYWORD='MODEL_THREED_FILENAME',           &
                    DESCRIPTION='Model 3D needed for FOM filtering')

    TYPE(RTP_CHAR), TARGET  ::  SYMMETRY  =               &
        RTP_CHAR(   VALUE='C1',                  &
                    KEYWORD='SYMMETRY',           &
                    DESCRIPTION='Point group symmetry using Sch�nflies notation')


    TYPE(RTP_REAL), TARGET  ::  TWIST_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='In degrees')


    TYPE(RTP_REAL), TARGET  ::  RISE_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='In Angstroems')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='Rotational symmetry redundancy about the helical axis')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='Rotational symmetry redundancy perpendicular to the helical axis (e.g. 2 for DNA)')


    TYPE(RTP_REAL), TARGET  ::  ASYM_UNIT_MW  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ASYM_UNIT_MW',           &
                    DESCRIPTION='Expected molecular weight (Da) of the asymetric unit')


    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='HELIX_MAX_RADIUS',           &
                    DESCRIPTION='Maximum outer radius of the helix, in Angstroms')


    TYPE(RTP_INTG), TARGET  ::  NUM_SUBUNITS_PER_IMAGE  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='NUM_SUBUNITS_PER_IMAGE',           &
                    DESCRIPTION='For each image in the stack, how many subunits should be added to the 3D reconstruction')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='Pixel size (Angstroms)')


    TYPE(RTP_REAL), TARGET  ::  ACCEL_VOLTAGE  =               &
        RTP_REAL(   VALUE=200.0,                  &
                    KEYWORD='ACCEL_VOLTAGE',           &
                    DESCRIPTION='Acceleration voltage (kV)')


    TYPE(RTP_REAL), TARGET  ::  AMPLITUDE_CONTRAST  =               &
        RTP_REAL(   VALUE=0.07,                  &
                    KEYWORD='AMPLITUDE_CONTRAST',           &
                    DESCRIPTION='Amplitude contast (fraction)')


    TYPE(RTP_REAL), TARGET  ::  CS  =               &
        RTP_REAL(   VALUE=2.3,                  &
                    KEYWORD='CS',           &
                    DESCRIPTION='Spherical aberation (mm)')


    TYPE(RTP_LOGI), TARGET  ::  APPLY_FOM_FILTER  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='APPLY_FOM_FILTER',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  APPLY_HELICAL_SYMMETRY  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='APPLY_HELICAL_SYMMETRY',           &
                    DESCRIPTION='')

    TYPE(RTP_LOGI), TARGET  ::  WRITE_MATCHING_PROJECTIONS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WRITE_MATCHING_PROJECTIONS',           &
                    DESCRIPTION='Output a stack of matching projections generated from the input model 3D')

CONTAINS

SUBROUTINE MODE_FREALIGN_STACK_TO_THREED_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL FREALIGN_STACK_TO_THREED%INIT(COMMAND='FREALIGN_STACK_TO_THREED',SUBROUTINE=MODE_FREALIGN_STACK_TO_THREED_SLAVE,   &
                                        SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(FREALIGN_PAR_FILENAME)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(STACK_FILENAME)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(THREED_FILENAME)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(MODEL_THREED_FILENAME)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(SYMMETRY)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(TWIST_PER_SUBUNIT)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(RISE_PER_SUBUNIT)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(SYMMETRY_AXIAL)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(SYMMETRY_PERP)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(ASYM_UNIT_MW)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(HELIX_MAX_RADIUS)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(NUM_SUBUNITS_PER_IMAGE)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(PIXEL_SIZE)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(ACCEL_VOLTAGE)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(AMPLITUDE_CONTRAST)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(CS)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(APPLY_FOM_FILTER)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(APPLY_HELICAL_SYMMETRY)
    CALL FREALIGN_STACK_TO_THREED%ADD_RTP(WRITE_MATCHING_PROJECTIONS)

    ! Run the mode
    CALL FREALIGN_STACK_TO_THREED%RUN()


END SUBROUTINE MODE_FREALIGN_STACK_TO_THREED_INIT_AND_RUN

SUBROUTINE MODE_FREALIGN_STACK_TO_THREED_SLAVE
    USE IMAGES
    USE SINGLE_PARTICLES
    USE POINT_SYMMETRIES
    USE FREALIGN_PARAMETERS
    USE THREEDS
    USE UI, ONLY : INTERVAL_PRINT
    USE HELICES
    USE ROTATION_MATRICES
    USE IMAGES_HELICES
    USE PARSERS, ONLY: PATH_FROM_FILENAME, FILENAME_FROM_WILD
    USE IMAGES_THREED, ONLY : EXTRACT_SLICE
    USE IMAGES_CTF, ONLY : IMAGE_APPLY_CTF
    USE INTERPOLATION, ONLY : INTERPOLATION_SCHEME_INIT
    use UsefulFunctions, only : IsOdd
    use ImageFiles
    use RUNTIME_PARAMETERS, only : FILENAME_LEN
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)                             ::  VOL, SLICE, PROJ, MODEL_3D
    TYPE(THREED)                            ::  REC3D_CURRENT, REC3D
    TYPE(IMAGEFILE)                         ::  STACK,THREED_FILE, MATCHING_PROJS, MATCHING_BOXES, MODEL_3D_FILE
    !TYPE(IMAGEFILE)                         ::  SLICES_XY_FILE, SLICES_YZ_FILE
    TYPE(SINGLE_PARTICLE),  ALLOCATABLE     ::  SP(:)
    TYPE(POINT_SYMMETRY_SYMBOL)             ::  SYM_SYMBOL
    TYPE(HELICAL_SYMMETRY)                  ::  HELIX
    TYPE(WINDOW)                            ::  EXTRACT_INTERPOL_WINDOW
    TYPE(INTERPOLATION_SCHEME)              ::  INTERPOL
    !
    INTEGER                                 ::  I,STACK_DIM(3)
    INTEGER                                 ::  CURRENT_SUBUNIT, SUBUNIT_START, SUBUNIT_FINISH
    REAL                                    ::  MAT(4,4)
    REAL                                    ::  EXTRA_SHIFT_XY(2)
    TYPE(EULER)                             ::  TMP_ANGLES
    REAL                                    ::  H_X,H_Y,SHIFT_ALONG_H_AXIS
    LOGICAL,        PARAMETER               ::  DEBUG   =   .FALSE.
    REAL(KIND=8)                            ::  AVERAGE_SCORE
    REAL                                    ::  BFACTOR
    LOGICAL                                 ::  NEW_FILAMENT, LAST_IN_FILAMENT
    TYPE(IMAGE)                             ::  REC3D_CURRENT_VOL, REC3D_CURRENT_VOL_SLICE
    CHARACTER(LEN=FILENAME_LEN)             ::  TEMP_FILENAME
    INTEGER                                 ::  FIRST_LOC,LAST_LOC, CURRENT_OUTPUT_LOC


    ! Start work

    ! If we need to insert each image several times (because there are >1 helical subunits per image),
    ! let's work out some details now
    IF (IsOdd(NUM_SUBUNITS_PER_IMAGE%VALUE)) THEN
        SUBUNIT_FINISH = (NUM_SUBUNITS_PER_IMAGE%VALUE-1)/2
        SUBUNIT_START  = - SUBUNIT_FINISH
    ELSE
        SUBUNIT_FINISH = NUM_SUBUNITS_PER_IMAGE%VALUE/2
        SUBUNIT_START  = - SUBUNIT_FINISH + 1
    ENDIF

    ! Override with ugly frealign way
    SUBUNIT_START = - NINT(NUM_SUBUNITS_PER_IMAGE%VALUE/2.0)
    SUBUNIT_FINISH =  NINT(NUM_SUBUNITS_PER_IMAGE%VALUE/2.0)
    IF (SUBUNIT_FINISH-SUBUNIT_START+1 .GT. NUM_SUBUNITS_PER_IMAGE%VALUE) SUBUNIT_FINISH = SUBUNIT_FINISH - 1
    IF (SUBUNIT_FINISH-SUBUNIT_START+1 .GT. NUM_SUBUNITS_PER_IMAGE%VALUE) SUBUNIT_START = SUBUNIT_START + 1


    IF (DEBUG) WRITE(*,'(A,2(I0,1X))') '**debug(MODE_FREALIGN_STACK_TO_THREED): subunit start, finish = ', &
                                        SUBUNIT_START, SUBUNIT_FINISH

    !
    ! Initialise the average helical parameters
    !
    IF (APPLY_HELICAL_SYMMETRY%VALUE) THEN
        CALL HELICAL_SYMMETRY_INIT(HELIX,RISE_PER_SUBUNIT%VALUE,TWIST_PER_SUBUNIT%VALUE,    &
                                    SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE)
    ENDIF

    ! Initialise interpolation
    INTERPOL = DEFAULT_INTERPOL_SCHEME()
    CALL ALLOCATE_WINDOW(EXTRACT_INTERPOL_WINDOW,INTERPOL%WINDOW_WIDTH,COMP=.TRUE.,THREED=.TRUE.)


    ! Grab values from input Frealign parameter file
    SP = FREALIGN_PAR_TO_SP(FREALIGN_PAR_FILENAME%VALUE)
    IF (SIZE(SP) .LT. 1) THEN
        WRITE(*,'(A,I0)') '**info(MODE_FREALIGN_STACK_TO_THREED): number of single particles = ', SIZE(SP)
        CALL TERMINATE('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    ENDIF
    ! Fill in the other values
    SP%CTF%CS = CS%VALUE
    SP%CTF%AMPL_CONT = AMPLITUDE_CONTRAST%VALUE
    SP%CTF%WL = AKV_TO_WL(ACCEL_VOLTAGE%VALUE)
    ! Convert CTF parameters to pixel units
    DO I=1,SIZE(SP)
        CALL ANG_TO_PIX(SP(I)%CTF,PIXEL_SIZE%VALUE)
    END DO
    ! Convert scores from phase residuals to cross-correlation scores, and compute average score
    AVERAGE_SCORE = 0.0D0
    DO I=1,SIZE(SP)
        SP(I)%SCORE = COS(RADIANS(SP(I)%SCORE))
        AVERAGE_SCORE = AVERAGE_SCORE + SP(I)%SCORE
    ENDDO
    AVERAGE_SCORE = AVERAGE_SCORE / REAL(SIZE(SP),KIND=8)
    WRITE(*,'(A,F0.4)') '**info(MODE_FREALIGN_STACK_TO_THREED): average score = ', AVERAGE_SCORE

    !
    ! In the helical case, if we have a large enough shift along the helical axis,
    ! we may be able to rotate the single particle by an integer number of
    ! helical repeat operators and so reduce the shift (with a concomittent change
    ! in phi)
    !
    IF (.FALSE.) THEN   !!!!!! TESTING TESTING TESTING !!!!
    IF (HELIX%RISE .NE. 0.0) THEN
        DO I=1,SIZE(SP)
            ! Compute the shift componenent along the helical axis
            H_X = -COS(RADIANS(SP(I)%ANGLES%PSI))
            H_Y =  SIN(RADIANS(SP(I)%ANGLES%PSI))
            SHIFT_ALONG_H_AXIS =    SP(I)%SHIFT_X *  H_X + &
                                    SP(I)%SHIFT_Y *  H_Y
            SHIFT_ALONG_H_AXIS = SHIFT_ALONG_H_AXIS / SIN(RADIANS(SP(I)%ANGLES%THETA))
            ! In number of h repeats
            IF (DEBUG) WRITE(*,'(A,F0.3)') '**debug(MODE_FREALIGN_STACK_TO_THREED): shift along axis (in number of h repeats) = ',&
                                             SHIFT_ALONG_H_AXIS/ HELIX%RISE
            SHIFT_ALONG_H_AXIS =    NINT(SHIFT_ALONG_H_AXIS / HELIX%RISE)
            !

            ! Modify the shifts & phi angles accordingly
            SP(I)%SHIFT_X    = SP(I)%SHIFT_X    - SHIFT_ALONG_H_AXIS * H_X * HELIX%RISE * SIN(RADIANS(SP(I)%ANGLES%THETA))
            SP(I)%SHIFT_Y    = SP(I)%SHIFT_Y    - SHIFT_ALONG_H_AXIS * H_Y * HELIX%RISE * SIN(RADIANS(SP(I)%ANGLES%THETA))
            SP(I)%ANGLES%PHI = SP(I)%ANGLES%PHI + SHIFT_ALONG_H_AXIS * HELIX%TWIST
            SP(I)%ANGLES%PHI = DEGREES_M180_TO_180(SP(I)%ANGLES%PHI)
        END DO
    ENDIF
    ENDIF
    !!!!!! TESTING

    ! Calculate symmetry matrices
    SYM_SYMBOL%SYMBOL = TRIM(SYMMETRY%VALUE)

    ! Initialize IMAGEFILE object to open stack
    CALL STACK%INIT(STACK_FILENAME%VALUE)
    CALL STACK%PrintInfo()
    STACK_DIM = STACK%GetDimensions()

    ! Initialise output stacks for matching projections
    IF (WRITE_MATCHING_PROJECTIONS%VALUE) THEN
        CALL MATCHING_PROJS%INIT('dbg_match_proj.mrc',DIM_X=STACK_DIM(1),DIM_Y=STACK_DIM(2),PIXEL_SIZE=PIXEL_SIZE%VALUE)
        CALL MODEL_3D_FILE%INIT(MODEL_THREED_FILENAME%VALUE)
        CALL MODEL_3D_FILE%PrintInfo()
        CALL MODEL_3D%ReadFromDisk(MODEL_3D_FILE,read_volume_not_slice=.true.)
        CALL MODEL_3D_FILE%CLOSE()
        CALL IMAGE_PRINT_INFO(MODEL_3D)
        CALL MATCHING_BOXES%INIT('dbg_match_boxes.mrc',DIM_X=STACK_DIM(1),DIM_Y=STACK_DIM(2),  &
                                PIXEL_SIZE=PIXEL_SIZE%VALUE)
    ENDIF

    ! Check whether we have expected number of images
    IF (STACK_DIM(3) .NE. SIZE(SP)) THEN
        WRITE(*,'(A,I5)') '**WARNING(MODE_FREALIGN_STACK_TO_THREED): number of particles described by parameter file = ', SIZE(SP)
        WRITE(*,'(A,I5)') '                                          number of images in input stack = ', STACK_DIM(3)
        !CALL TERMINATE('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    ENDIF

    ! Check images in the stack are square
    IF (STACK_DIM(1) .NE. STACK_DIM(2)) THEN
        WRITE(*,'(A,2I4)') '**ERROR(MODE_FREALIGN_STACK_TO_THREED): input stack images are not square. ', STACK_DIM(1), STACK_DIM(2)
        CALL TERMINATE('Fatal error in MODE_FREALIGN_STACK_TO_THREED')
    ENDIF

    ! We're ignoring shift_x and shift_y
    ! WRITE(*,'(A)') '**INFO(MODE_FREALIGN_STACK_TO_THREED): X and Y shifts from frealign parameter file are ignored'

    ! Initialise & memory for the THREED object
    REC3D%logical_dimensions(1:3) = STACK_DIM(1)
    REC3D_CURRENT%logical_dimensions(1:3) = STACK_DIM(1)
    CALL INIT(  REC3D,REC3D%logical_dimensions(1)/2.0*0.9,REC3D%logical_dimensions(1)/2.0*0.1, &
                'LINEAR','RECT',2,SYM_SYMBOL=SYM_SYMBOL, &
                HELIX=HELIX,HELIX_MAX_RAD=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE,  &
                ASYM_UNIT_MW=ASYM_UNIT_MW%VALUE,PIXEL_SIZE=PIXEL_SIZE%VALUE,        &
                FOM_FILTER=APPLY_FOM_FILTER%VALUE)
    CALL INIT(  REC3D_CURRENT,REC3D_CURRENT%logical_dimensions(1)/2.0*0.9,REC3D_CURRENT%logical_dimensions(1)/2.0*0.1,&
                'LINEAR','RECT',2,SYM_SYMBOL=SYM_SYMBOL, &
                HELIX=HELIX,HELIX_MAX_RAD=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE,  &
                ASYM_UNIT_MW=ASYM_UNIT_MW%VALUE,PIXEL_SIZE=PIXEL_SIZE%VALUE,        &
                FOM_FILTER=APPLY_FOM_FILTER%VALUE)

    ! Prepare the slice image
    ! SLICE%logical_dimensions(1:2) = STACK_DIM(1)
    ! SLICE%logical_dimensions(3) = 1
    ! SLICE%TYPE = 'REAL'
    ! SLICE%ORIGIN_AT_CENTER = .TRUE.
    ! CALL ALLOCATE(SLICE)


    ! Insert slices
    FIRST_LOC = 1    !1
    LAST_LOC = 133    !SIZE(SP)
    FIRST_LOC = 1
    LAST_LOC = SIZE(SP)
    CURRENT_OUTPUT_LOC = 0
    DO I=FIRST_LOC,LAST_LOC  !1,SIZE(SP)
        IF (INTERVAL_PRINT(I,SIZE(SP)) .OR. .TRUE.) THEN
            WRITE(*,'(2(A,I6),A,3(F7.2,1X),A,2(F7.2,1X))') '**info(MODE_FREALIGN_STACK_TO_THREED): inserting stack image # ', &
                                                            I, ' of ', SIZE(SP), ' with angles ', SP(I)%ANGLES, &
                                                            ' and shifts ', SP(I)%SHIFT_X, SP(I)%SHIFT_Y
        ENDIF
        call slice%ReadFromDisk(stack,wanted_location=i)
        ! Are we on a new filament?
        NEW_FILAMENT = .TRUE.
        IF (I .GT. 1) NEW_FILAMENT = SP(I)%FILM_NUM .NE. SP(I-1)%FILM_NUM
        IF (I .EQ. FIRST_LOC) NEW_FILAMENT = .TRUE.
        ! Is this the last image from the current filament
        LAST_IN_FILAMENT = .TRUE.
        IF (I .LT. SIZE(SP)) LAST_IN_FILAMENT = SP(I)%FILM_NUM .NE. SP(I+1)%FILM_NUM
        IF (I .EQ. LAST_LOC) LAST_IN_FILAMENT = .TRUE.
        ! If we're starting a new filament, wipe the current 3d
        IF (NEW_FILAMENT) CALL THREED_WIPE(REC3D_CURRENT)
        ! Insert all subunits found in the image
        DO CURRENT_SUBUNIT=SUBUNIT_START,SUBUNIT_FINISH
            !write(*,'(a,i0)') '**debug(MODE_FREALIGN_STACK_TO_THREED): current_subunit = ', current_subunit
            ! Work out the additional shift in XY and change in PHI
            MAT = EULER_TO_ROTMAT(SP(I)%ANGLES%PHI,SP(I)%ANGLES%THETA,SP(I)%ANGLES%PSI,REVERSE=.FALSE.)
            EXTRA_SHIFT_XY(1:2) = MAT(1:2,3) * REAL(CURRENT_SUBUNIT)*HELIX%RISE
            TMP_ANGLES = SP(I)%ANGLES
            TMP_ANGLES%PHI = TMP_ANGLES%PHI + REAL(CURRENT_SUBUNIT)*HELIX%TWIST
            !write(*,'(a,3(f0.5,1x))') '**debug(MODE_FREALIGN_STACK_TO_THREED): dphi, dx, dy = ', &
            !                            REAL(CURRENT_SUBUNIT)*HELIX%TWIST, EXTRA_SHIFT_XY
            ! Insert image into 3D
            BFACTOR  = BFACTOR_FROM_SCORE(SP(I)%SCORE,REAL(AVERAGE_SCORE))
            !WRITE(*,'(A,F0.4)') '**debug(MODE_FREALIGN_STACK_TO_THREED): bfactor = ', BFACTOR
            CALL THREED_ADD_SLICE(  REC3D_CURRENT,SLICE,SP(I)%CTF,TMP_ANGLES,   &
                                    SHIFT_X = SP(I)%SHIFT_X - EXTRA_SHIFT_XY(1),  &
                                    SHIFT_Y = SP(I)%SHIFT_Y - EXTRA_SHIFT_XY(2),   &
                                    BFACTOR=BFACTOR)
            IF (CURRENT_SUBUNIT .EQ. 0) THEN
                ! Write out matching projections
                IF (WRITE_MATCHING_PROJECTIONS%VALUE) THEN
                    CURRENT_OUTPUT_LOC = CURRENT_OUTPUT_LOC + 1
                    ! Compute the projection
                    IF (IS_REAL(MODEL_3D)) CALL FWD_FT(MODEL_3D)
                    IF (MODEL_3D%ORIGIN_AT_CENTER) CALL SHIFT_REAL_SPACE_ORIGIN(MODEL_3D)
                    CALL EXTRACT_SLICE( MODEL_3D,PROJ,   &
                                        PHI=TMP_ANGLES%PHI,THETA=TMP_ANGLES%THETA,PSI=TMP_ANGLES%PSI,   &
                                        INTERPOL_SCHEME=INTERPOL, &
                                        WIN=EXTRACT_INTERPOL_WINDOW, &
                                        X_SHIFT=-(SP(I)%SHIFT_X - EXTRA_SHIFT_XY(1)),  &
                                        Y_SHIFT=-(SP(I)%SHIFT_Y - EXTRA_SHIFT_XY(2)) )
                    CALL SHIFT_REAL_SPACE_ORIGIN(PROJ)
                    CALL IMAGE_APPLY_CTF(PROJ,SP(I)%CTF)
                    CALL BWD_FT(PROJ)
                    ! Write out the projection
                    call proj%WriteToDisk(MATCHING_PROJS,wanted_location=current_output_loc)
                    ! Write out the slice
                    IF (IS_REAL(SLICE)) CALL FWD_FT(SLICE)
                    CALL APPLY_BFACTOR(SLICE,   BFACTOR=200.0/(PIXEL_SIZE%VALUE**2),   &
                                                LP_COSINE=.FALSE.,LP_COSINE_FREQ=PIXEL_SIZE%VALUE/12.0, &
                                                LP_COSINE_FALLOFF=PIXEL_SIZE%VALUE/50.0)
                    CALL BWD_FT(SLICE)
                    call slice%WriteToDisk(MATCHING_BOXES,wanted_location=current_output_loc)
                ENDIF
            ENDIF
        ENDDO   ! End of loop over subunits in the current image
        ! If this was the last image for this filament, add current 3D to the total 3D
        ! and write out the central slice of the current 3D to disk
        IF (LAST_IN_FILAMENT) THEN
            CALL THREED_ADD(REC3D,REC3D_CURRENT,REC3D)
            CALL THREED_EVAL_RECONSTRUCTION(REC3D_CURRENT,REC3D_CURRENT_VOL)
            ! Get XY slice
            CALL IMAGE_GET_SLICE(REC3D_CURRENT_VOL,REC3D_CURRENT_VOL_SLICE,3,REC3D_CURRENT_VOL%logical_dimensions(3)/2+1)
            TEMP_FILENAME = PATH_FROM_FILENAME(THREED_FILENAME%VALUE)
            TEMP_FILENAME = TRIM(ADJUSTL(TEMP_FILENAME))//'3drec_slice_xy_??????.mrc'
            CALL FILENAME_FROM_WILD(TEMP_FILENAME,SP(I)%FILM_NUM,TEMP_FILENAME)
            call REC3D_CURRENT_VOL_SLICE%WriteToDisk(TEMP_FILENAME,PIXEL_SIZE=pixel_size%VALUE)
            ! Get YZ slice
            CALL IMAGE_GET_SLICE(REC3D_CURRENT_VOL,REC3D_CURRENT_VOL_SLICE,1,REC3D_CURRENT_VOL%logical_dimensions(1)/2+1)
            TEMP_FILENAME = PATH_FROM_FILENAME(THREED_FILENAME%VALUE)
            TEMP_FILENAME = TRIM(ADJUSTL(TEMP_FILENAME))//'3drec_slice_yz_??????.mrc'
            CALL FILENAME_FROM_WILD(TEMP_FILENAME,SP(I)%FILM_NUM,TEMP_FILENAME)
            call REC3D_CURRENT_VOL_SLICE%WriteToDisk(TEMP_FILENAME,PIXEL_SIZE=PIXEL_SIZE%VALUE)
        ENDIF
    ENDDO

    ! Close stack file
    CALL STACK%CLOSE()


    ! Close the matching projections file
    IF (WRITE_MATCHING_PROJECTIONS%VALUE) THEN
        CALL MATCHING_PROJS%PrintInfo()
        CALL MATCHING_PROJS%CLOSE()
        CALL MATCHING_BOXES%PrintInfo()
        CALL MATCHING_BOXES%CLOSE()
    ENDIF

    ! Write the 3D to disk
    CALL THREED_WRITE_TO_DISK(REC3D,THREED_FILENAME%VALUE)

    ! Get the final 3D
    PRINT '(A)', '**info(MODE_FREALIGN_STACK_TO_THREED): finalising 3D'
    CALL THREED_EVAL_RECONSTRUCTION(REC3D,VOL,CYLINDER_MASK=.TRUE.)
    IF (APPLY_HELICAL_SYMMETRY%VALUE) THEN
        WRITE(*,'(A)') '**info(MODE_FREALIGN_STACK_TO_THREED): imposing helical symmetry'
        CALL IMAGE_IMPOSE_HELICAL_SYMMETRY_NEW(VOL,HELIX,REC3D%INTERPOL_SCHEME)
    ENDIF
    PRINT '(A)', '**info(MODE_FREALIGN_STACK_TO_THREED): nearly done'



    CALL THREED_FILE%INIT(  THREED_FILENAME%VALUE, &
                            DIM_X=VOL%logical_dimensions(1),DIM_Y=VOL%logical_dimensions(2),DIM_Z=VOL%logical_dimensions(3),&
                            PIXEL_SIZE=PIXEL_SIZE%VALUE)
    CALL VOL%WriteToDisk(THREED_FILE)
    CALL THREED_FILE%PrintInfo()
    CALL THREED_FILE%CLOSE()

END SUBROUTINE MODE_FREALIGN_STACK_TO_THREED_SLAVE

END MODULE MODE_FREALIGN_STACK_TO_THREED
