!>  \brief  Prepare files for Frealign
MODULE MODE_FREALIX_TO_FREALIGN
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE)  ::  FREALIX_TO_FREALIGN

    TYPE(RTP_CHAR), TARGET  ::  INPUT_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='waypoints.ali',                  &
                    KEYWORD='INPUT_PAR_FILENAME',           &
                    DESCRIPTION='Waypoints file describing the filaments')

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='waypoints.par',                  &
                    KEYWORD='OUTPUT_PAR_FILENAME',           &
                    DESCRIPTION='Frealign parameter file for the output')

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_STACK_FILENAME  =               &
        RTP_CHAR(   VALUE='stack.mrc',                  &
                    KEYWORD='OUTPUT_STACK_FILENAME',           &
                    DESCRIPTION='Stack of images to be used as input to Frealign')

    TYPE(RTP_REAL), TARGET  ::  SEGMENT_LENGTH  =               &
        RTP_REAL(   VALUE=128,                  &
                    KEYWORD='SEGMENT_LENGTH',           &
                    DESCRIPTION='Length of filament segments to be extracted (Angstroms)')


    TYPE(RTP_CHAR), TARGET  ::  FILM_FILES  =               &
        RTP_CHAR(   VALUE='film_????.mrc',                  &
                    KEYWORD='FILM_FILES',           &
                    DESCRIPTION='Filename pattern for the micrographs where the filaments are found')

    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=9.99,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='Pixel size of the micrographs, in Angstroms')

    TYPE(RTP_REAL), TARGET  ::  MAGNIFICATION  =               &
        RTP_REAL(   VALUE=60000.,                  &
                    KEYWORD='MAGNIFICATION',           &
                    DESCRIPTION='Magnification value to be written to Frealign parameter file')

    TYPE(RTP_CHAR), TARGET  ::  CTFTILT_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFTILT_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFFIND_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFFIND_FILES',           &
                    DESCRIPTION='')

    TYPE(RTP_LOGI), TARGET  ::  APPLY_SHIFTS_TO_IMAGES = &
        RTP_LOGI(   VALUE=.FALSE.,              &
                    KEYWORD='APPLY_SHIFTS_TO_IMAGES',   &
                    DESCRIPTION='Shift the images so that Frealign will just have to rotate them for reconstruction')

    TYPE(RTP_LOGI), TARGET  ::  APPLY_CTF_TO_IMAGES = &
        RTP_LOGI(   VALUE=.FALSE.,              &
                    KEYWORD='APPLY_CTF_TO_IMAGES',   &
                    DESCRIPTION='Apply CTF to the images. Should be false if stack will be used by Frealign.')

    TYPE(RTP_LOGI), TARGET  ::  IGNORE_SUBUNIT_NUM = &
        RTP_LOGI(   VALUE=.FALSE.,              &
                    KEYWORD='IGNORE_SUBUNIT_NUM',   &
                    DESCRIPTION='Ignore subunit_num value of waypoints when converting them to single particles')

    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS = &
        RTP_REAL(   VALUE=100., &
                    KEYWORD='HELIX_MAX_RADIUS', &
                    DESCRIPTION='Maximum radius of filament, in Angstroms')

CONTAINS

SUBROUTINE MODE_FREALIX_TO_FREALIGN_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL FREALIX_TO_FREALIGN%INIT(COMMAND='FREALIX_TO_FREALIGN',SUBROUTINE=MODE_FREALIX_TO_FREALIGN_SLAVE,  &
                                    SHORT_DESCRIPTION='Prepare a set of input files for Frealign')

    ! Add RTPs to the mode
    CALL FREALIX_TO_FREALIGN%ADD_RTP(INPUT_PAR_FILENAME)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(OUTPUT_PAR_FILENAME)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(OUTPUT_STACK_FILENAME)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(SEGMENT_LENGTH)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(FILM_FILES)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(PIXEL_SIZE)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(MAGNIFICATION)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(CTFTILT_FILES)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(CTFFIND_FILES)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(APPLY_SHIFTS_TO_IMAGES)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(APPLY_CTF_TO_IMAGES)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(IGNORE_SUBUNIT_NUM)
    CALL FREALIX_TO_FREALIGN%ADD_RTP(HELIX_MAX_RADIUS)

    ! Run the mode
    CALL FREALIX_TO_FREALIGN%RUN()
END SUBROUTINE MODE_FREALIX_TO_FREALIGN_INIT_AND_RUN


SUBROUTINE MODE_FREALIX_TO_FREALIGN_SLAVE
    USE FILAMENTS, ONLY : WP_TO_SP
    USE FREALIX_PARAMETERS
    use Images
    USE CTFTILTS
    USE FILAMENTS_BASE
    USE SINGLE_PARTICLES
    USE FREALIGN_PARAMETERS
    USE PARSERS, ONLY : STRING_EQUAL, FILENAME_FROM_WILD
    USE UI, ONLY : INTERVAL_PRINT
    USE RUNTIME_PARAMETERS, ONLY : IS_ODD, FILENAME_LEN, TERMINATE
    use ImageFiles
    IMPLICIT NONE
    ! Private variables
    TYPE(FILAMENT), ALLOCATABLE ::  FILS(:)
    TYPE(SINGLE_PARTICLE),  ALLOCATABLE ::  SP(:), SP_TMP(:)
    INTEGER ::  CURRENT_FIL
    INTEGER ::  CURRENT_WP
    INTEGER ::  NUMBER_OF_WAYPOINTS
    TYPE(IMAGE) ::  SP_IMAGE
    CHARACTER(LEN=FILENAME_LEN) :: CURRENT_FILM_FILENAME, CURRENT_CTF_FILENAME
    TYPE(IMAGE) ::  CURRENT_FILM_IMAGE
    TYPE(IMAGEFILE) ::  OUTPUT_STACK_FILE
    INTEGER ::  CURRENT_OUTPUT_LOC, START_OUTPUT_LOC
    LOGICAL ::  USE_CTFTILT
    TYPE(CTFTILT) :: CTFTILT_PARAMS
    LOGICAL,    PARAMETER   ::  DEBUG   =   .FALSE.
    INTEGER ::  BOX_DIM(2)
    ! Start work

    CALL TERMINATE('This mode is deprecated. Use STACK_FROM_PAR instead.')

    ! Are we given CTFFIND files or CTFTILT files?
    USE_CTFTILT = .FALSE.
    IF (STRING_EQUAL(CTFTILT_FILES%VALUE,'none')) THEN
        IF (STRING_EQUAL(CTFFIND_FILES%VALUE,'none')) THEN
            WRITE(*,'(A)') '**ERROR(MODE_FREALIX_TO_FREALIGN_SLAVE): neither CTFFIND nor CTFTILT files were supplied'
            CALL TERMINATE('Fatal error in MODE_FREALIX_TO_FREALIGN_SLAVE')
        ELSE
            USE_CTFTILT = .FALSE.
        ENDIF
    ELSE
        USE_CTFTILT = .TRUE.
    ENDIF

    ! Get filaments from waypoints file
    CALL FILAMENTS_FROM_FREALIX_PAR_NOINIT(FILS,INPUT_PAR_FILENAME%VALUE,PIXEL_SIZE%VALUE)
    FILS%MAX_RADIUS = HELIX_MAX_RADIUS%VALUE / PIXEL_SIZE%VALUE ! This is normally done by FILAMENT_INIT

    ! Work out total number of waypoints
    NUMBER_OF_WAYPOINTS = 0

    ! Loop over filaments
    DO CURRENT_FIL=1,SIZE(FILS)
        NUMBER_OF_WAYPOINTS = NUMBER_OF_WAYPOINTS + SIZE(FILS(CURRENT_FIL)%WP)
    ENDDO

    ! Allocate array of single particles
    ALLOCATE(SP(NUMBER_OF_WAYPOINTS))

    ! Open the output image stack file for writing
    BOX_DIM = MAX ( CEILING(3.*HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE), &
                    CEILING(SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE) )
    IF (IS_ODD(BOX_DIM(1))) BOX_DIM = BOX_DIM+1
    CALL OUTPUT_STACK_FILE%INIT(FILENAME=OUTPUT_STACK_FILENAME%VALUE,  &
                                DIM_X=BOX_DIM(1),DIM_Y=BOX_DIM(2),  &
                                PIXEL_SIZE=PIXEL_SIZE%VALUE)

    ! Loop over filaments
    CURRENT_OUTPUT_LOC = 0
    DO CURRENT_FIL=1,SIZE(FILS)
        IF (INTERVAL_PRINT(CURRENT_FIL,SIZE(FILS))) THEN
            WRITE(*,'(A,I0,A,I0)') '**info(MODE_FREALIX_TO_FREALIGN): on filament ', CURRENT_FIL, ' of ', SIZE(FILS)
        ENDIF
        ! Work out micrograph filename and the CTFTILT filename for this filament
        CALL FILENAME_FROM_WILD(FILM_FILES%VALUE,FILS(CURRENT_FIL)%FILM%ID,CURRENT_FILM_FILENAME)
        IF (USE_CTFTILT) THEN
            CALL FILENAME_FROM_WILD(CTFTILT_FILES%VALUE,FILS(CURRENT_FIL)%FILM%ID,CURRENT_CTF_FILENAME)
            CALL CTFTILT_READ_FROM_CTFTILT(CTFTILT_PARAMS,CURRENT_CTF_FILENAME)
        ELSE
            CALL FILENAME_FROM_WILD(CTFFIND_FILES%VALUE,FILS(CURRENT_FIL)%FILM%ID,CURRENT_CTF_FILENAME)
            CALL CTFTILT_READ_FROM_CTFFIND(CTFTILT_PARAMS,CURRENT_CTF_FILENAME)
        ENDIF
        ! Read the micrograph image from disk (we should check whether this is necessary -
        ! we could have the correct image in memory already; won't do this for now - OS caching
        ! probably good enough in most cases)
        CALL READ_FROM_DISK(CURRENT_FILM_IMAGE,CURRENT_FILM_FILENAME)
        ! Allocate temporary array of single particles for this filament's waypoints
        IF (ALLOCATED(SP_TMP)) DEALLOCATE(SP_TMP)
        ALLOCATE(SP_TMP(SIZE(FILS(CURRENT_FIL)%WP)))
        ! Loop over waypoints
        START_OUTPUT_LOC = CURRENT_OUTPUT_LOC + 1
        DO CURRENT_WP=1,SIZE(FILS(CURRENT_FIL)%WP)
            CURRENT_OUTPUT_LOC = CURRENT_OUTPUT_LOC + 1
            CALL WP_TO_SP(  FILS(CURRENT_FIL)%WP(CURRENT_WP),SP_TMP(CURRENT_WP),  &
                            CTF=CTFTILT_PARAMS%CTF, &
                            SP_IMAGE=SP_IMAGE,SEGMENT_LENGTH=BOX_DIM(1),  &
                            FILM_IMAGE=CURRENT_FILM_IMAGE,  &
                            PIXEL_SIZE=PIXEL_SIZE%VALUE,    &
                            APPLY_SHIFT_TO_IMAGE=APPLY_SHIFTS_TO_IMAGES%VALUE, &
                            APPLY_CTF_TO_IMAGE=APPLY_CTF_TO_IMAGES%VALUE,&
                            IGNORE_SUBUNIT_NUM=IGNORE_SUBUNIT_NUM%VALUE,&
                            FIL=FILS(CURRENT_FIL))
            IF (DEBUG) THEN
                WRITE(*,'(2(A,I0))') '**debug(MODE_FREALIX_TO_FREALIGN_SLAVE): on wp ', CURRENT_WP, ' of filament ', CURRENT_FIL
                CALL WP_PRINT_INFO(FILS(CURRENT_FIL)%WP(CURRENT_WP))
                CALL SP_PRINT_INFO(SP_TMP(CURRENT_WP))
            ENDIF
            CALL SP_IMAGE%WriteToDisk(OUTPUT_STACK_FILE,wanted_location=current_output_loc)
        ENDDO
        ! Now that we've filled the temporary array of SPs, add it to the output array of SPs
        SP_TMP%FILM_NUM = FILS(CURRENT_FIL)%ID
        SP(START_OUTPUT_LOC:START_OUTPUT_LOC+SIZE(SP_TMP)-1) = SP_TMP(1:SIZE(SP_TMP))
    ENDDO

    ! Close the output stack file
    CALL OUTPUT_STACK_FILE%PrintInfo()
    CALL OUTPUT_STACK_FILE%CLOSE()

    ! Write out the Frealign parameter file
    CALL SP_TO_FREALIGN_PAR(SP,OUTPUT_PAR_FILENAME%VALUE,MAGNIFICATION%VALUE,PIXEL_SIZE%VALUE)

    ! Sanity check
    IF (.NOT. CURRENT_OUTPUT_LOC .EQ. NUMBER_OF_WAYPOINTS) THEN
        WRITE(*,'(A)') '**ERROR(MODE_FREALIX_TO_FREALIGN_SLAVE): failed sanity check'
        CALL TERMINATE('Failed sanity check in MODE_FREALIX_TO_FREALIGN_SLAVE')
    ENDIF
END SUBROUTINE MODE_FREALIX_TO_FREALIGN_SLAVE

END MODULE MODE_FREALIX_TO_FREALIGN
