!>  \brief  Mode to impose helical symmetry onto an input image
MODULE MODE_HEXTEND
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: HEXTEND

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='test_out',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  TWIST  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  RISE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  MAX_RAD  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='HELIX_MAX_RADIUS',           &
                    DESCRIPTION='Maximum radius (Angstroms)')


    TYPE(RTP_INTG), TARGET  ::  NUM_CENTRAL_SLICES  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='NUM_CENTRAL_SLICES',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='')

    TYPE(RTP_INTG), TARGET  ::  HELICAL_STARTS  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='NUM_HELICAL_STARTS',           &
                    DESCRIPTION='Number of helical starts')

     TYPE(RTP_LOGI), TARGET  ::  STACK  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='STACK',           &
                    DESCRIPTION='The assembly is a stack rather than a true helix')

    TYPE(RTP_LOGI), TARGET  ::  NEW_ALGORITHM  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='NEW_ALGORITHM',           &
                    DESCRIPTION='')

    TYPE(RTP_LOGI), TARGET  ::  WRITE_OUT_ASYM_UNIT  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WRITE_OUT_ASYM_UNIT',           &
                    DESCRIPTION='Dump a 3D density to disk with 1 within and 0 outside the asymetric unit')

    TYPE(RTP_CHAR), TARGET  ::  ASYM_UNIT_FILENAME  =               &
        RTP_CHAR(   VALUE='asym_unit.mrc',                  &
                    KEYWORD='ASYM_UNIT_FILENAME',           &
                    DESCRIPTION='If an asymetric unit 3D density is written out, use this filename')
CONTAINS

SUBROUTINE MODE_HEXTEND_INIT_AND_RUN
    ! Intialise the mode
    CALL HEXTEND%INIT(COMMAND='HEXTEND',SUBROUTINE=MODE_HEXTEND_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL HEXTEND%ADD_RTP(INPUT_FILENAME)
    CALL HEXTEND%ADD_RTP(OUTPUT_FILENAME)
    CALL HEXTEND%ADD_RTP(TWIST)
    CALL HEXTEND%ADD_RTP(RISE)
    CALL HEXTEND%ADD_RTP(PIXEL_SIZE)
    CALL HEXTEND%ADD_RTP(MAX_RAD)
    CALL HEXTEND%ADD_RTP(NUM_CENTRAL_SLICES)
    CALL HEXTEND%ADD_RTP(SYMMETRY_AXIAL)
    CALL HEXTEND%ADD_RTP(SYMMETRY_PERP)
    CALL HEXTEND%ADD_RTP(HELICAL_STARTS)
    CALL HEXTEND%ADD_RTP(STACK)
    CALL HEXTEND%ADD_RTP(NEW_ALGORITHM)
    CALL HEXTEND%ADD_RTP(WRITE_OUT_ASYM_UNIT)
    CALL HEXTEND%ADD_RTP(ASYM_UNIT_FILENAME)

    ! Run the mode
    CALL HEXTEND%RUN()
END SUBROUTINE MODE_HEXTEND_INIT_AND_RUN

SUBROUTINE MODE_HEXTEND_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES_HELICES
    USE INTERPOLATION
    USE IMAGEFILES
    USE INTERPOLATION
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)                 ::  IMGIN
    TYPE(HELICAL_SYMMETRY)      ::  HELIX
    TYPE(INTERPOLATION_SCHEME)  ::  INTERPOL_SCHEME
    TYPE(IMAGEFILE)             ::  ASYM_UNIT_FILE
    TYPE(IMAGE)                 ::  ASYM_UNIT_VOL


    ! Start work

    ! Convert the maximum radius to pixels
    IF (MAX_RAD%VALUE .NE. 0.0) THEN
        MAX_RAD%VALUE = MAX_RAD%VALUE / PIXEL_SIZE%VALUE
    ENDIF
    ! Setup interpolation scheme
    CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'LINEAR','RECT',2)
    !CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'SINC','HANN',8)

    ! Setup the helical symmetry operator
    CALL HELICAL_SYMMETRY_INIT(HELIX,RISE%VALUE,TWIST%VALUE,SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE,&
                                NUM_STARTS=HELICAL_STARTS%VALUE,STACK=STACK%VALUE)

    ! Read the image from disk
    call imgin%ReadFromDisk(input_filename%value,read_volume_not_slice=.true.)
    ! Symmetrise
    IF (NEW_ALGORITHM%VALUE) THEN
        CALL IMAGE_IMPOSE_HELICAL_SYMMETRY_NEW(IMGIN,HELIX,INTERPOL_SCHEME,MAX_RADIUS=MAX_RAD%VALUE)
    ELSE
        CALL IMAGE_IMPOSE_HELICAL_SYMMETRY(IMGIN,HELIX,INTERPOL_SCHEME,NUM_CENTRAL_SLICES=NUM_CENTRAL_SLICES%VALUE, &
                                            MAX_RADIUS=MAX_RAD%VALUE,&
                                            PROGRESS_REPORT=.TRUE.)
    ENDIF
    ! Write to disk
    CALL IMGIN%WriteToDisk(OUTPUT_FILENAME%VALUE,PIXEL_SIZE=PIXEL_SIZE%VALUE,PRINT_FILE_INFO=.TRUE.,delete_if_already_exists=.true.)

    ! Compute & write to disk the asymetric unit mask
    IF (WRITE_OUT_ASYM_UNIT%VALUE) THEN
        CALL ASYM_UNIT_VOL%Allocate(MOULD=IMGIN)
        ASYM_UNIT_VOL%real_values = 1.0
        CALL IMAGE_MASK_HELICAL_SUBUNIT(ASYM_UNIT_VOL,HELIX,MAX_RADIUS=MAX_RAD%VALUE)
        CALL ASYM_UNIT_FILE%INIT(ASYM_UNIT_FILENAME%VALUE)
        CALL ASYM_UNIT_VOL%WriteToDisk(ASYM_UNIT_FILE)
        CALL ASYM_UNIT_FILE%PrintInfo()
        CALL ASYM_UNIT_FILE%CLOSE()
    ENDIF


END SUBROUTINE MODE_HEXTEND_SLAVE

END MODULE MODE_HEXTEND
