!>  \brief  Mode to impose helical symmetry onto an input image
MODULE MODE_HMASK
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: HMASK

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='test_out',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  TWIST  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  RISE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  MAX_RAD  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='MAX_RAD',           &
                    DESCRIPTION='Maximum radius (Angstroms)')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='Axial symmetry (->number of helical starts)')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='Perpendicular symmetry (1 or 2)')


    TYPE(RTP_REAL), TARGET  ::  ASYM_UNIT_MW  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ASYM_UNIT_MW',           &
                    DESCRIPTION='Expected molecular weight (Da) of the asymetric unit')


    TYPE(RTP_REAL), TARGET  ::  MASK_RESOLUTION  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='MASK_RESOLUTION',           &
                    DESCRIPTION='In Angstroms, the maximum resolution of the mask')
CONTAINS

SUBROUTINE MODE_HMASK_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL HMASK%INIT(COMMAND='HMASK',SUBROUTINE=MODE_HMASK_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL HMASK%ADD_RTP(INPUT_FILENAME)
    CALL HMASK%ADD_RTP(OUTPUT_FILENAME)
    CALL HMASK%ADD_RTP(TWIST)
    CALL HMASK%ADD_RTP(RISE)
    CALL HMASK%ADD_RTP(PIXEL_SIZE)
    CALL HMASK%ADD_RTP(MAX_RAD)
    CALL HMASK%ADD_RTP(SYMMETRY_AXIAL)
    CALL HMASK%ADD_RTP(SYMMETRY_PERP)
    CALL HMASK%ADD_RTP(ASYM_UNIT_MW)
    CALL HMASK%ADD_RTP(MASK_RESOLUTION)

    ! Run the mode
    CALL HMASK%RUN()

END SUBROUTINE MODE_HMASK_INIT_AND_RUN


SUBROUTINE MODE_HMASK_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES_HELICES
    USE INTERPOLATION
    USE IMAGEFILES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)             ::  IMGIN
    TYPE(HELICAL_SYMMETRY)  ::  HELIX


    ! Start work

    ! Convert the maximum radius to pixels
    IF (MAX_RAD%VALUE .NE. 0.0) THEN
        MAX_RAD%VALUE = MAX_RAD%VALUE / PIXEL_SIZE%VALUE
    ENDIF

    ! Setup the helical symmetry operator
    CALL HELICAL_SYMMETRY_INIT(HELIX,RISE%VALUE,TWIST%VALUE,SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE)
    ! Read the image from disk
    call imgin%ReadFromDisk(input_filename%value,read_volume_not_slice=.true.)
    ! Mask
    CALL IMAGE_HELIX_MASK(IMGIN,HELIX,ASYM_UNIT_MW%VALUE,PIXEL_SIZE%VALUE,HELIX_MAX_RAD=MAX_RAD%VALUE,  &
                            MASK_RESOLUTION=PIXEL_SIZE%VALUE/MASK_RESOLUTION%VALUE)
    ! Write to disk
    CALL WRITE_TO_DISK(IMGIN,OUTPUT_FILENAME%VALUE)

END SUBROUTINE MODE_HMASK_SLAVE

END MODULE MODE_HMASK
