!>  \brief  Mode to float and normalise images or a volume
MODULE MODE_NORMALISE
    ! Modules
    USE MODES
    USE RTPS


    IMPLICIT NONE
    TYPE(MODE) :: NORMALISE

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='output.mrc',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')

    TYPE(RTP_REAL), TARGET  ::  MASK_RADIUS =           &
        RTP_REAL(   VALUE=0.0,                        &
                    KEYWORD='MASK_RADIUS',              &
                    DESCRIPTION='In pixels. If 0.0, no mask.')


    TYPE(RTP_LOGI), TARGET  ::  TWO_D  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='TWO_D',           &
                    DESCRIPTION='Process 2D slices rather than the 3D volume')

CONTAINS

SUBROUTINE MODE_NORMALISE_INIT_AND_RUN
    ! Intialise the mode
    CALL NORMALISE%INIT(COMMAND='NORMALISE',SUBROUTINE=MODE_NORMALISE_SLAVE,    &
                        SHORT_DESCRIPTION='Float and normalise iamges or a volume')

    ! Add RTPs to the mode
    CALL NORMALISE%ADD_RTP(INPUT_FILENAME)
    CALL NORMALISE%ADD_RTP(OUTPUT_FILENAME)
    CALL NORMALISE%ADD_RTP(MASK_RADIUS)
    CALL NORMALISE%ADD_RTP(TWO_D)

    ! Run the mode
    CALL NORMALISE%RUN()
END SUBROUTINE MODE_NORMALISE_INIT_AND_RUN

SUBROUTINE MODE_NORMALISE_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES_ARITHM, ONLY : IMAGE_FLOAT_NORM_MASK
    use Images
    USE IMAGEFILES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)     ::  VOL, STACK
    TYPE(IMAGEFILE) ::  INPUT_FILE, OUTPUT_FILE
    INTEGER         ::  I


    ! Start work
    CALL INPUT_FILE%INIT(INPUT_FILENAME%VALUE)


    !
    IF (TWO_D%VALUE) THEN

    ELSE
        call stack%ReadFromDisk(input_file,read_volume_not_slice=.true.)
        VOL = STACK
    ENDIF

    CALL OUTPUT_FILE%INIT(OUTPUT_FILENAME%VALUE,MOULD=INPUT_FILE,PIXEL_SIZE=INPUT_FILE%GetPixelSize())

    IF (TWO_D%VALUE) THEN
        DO I=1,INPUT_FILE%GetStackSize()
            call vol%ReadFromDisk(input_file,wanted_location=i)
            CALL IMAGE_FLOAT_NORM_MASK(VOL,MASK_RADIUS=MASK_RADIUS%VALUE)
            call vol%WriteToDisk(output_file,wanted_location=i)
        ENDDO
    ELSE
        CALL IMAGE_FLOAT_NORM_MASK(VOL,MASK_RADIUS=MASK_RADIUS%VALUE)
        call vol%WriteToDisk(output_file)
    ENDIF

    CALL INPUT_FILE%CLOSE()
    CALL OUTPUT_FILE%PrintInfo()
    CALL OUTPUT_FILE%CLOSE()

END SUBROUTINE MODE_NORMALISE_SLAVE

END MODULE MODE_NORMALISE
