MODULE MODE_PAD
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: PAD

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='output.mrc',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  NEW_DIM  =               &
        RTP_INTG(   VALUE=256,                  &
                    KEYWORD='NEW_DIM',           &
                    DESCRIPTION='')
    TYPE(RTP_INTG), TARGET  ::  NEW_DIM_X  =               &
        RTP_INTG(   VALUE=256,                  &
                    KEYWORD='NEW_DIM_X',           &
                    DESCRIPTION='')
    TYPE(RTP_INTG), TARGET  ::  NEW_DIM_Y  =               &
        RTP_INTG(   VALUE=256,                  &
                    KEYWORD='NEW_DIM_Y',           &
                    DESCRIPTION='')
    TYPE(RTP_INTG), TARGET  ::  NEW_DIM_Z  =               &
        RTP_INTG(   VALUE=256,                  &
                    KEYWORD='NEW_DIM_Z',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  REAL_SPACE =    &
        RTP_LOGI(   VALUE=.TRUE.,               &
                    KEYWORD='REAL_SPACE',       &
                    DESCRIPTION='Resize the real-space image, rather than its Fourier transform')
CONTAINS

SUBROUTINE MODE_PAD_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL PAD%INIT(COMMAND='PAD',SUBROUTINE=MODE_PAD_SLAVE,SHORT_DESCRIPTION='Resize an image')

    ! Add RTPs to the mode
    CALL PAD%ADD_RTP(INPUT_FILENAME)
    CALL PAD%ADD_RTP(OUTPUT_FILENAME)
    CALL PAD%ADD_RTP(NEW_DIM)
    CALL PAD%ADD_RTP(NEW_DIM_X)
    CALL PAD%ADD_RTP(NEW_DIM_Y)
    CALL PAD%ADD_RTP(NEW_DIM_Z)
    CALL PAD%ADD_RTP(REAL_SPACE)

    ! Run the mode
    CALL PAD%RUN()
END SUBROUTINE MODE_PAD_INIT_AND_RUN


SUBROUTINE MODE_PAD_SLAVE
    USE IMAGES
    USE IMAGEFILES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)     ::  VOL
    TYPE(IMAGEFILE) ::  INPUT_FILE, OUTPUT_FILE
    REAL            ::  OUTPUT_PIXEL_SIZE
    INTEGER ::  DIM(3)


    ! Start work

    ! Open input file, load data
    CALL INPUT_FILE%INIT(INPUT_FILENAME%VALUE)
    call vol%ReadFromDisk(input_file,read_volume_not_slice=.true.)
    CALL INPUT_FILE%PrintInfo()
    OUTPUT_PIXEL_SIZE = INPUT_FILE%GetPixelSize()
    CALL INPUT_FILE%CLOSE()

    ! Work out output dimensions
    IF (NEW_DIM_X%SET_BY_USER) THEN
        DIM = [NEW_DIM_X%VALUE,NEW_DIM_Y%VALUE,NEW_DIM_Z%VALUE]
    ELSE
        IF (VOL%logical_dimensions(3) .EQ. 1) THEN ! The 2D case
            DIM = [NEW_DIM%VALUE,NEW_DIM%VALUE,1]
        ELSE
            DIM = [NEW_DIM%VALUE,NEW_DIM%VALUE,NEW_DIM%VALUE]
        ENDIF
    ENDIF

    ! Work out output pixel size
    IF (.NOT. REAL_SPACE%VALUE) THEN
        OUTPUT_PIXEL_SIZE = OUTPUT_PIXEL_SIZE / DIM(1) * VOL%logical_dimensions(1)
    ENDIF

    ! Resize
    WRITE(*,'(2(A,3(I0,1X)))') '**info(MODE_PAD): resizing from ', VOL%logical_dimensions, ' to ', DIM
    IF (.NOT. REAL_SPACE%VALUE) THEN
        WRITE(*,'(A,F0.3)') '**info(MODE_PAD): resizing in Fourier space. New pixel size = ', OUTPUT_PIXEL_SIZE
    ENDIF
    IF (.NOT. REAL_SPACE%VALUE) CALL FWD_FT(VOL)
    CALL RESIZE(VOL,DIM)
    IF (.NOT. REAL_SPACE%VALUE) CALL BWD_FT(VOL)



    ! Write out to disk
    CALL OUTPUT_FILE%INIT(OUTPUT_FILENAME%VALUE,DIM_X=DIM(1),DIM_Y=DIM(2),DIM_Z=DIM(3),PIXEL_SIZE=OUTPUT_PIXEL_SIZE)
    call vol%WriteToDisk(output_file)
    CALL OUTPUT_FILE%PrintInfo()
    CALL OUTPUT_FILE%CLOSE()



END SUBROUTINE MODE_PAD_SLAVE

END MODULE MODE_PAD
