!>  \brief  Add disorder to waypoints
MODULE MODE_RANDOMIZE_WAYPOINTS
    USE MODES
    USE RTPS
    IMPLICIT NONE

    TYPE(MODE)  ::  RANDOMIZE_WAYPOINTS

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME = &
                        RTP_CHAR(   VALUE = 'waypoints.ali',    &
                                    KEYWORD = 'INPUT_FILENAME', &
                                    DESCRIPTION = 'Filename for input waypoints')

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME = &
                        RTP_CHAR(   VALUE = 'waypoints_out.ali',    &
                                    KEYWORD = 'OUTPUT_FILENAME', &
                                    DESCRIPTION = 'Filename for output waypoints')

    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'PIXEL_SIZE',  &
                                    DESCRIPTION = 'Pixel size (Angstroms)')

    TYPE(RTP_REAL), TARGET  ::  STD_X = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'STD_X',  &
                                    DESCRIPTION = 'Standard deviation of Gaussian distribution of errors '//&
                                                  'introduced in X coordinate (Angstroms)')

    TYPE(RTP_REAL), TARGET  ::  STD_Y = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'STD_Y',  &
                                    DESCRIPTION = 'Standard deviation of Gaussian distribution of errors '//&
                                                  'introduced in Y coordinate (Angstroms)')

    TYPE(RTP_REAL), TARGET  ::  STD_PHI = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'STD_PHI',  &
                                    DESCRIPTION = 'Standard deviation of Gaussian distribution of errors introduced in phi')

    TYPE(RTP_REAL), TARGET  ::  STD_THETA = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'STD_THETA',  &
                                    DESCRIPTION = 'Standard deviation of Gaussian distribution of errors introduced in theta')

    TYPE(RTP_REAL), TARGET  ::  STD_PSI = &
                        RTP_REAL(   VALUE = 0.0,    &
                                    KEYWORD = 'STD_PSI',  &
                                    DESCRIPTION = 'Standard deviation of Gaussian distribution of errors introduced in psi')
CONTAINS

SUBROUTINE MODE_RANDOMIZE_WAYPOINTS_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL RANDOMIZE_WAYPOINTS%INIT(COMMAND='RANDOMIZE_WAYPOINTS',SUBROUTINE=MODE_RANDOMIZE_WAYPOINTS_SLAVE,  &
                                SHORT_DESCRIPTION='Modify waypoints by adding random noise')

    ! Add RTPs to the mode
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(INPUT_FILENAME)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(OUTPUT_FILENAME)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(PIXEL_SIZE)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(STD_X)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(STD_Y)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(STD_PHI)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(STD_THETA)
    CALL RANDOMIZE_WAYPOINTS%ADD_RTP(STD_PSI)

    ! Run the mode
    CALL RANDOMIZE_WAYPOINTS%RUN()
END SUBROUTINE MODE_RANDOMIZE_WAYPOINTS_INIT_AND_RUN

SUBROUTINE MODE_RANDOMIZE_WAYPOINTS_SLAVE
    USE COOS, ONLY : WAYPOINTS_ADD_DISORDER
    USE FILAMENTS_BASE
    USE FREALIX_PARAMETERS
    IMPLICIT NONE
    TYPE(FILAMENT), ALLOCATABLE ::  FILS(:)
    INTEGER ::  CURRENT_FIL
    ! Parse the input waypoints file
    CALL FILAMENTS_FROM_FREALIX_PAR_NOINIT(FILS,INPUT_FILENAME%VALUE,PIXEL_SIZE%VALUE)
    ! Loop over filaments
    DO CURRENT_FIL=1,SIZE(FILS)
        ! Add noise to waypoints
        CALL WAYPOINTS_ADD_DISORDER(FILS(CURRENT_FIL)%WP,   &
                                    CUMULATIVE_DISORDER_STD    =(/0.,0.,0.,0.,0.,0.,0./),   &
                                    NON_CUMULATIVE_DISORDER_STD=(/STD_X%VALUE,STD_Y%VALUE,0.,   &
                                                                STD_PHI%VALUE,STD_THETA%VALUE,STD_PSI%VALUE,0./))
    ENDDO
    ! Write out the new waypoints file
    CALL FILAMENTS_TO_FREALIX_PAR(FILS,OUTPUT_FILENAME%VALUE)
    WRITE(*,'(A)') '**info(MODE_RANDOMIZE_WAYPOINTS_SLAVE): all done.'

END SUBROUTINE MODE_RANDOMIZE_WAYPOINTS_SLAVE


END MODULE MODE_RANDOMIZE_WAYPOINTS
