!>  \brief  Mode to impose helical symmetry onto an input image
MODULE MODE_REBUILD_SEAM
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: REBUILD_SEAM

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILE  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='INPUT_FILE',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILE  =               &
        RTP_CHAR(   VALUE='test_out',                  &
                    KEYWORD='OUTPUT_FILE',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_REAL), TARGET  ::  TWIST  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_REAL), TARGET  ::  RISE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_REAL), TARGET  ::  MAX_RAD  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='MAX_RAD',           &
                    DESCRIPTION='Maximum radius (Angstroms)')
  
  
    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_INTG), TARGET  ::  MT_PF_NUMBER  =               &
        RTP_INTG(   VALUE=0,                  &
                    KEYWORD='MT_PF_NUMBER',           &
                    DESCRIPTION='For microtubules only: number of protofilaments (e.g. 13)')
  
  
    TYPE(RTP_INTG), TARGET  ::  MT_RISE  =               &
        RTP_INTG(   VALUE=0,                  &
                    KEYWORD='MT_RISE',           &
                    DESCRIPTION='For microtubules only: how much further along the filament (distance measured in '//&
                                'tubulin subunits) the end of a MT_PF_NUMBER-element "washer" is compared to its '//&
                                'beginning. The 13-pf microtobules have a value of 3.')
  
  
    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='',           &
                    DESCRIPTION='Maximum outer radius of the helix, in Angstroms')
CONTAINS

SUBROUTINE MODE_REBUILD_SEAM_INIT_AND_RUN
    IMPLICIT NONE
  
    ! Intialise the mode
    CALL REBUILD_SEAM%INIT(COMMAND='REBUILD_SEAM',SUBROUTINE=MODE_REBUILD_SEAM_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL REBUILD_SEAM%ADD_RTP(INPUT_FILE)
    CALL REBUILD_SEAM%ADD_RTP(OUTPUT_FILE)
    CALL REBUILD_SEAM%ADD_RTP(TWIST)
    CALL REBUILD_SEAM%ADD_RTP(RISE)
    CALL REBUILD_SEAM%ADD_RTP(PIXEL_SIZE)
    CALL REBUILD_SEAM%ADD_RTP(MAX_RAD)
    CALL REBUILD_SEAM%ADD_RTP(SYMMETRY_AXIAL)
    CALL REBUILD_SEAM%ADD_RTP(SYMMETRY_PERP)
    CALL REBUILD_SEAM%ADD_RTP(MT_PF_NUMBER)
    CALL REBUILD_SEAM%ADD_RTP(MT_RISE)
    CALL REBUILD_SEAM%ADD_RTP(HELIX_MAX_RADIUS)

    ! Run the mode
    CALL REBUILD_SEAM%RUN()
END SUBROUTINE MODE_REBUILD_SEAM_INIT_AND_RUN

SUBROUTINE MODE_REBUILD_SEAM_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES_HELICES
    USE INTERPOLATION
    USE IMAGEFILES
    USE FILAMENTS
    USE FILAMENTS_MICROTUBULES, ONLY : IMAGE_IMPOSE_MICROTUBULE_SYMMETRY
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)             ::  IMGIN
    TYPE(FILAMENT)          ::  FIL
    TYPE(HELICAL_SYMMETRY)  ::  HELIX


    ! Start work

    ! Conver the maximum radius to pixels
    IF (MAX_RAD%VALUE .NE. 0.0) THEN
        MAX_RAD%VALUE = MAX_RAD%VALUE / PIXEL_SIZE%VALUE
    ENDIF

    ! Setup the helical symmetry operator
    !CALL HELIX%INIT(RISE%VALUE,TWIST%VALUE,SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE)
    CALL HELICAL_SYMMETRY_INIT(HELIX,RISE%VALUE,TWIST%VALUE,SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE)

    ! Set up a filament object. We don't do a full initialisation, to save time
    FIL%MICROTUBULE = .TRUE.
    FIL%MT_PF_NUMBER = MT_PF_NUMBER%VALUE
    FIL%MT_RISE = MT_RISE%VALUE
    FIL%MAX_RADIUS = MAX_RAD%VALUE
    FIL%HELIX = HELIX

    ! Read the image from disk
    CALL READ_FROM_DISK(IMGIN,INPUT_FILE%VALUE)
    ! Symmetrise
    !CALL IMAGE_IMPOSE_HELICAL_SYMMETRY(IMGIN,HELIX,NUM_CENTRAL_SLICES,LINEAR,RECT,2)
    !CALL IMAGE_IMPOSE_HELICAL_SYMMETRY(IMGIN,HELIX,NUM_CENTRAL_SLICES,LINEAR,UNITY_WINDOW,2,MAX_RADIUS=MAX_RAD%VALUE)
    CALL IMAGE_IMPOSE_MICROTUBULE_SYMMETRY(FIL,IMGIN)
    ! Write to disk
    CALL WRITE_TO_DISK(IMGIN,OUTPUT_FILE%VALUE)

END SUBROUTINE MODE_REBUILD_SEAM_SLAVE

END MODULE MODE_REBUILD_SEAM
