!>  \brief  Mode to test the ROTATE_IMAGE subroutine
!!  \todo   Make the interpolation scheme and the real- Fourier-space choice runtime parameters
MODULE MODE_ROTATE
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: ROTATE

    TYPE(RTP_CHAR), TARGET  ::  FILENAME_INPUT  =               &
        RTP_CHAR(   VALUE='in.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  FILENAME_OUTPUT  =               &
        RTP_CHAR(   VALUE='out.mrc',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PHI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PHI',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  THETA  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='THETA',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PSI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PSI',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  SHIFT_X  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='SHIFT_X',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  SHIFT_Y  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='SHIFT_Y',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  SHIFT_Z  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='SHIFT_Z',           &
                    DESCRIPTION='')

CONTAINS

SUBROUTINE MODE_ROTATE_INIT_AND_RUN
    ! Intialise the mode
    CALL ROTATE%INIT(COMMAND='ROTATE',SUBROUTINE=MODE_ROTATE_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL ROTATE%ADD_RTP(FILENAME_INPUT)
    CALL ROTATE%ADD_RTP(FILENAME_OUTPUT)
    CALL ROTATE%ADD_RTP(PHI)
    CALL ROTATE%ADD_RTP(THETA)
    CALL ROTATE%ADD_RTP(PSI)
    CALL ROTATE%ADD_RTP(SHIFT_X)
    CALL ROTATE%ADD_RTP(SHIFT_Y)
    CALL ROTATE%ADD_RTP(SHIFT_Z)

    ! Run the mode
    CALL ROTATE%RUN()
END SUBROUTINE MODE_ROTATE_INIT_AND_RUN


SUBROUTINE MODE_ROTATE_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    use interpolation, only : INTERPOLATION_SCHEME, INTERPOLATION_SCHEME_INIT
    USE ROTATION_MATRICES
    use ImageFiles
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)             ::  IMGIN,IMGOUT
    REAL                    ::  MATRIX(4,4)
    LOGICAL,    PARAMETER   ::  DEBUG               =   .TRUE.
    LOGICAL,    PARAMETER   ::  REAL_SPACE_ROTATION =   .TRUE.
    TYPE(INTERPOLATION_SCHEME)  ::  INTERPOL_SCHEME
    TYPE(IMAGEFILE)         ::  INPUT_FILE
    TYPE(WINDOW)            ::  INTERPOL_WINDOW


    ! Start work
    CALL INPUT_FILE%INIT(FILENAME_INPUT%VALUE)
    call IMGIN%ReadFromDisk(input_file,read_volume_not_slice=.true.)
    CALL INPUT_FILE%PrintInfo()
    CALL INPUT_FILE%CLOSE()
    WRITE(*,'(A)') '**warning(MODE_ROTATE): reading image in as 3D, without checking whether it actually is'

    ! Set up interpolation
    CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'LINEAR','RECT',2)
    CALL INTERPOL_WINDOW%AllocateWindow(INTERPOL_SCHEME%WINDOW_WIDTH,COMP=.NOT.REAL_SPACE_ROTATION,THREED=IMGIN%IsAVolume())

    ! Get the appropriate rotation matrix (ROTATE_IMAGE needs to matrix to go from output back to input, which is why we
    ! use REVERSE=.TRUE.)
    MATRIX = EULER_TO_ROTMAT(DBLE(PHI%VALUE),DBLE(THETA%VALUE),DBLE(PSI%VALUE),REVERSE=.TRUE.)
    MATRIX(1,4) = SHIFT_X%VALUE
    MATRIX(2,4) = SHIFT_Y%VALUE
    MATRIX(3,4) = SHIFT_Z%VALUE
    IF (DEBUG) THEN
        WRITE(*,'(A)') '**debug(MODE_ROTATE): rotation matrix will be: '
        CALL PRINT_ROTMAT(MATRIX)
    ENDIF


    CALL IMAGE_PRINT_INFO(IMGIN)

    ! Rotate the image
    IF (.NOT. REAL_SPACE_ROTATION) THEN
        CALL FWD_FT(IMGIN)
        CALL SHIFT_REAL_SPACE_ORIGIN(IMGIN)
    ENDIF
    CALL IMAGE_ALLOCATE(IMGOUT,IMGIN)
    CALL ROTATE_IMAGE(IMGIN,IMGOUT,MATRIX,INTERPOL_SCHEME,INTERPOL_WINDOW)
    IF (.NOT. REAL_SPACE_ROTATION) THEN
        CALL SHIFT_REAL_SPACE_ORIGIN(IMGOUT)
        CALL BWD_FT(IMGOUT)
    ENDIF

    CALL IMAGE_PRINT_INFO(IMGOUT)


    ! Write output to disk
    CALL IMGOUT%WriteToDisk(FILENAME_OUTPUT%VALUE,PIXEL_SIZE=INPUT_FILE%GetPixelSize(),print_file_info=.TRUE.)

END SUBROUTINE MODE_ROTATE_SLAVE

END MODULE MODE_ROTATE
