MODULE MODE_RPS
    ! Modules
    USE MODES
    USE RTPS

    IMPLICIT NONE
    TYPE(MODE) :: RPS

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input1.mrc',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=0.000,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')
CONTAINS

SUBROUTINE MODE_RPS_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL RPS%INIT(COMMAND='RPS',SUBROUTINE=MODE_RPS_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL RPS%ADD_RTP(INPUT_FILENAME)
    CALL RPS%ADD_RTP(PIXEL_SIZE)

    ! Run the mode
    CALL RPS%RUN()
END SUBROUTINE MODE_RPS_INIT_AND_RUN

SUBROUTINE MODE_RPS_SLAVE
    USE IMAGES
    USE RUNTIME_PARAMETERS
    USE IMAGEFILES
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)         ::  IMG
    REAL,   ALLOCATABLE ::  RPS(:)
    INTEGER             ::  I
    REAL                ::  FREQ, RESOL
    CHARACTER(LEN=10)   ::  COL_TITLE
    LOGICAL,    PARAMETER   ::  LOG_SCALE = .FALSE.
    TYPE(IMAGEFILE)         ::  INPUT_FILE


    ! Start work
    CALL INPUT_FILE%INIT(INPUT_FILENAME%VALUE,PIXEL_SIZE=PIXEL_SIZE%VALUE)
    call img%ReadFromDisk(input_file)
    CALL INPUT_FILE%PrintInfo()
    CALL INPUT_FILE%CLOSE()
    !CALL READ_FROM_DISK(IMG,INPUT_FILENAME%VALUE)
    !RPS =  ROT_POWER_SPECTRUM(IMG,LOG_SCALE=LOG_SCALE)
    WRITE(*,'(A)') '**info(RPS): computing filtered amplitude spectrum (routine THON_RINGS)'
    RPS =  THON_RINGS(IMG,LOG_SCALE=LOG_SCALE)
    ! Print out rotationally-averaged power spectrum
    IF (LOG_SCALE) THEN
        WRITE(COL_TITLE,'(A)') 'Log(Power)'
    ELSE
        WRITE(COL_TITLE,'(A)') 'Power'
    ENDIF
    IF (PIXEL_SIZE%VALUE .EQ. 0.0) THEN
        WRITE(*,'(A5,2X,3X,A3)') 'Shell', TRIM(ADJUSTL(COL_TITLE))
        DO I=1,SIZE(RPS)
            WRITE(*,'(1X,I4,2X,G15.3)')   I-1, RPS(I)
        ENDDO
    ELSE
        WRITE(*,'(1X,4(A10,1X))') 'Shell', 'Freq', 'Resol', TRIM(ADJUSTL(COL_TITLE))
        DO I=1,SIZE(RPS)
            FREQ = REAL(I-1)/REAL(SIZE(RPS))/2.0/PIXEL_SIZE%VALUE
            RESOL = 0.0
            IF (I .GT. 1) RESOL = 1/FREQ
            WRITE(*,'(1X,I10,1X,3(F10.3,1X))')   I-1, FREQ, RESOL, RPS(I)
        ENDDO
    ENDIF





END SUBROUTINE MODE_RPS_SLAVE

END MODULE MODE_RPS
