MODULE MODE_SPLIT_PAR
!DEC$ OPTIMIZE:0
    ! Modules
    USE RUNTIME_PARAMETERS
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: SPLIT_PAR

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='input.par',                  &
                    KEYWORD='INPUT_FILENAME',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_ROOT_FILENAME  =               &
        RTP_CHAR(   VALUE='output_???.par',                  &
                    KEYWORD='OUTPUT_ROOT_FILENAME',           &
                    DESCRIPTION='')
  
  
    TYPE(RTP_INTG), TARGET  ::  NUMBER_OF_GROUPS  =               &
        RTP_INTG(   VALUE=50,                  &
                    KEYWORD='NUMBER_OF_GROUPS',           &
                    DESCRIPTION='')
  
CONTAINS

SUBROUTINE MODE_SPLIT_PAR_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL SPLIT_PAR%INIT(COMMAND='SPLIT_PAR',SUBROUTINE=MODE_SPLIT_PAR_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL SPLIT_PAR%ADD_RTP(INPUT_FILENAME)
    CALL SPLIT_PAR%ADD_RTP(OUTPUT_ROOT_FILENAME)
    CALL SPLIT_PAR%ADD_RTP(NUMBER_OF_GROUPS)

    ! Run the mode
    CALL SPLIT_PAR%RUN()
END SUBROUTINE MODE_SPLIT_PAR_INIT_AND_RUN


SUBROUTINE MODE_SPLIT_PAR_SLAVE
    USE FREALIX_PARAMETERS
    USE FILAMENTS
    USE PARSERS
    IMPLICIT NONE
    ! Variables
    TYPE(FILAMENT), ALLOCATABLE ::  FILAMENT_ARRAY(:), FILAMENT_GROUP(:)
    INTEGER,        ALLOCATABLE ::  GROUP_MEMBERSHIP(:), MEMBERS_PER_GROUP(:)
    INTEGER     ::  I, J, IERR
    INTEGER     ::  COUNT_IN_CURRENT_GROUP
    CHARACTER(LEN=FILENAME_LEN) ::  GROUP_FILENAME


    ! Start work

    ! Read in filament array from file
    CALL FILAMENTS_FROM_FREALIX_PAR_NOINIT(FILAMENT_ARRAY,INPUT_FILENAME%VALUE,1.0)

    ! Assign filaments to groups
    CALL FILAMENT_ARRAY_SPLIT(FILAMENT_ARRAY,NUMBER_OF_GROUPS%VALUE,GROUP_MEMBERSHIP,MEMBERS_PER_GROUP)

    ! Loop over the group
    DO I=1,NUMBER_OF_GROUPS%VALUE
        ! Reallocate filament_group
        IF (ALLOCATED(FILAMENT_GROUP)) DEALLOCATE(FILAMENT_GROUP)
        ALLOCATE(FILAMENT_GROUP(MEMBERS_PER_GROUP(I)),STAT=IERR)
        IF (IERR .NE. 0) CALL TERMINATE('Memory allocation failed')
        ! Copy filaments over to the group
        COUNT_IN_CURRENT_GROUP = 0
        DO J=1,SIZE(FILAMENT_ARRAY)
            IF (GROUP_MEMBERSHIP(J) .EQ. I) THEN
                !WRITE(*,'(3(A,I))') '**debug(MODE_SPLIT_PAR): adding filament ',  FILAMENT_ARRAY(J)%ID, ' which has ', FILAMENT_ARRAY(J)%NWP, ' waypoints ', SIZE(FILAMENT_ARRAY(J)%WP)
                COUNT_IN_CURRENT_GROUP = COUNT_IN_CURRENT_GROUP + 1
                FILAMENT_GROUP(COUNT_IN_CURRENT_GROUP) = FILAMENT_ARRAY(J)
                FILAMENT_GROUP(COUNT_IN_CURRENT_GROUP)%WP = FILAMENT_ARRAY(J)%WP
            ENDIF
        ENDDO
        ! Work out filename for this group's parameter file
        CALL FILENAME_FROM_WILD(OUTPUT_ROOT_FILENAME%VALUE,I,GROUP_FILENAME)
        !
        CALL FILAMENTS_TO_FREALIX_PAR(FILAMENT_GROUP,GROUP_FILENAME)
    ENDDO

END SUBROUTINE MODE_SPLIT_PAR_SLAVE

END MODULE MODE_SPLIT_PAR
