!>  \brief  Create a particle stack from a frealix waypoints parameter file
!!
MODULE MODE_STACK_FROM_PAR
    USE MODES
    USE RTPS
    IMPLICIT NONE

    TYPE(MODE) :: STACK_FROM_PAR

    TYPE(RTP_CHAR), TARGET  ::  PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='waypoints.ali',                  &
                    KEYWORD='INPUT_PAR_FILENAME',           &
                    DESCRIPTION='Parameter file')

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FREALIGN_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='OUTPUT_FREALIGN_PAR_FILENAME',           &
                    DESCRIPTION='Frealign parameter filename for the output')

    TYPE(RTP_CHAR), TARGET  ::  MICROGRAPH_FILENAME_WILD  =               &
        RTP_CHAR(   VALUE='micrograph???_bin8.mrc',                  &
                    KEYWORD='FILM_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFTILT_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFTILT_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFFIND_FILES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFFIND_FILES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  RECONSTRUCTION_FILENAME  =               &
        RTP_CHAR(   VALUE='orig_vol.mrc',                  &
                    KEYWORD='RECONSTRUCTION_FILENAME',           &
                    DESCRIPTION='If requested, projections will be calculated from this 3D reconstruction')


    TYPE(RTP_CHAR), TARGET  ::  STACK_FILENAME  =               &
        RTP_CHAR(   VALUE='stack.mrc',                  &
                    KEYWORD='OUTPUT_STACK_FILENAME',           &
                    DESCRIPTION='Filename for the stack of boxed particles')


    TYPE(RTP_CHAR), TARGET  ::  CTF_STACK_FILENAME  =               &
        RTP_CHAR(   VALUE='stack_ctf.mrc',                  &
                    KEYWORD='CTF_STACK_FILENAME',           &
                    DESCRIPTION='Filename for the stack of CTF functions')


    TYPE(RTP_CHAR), TARGET  ::  PROJ_FILENAME  =               &
        RTP_CHAR(   VALUE='proj.mrc',                  &
                    KEYWORD='PROJ_FILENAME',           &
                    DESCRIPTION='Output stack filename')


    TYPE(RTP_CHAR), TARGET  ::  AMPL_STACK_FILENAME  =               &
        RTP_CHAR(   VALUE='stack_ampl.mrc',                  &
                    KEYWORD='AMPL_STACK_FILENAME',           &
                    DESCRIPTION='Output stack filename (amplitude spectra)')


    TYPE(RTP_CHAR), TARGET  ::  AMPL_AVE_FILENAME  =               &
        RTP_CHAR(   VALUE='stack_ampl_ave.mrc',                  &
                    KEYWORD='AMPL_AVE_FILENAME',           &
                    DESCRIPTION='Output average amplitude spectrum')


    TYPE(RTP_REAL), TARGET  ::  SEGMENT_LENGTH  =       &
        RTP_REAL(   VALUE=120.,                         &
                    KEYWORD='SEGMENT_LENGTH',           &
                    DESCRIPTION='Length of filament segments to be extracted (Angstroms)')

    TYPE(RTP_LOGI), TARGET  ::  ONE_IMAGE_PER_REPEAT   =   &
        RTP_LOGI(   VALUE=.FALSE.,      &
                    KEYWORD='ONE_IMAGE_PER_REPEAT',    &
                    DESCRIPTION='Output one image per helical repeat, rather than one per waypoint')


    TYPE(RTP_REAL), TARGET  ::  PHI_ANGLE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PHI_ANGLE',           &
                    DESCRIPTION='Phi (helical twist) angle at which segments should be extracted')


    TYPE(RTP_REAL), TARGET  ::  PHI_PERIOD  =               &
        RTP_REAL(   VALUE=180.0,                  &
                    KEYWORD='PHI_PERIOD',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=1.17,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='Pixel size (Angstroms)')

    TYPE(RTP_REAL), TARGET  ::  MAGNIFICATION = &
        RTP_REAL(   VALUE=60000.,   &
                    KEYWORD='MAGNIFICATION', &
                    DESCRIPTION='Magnification value for the output Frealign parameter file')

    TYPE(RTP_LOGI), TARGET  ::  CENTER_IMAGES = &
        RTP_LOGI(   VALUE=.FALSE.,  &
                    KEYWORD='CENTER_IMAGES',    &
                    DESCRIPTION='Apply non-integer shift to boxed images so that they are centered')

    TYPE(RTP_LOGI), TARGET  ::  IGNORE_SUBUNIT_NUM = &
        RTP_LOGI(   VALUE=.FALSE.,  &
                    KEYWORD='IGNORE_SUBUNIT_NUM',    &
                    DESCRIPTION='Ignore the subunit_num value of input waypoints')

    TYPE(RTP_LOGI), TARGET  ::  CTF_CORRECTION  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='CTF_CORRECTION',           &
                    DESCRIPTION='Whether to correct boxed images for the CTF')


    TYPE(RTP_LOGI), TARGET  ::  CTF_FLIP_ONLY  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='CTF_FLIP_ONLY',           &
                    DESCRIPTION='Multiply images by the sign of the CTF. Else, actual multiplication is done.')


    TYPE(RTP_LOGI), TARGET  ::  WRITE_CTF_STACK  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WRITE_CTF_STACK',           &
                    DESCRIPTION='Whether to dump a stack of CTF functions to disk. One per image.')


    TYPE(RTP_LOGI), TARGET  ::  OUT_OF_PLANE_SELECT  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='OUT_OF_PLANE_SELECT',           &
                    DESCRIPTION='Whether to select for a range of out-of-plane tilts')


    TYPE(RTP_REAL), TARGET  ::  OUT_OF_PLANE_MIN  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='OUT_OF_PLANE_MIN',           &
                    DESCRIPTION='Minimum out-of-plane tilt allowed')


    TYPE(RTP_REAL), TARGET  ::  OUT_OF_PLANE_MAX  =               &
        RTP_REAL(   VALUE=5.0,                  &
                    KEYWORD='OUT_OF_PLANE_MAX',           &
                    DESCRIPTION='Maximum out-of-plane tilt allowed')


    TYPE(RTP_REAL), TARGET  ::  MIN_SCORE  =               &
        RTP_REAL(   VALUE=-1.0,                  &
                    KEYWORD='MIN_SCORE',           &
                    DESCRIPTION='Filaments with scores lower than this threshold will not be included')


    TYPE(RTP_LOGI), TARGET  ::  APPLY_ENV_MASK  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='APPLY_ENV_MASK',           &
                    DESCRIPTION='Apply an envelope mask to each image, based on the input 3D reconstruction provided')

    TYPE(RTP_LOGI), TARGET  ::  APPLY_RECT_MASK = &
        RTP_LOGI(   VALUE=.TRUE.,   &
                    KEYWORD='APPLY_RECT_MASK',  &
                    DESCRIPTION='Apply a rectangular mask to suppress solvent background')


    TYPE(RTP_LOGI), TARGET  ::  EXTRACT_SEGMENTS  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='EXTRACT_SEGMENTS',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  EXTRACT_PROJECTIONS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='EXTRACT_PROJECTIONS',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  ROTATE_PARTICLES  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='ROTATE_PARTICLES',           &
                    DESCRIPTION='')

    TYPE(RTP_LOGI), TARGET  ::  SQUARE_OUTPUT_IMAGES  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='SQUARE_OUTPUT_IMAGES',           &
                    DESCRIPTION='Generate square output images & spectra ')


    TYPE(RTP_LOGI), TARGET  ::  UNBEND_SEGMENTS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='UNBEND_SEGMENTS',           &
                    DESCRIPTION='Whether the filament segments should be unbent')


    TYPE(RTP_LOGI), TARGET  ::  AMPL_SPECTRA  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='AMPL_SPECTRA',           &
                    DESCRIPTION='Whether we need to calculate amplitude spectra')


    TYPE(RTP_LOGI), TARGET  ::  AMPL_SPECTRA_AVERAGE  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='AMPL_SPECTRA_AVERAGE',           &
                    DESCRIPTION='Whether we need to calculate average of power spectra')


    TYPE(RTP_LOGI), TARGET  ::  IGNORE_FAL_WPS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='IGNORE_FAL_WPS',           &
                    DESCRIPTION='If .TRUE., the first and last waypoints of each filament will be ignored')


    TYPE(RTP_REAL), TARGET  ::  TWIST_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  RISE_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  WPS_ARE_CROSSOVERS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='WPS_ARE_CROSSOVERS',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  FAL_WPS_ARE_CROSSOVERS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='FAL_WPS_ARE_CROSSOVERS',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  AMYLOID_BETA  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='HELIX_MAX_RADIUS',           &
                    DESCRIPTION='Maximum outer radius of the helix, in Angstroms')


    TYPE(RTP_REAL), TARGET  ::  ASYM_UNIT_MW  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ASYM_UNIT_MW',           &
                    DESCRIPTION='Expected molecular weight (Da) of the asymetric unit')


    TYPE(RTP_INTG), TARGET  ::  NUM_HELICAL_STARTS  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='NUM_HELICAL_STARTS',           &
                    DESCRIPTION='Number of helical starts. The helical parameters given above must describeone of'//    &
                                ' these starts. Not including symmetry (e.g. could be describing a 1-start helix '//    &
                                'with an additional 2-fold axial symmetry).')

     TYPE(RTP_LOGI), TARGET  ::  FRAGMENTED_PHI  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='FRAGMENTED_PHI',           &
                    DESCRIPTION='Phi function is discontinuous, with breaks between waypoints')

    TYPE(RTP_LOGI), TARGET  ::  FRAGMENTED_SUBUNIT_NUM  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='FRAGMENTED_SUBUNIT_NUM',           &
                    DESCRIPTION='Subunit_num function is discontinuous, with breaks between waypoints')

    TYPE(RTP_LOGI), TARGET  ::  CONSTANT_TWIST  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='CONSTANT_TWIST',           &
                    DESCRIPTION='The twist per unit length is constant.')


    TYPE(RTP_LOGI), TARGET  ::  CONSTANT_RISE_PER_SUBUNIT  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='CONSTANT_RISE_PER_SUBUNIT',           &
                    DESCRIPTION='The rise per subunit is constant. Note that non-cumulative axial disorder is not accounted for.')


    TYPE(RTP_CHAR), TARGET  ::  INTERPOL_FUNCTION  =               &
        RTP_CHAR(   VALUE='LINEAR',                  &
                    KEYWORD='INTERPOL_FUNCTION',           &
                    DESCRIPTION='Which interpolation function to use')


    TYPE(RTP_CHAR), TARGET  ::  WINDOW_FUNCTION  =               &
        RTP_CHAR(   VALUE='RECT',                  &
                    KEYWORD='WINDOW_FUNCTION',           &
                    DESCRIPTION='Which windowing function to use')


    TYPE(RTP_INTG), TARGET  ::  WINDOW_WIDTH  =               &
        RTP_INTG(   VALUE=2,                  &
                    KEYWORD='WINDOW_WIDTH',           &
                    DESCRIPTION='Window width for interpolation (kernel support)')

    TYPE(RTP_REAL), TARGET  ::  STEP_SIZE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='STEP_SIZE',           &
                    DESCRIPTION='Spacing between segments to be cut out (in Angstroms). Give 0.0 use segment length'//&
                                ' so there is no overlap between segments.')

CONTAINS

SUBROUTINE MODE_STACK_FROM_PAR_INIT_AND_RUN
    IMPLICIT NONE

    ! Intialise the mode
    CALL STACK_FROM_PAR%INIT(COMMAND='STACK_FROM_PAR',SUBROUTINE=MODE_STACK_FROM_PAR_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL STACK_FROM_PAR%ADD_RTP(PAR_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(OUTPUT_FREALIGN_PAR_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(MICROGRAPH_FILENAME_WILD)
    CALL STACK_FROM_PAR%ADD_RTP(CTFTILT_FILES)
    CALL STACK_FROM_PAR%ADD_RTP(CTFFIND_FILES)
    CALL STACK_FROM_PAR%ADD_RTP(RECONSTRUCTION_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(STACK_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(CTF_STACK_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(PROJ_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(AMPL_STACK_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(AMPL_AVE_FILENAME)
    CALL STACK_FROM_PAR%ADD_RTP(SEGMENT_LENGTH)
    CALL STACK_FROM_PAR%ADD_RTP(ONE_IMAGE_PER_REPEAT)
    CALL STACK_FROM_PAR%ADD_RTP(PHI_ANGLE)
    CALL STACK_FROM_PAR%ADD_RTP(PHI_PERIOD)
    CALL STACK_FROM_PAR%ADD_RTP(PIXEL_SIZE)
    CALL STACK_FROM_PAR%ADD_RTP(MAGNIFICATION)
    CALL STACK_FROM_PAR%ADD_RTP(CENTER_IMAGES)
    CALL STACK_FROM_PAR%ADD_RTP(IGNORE_SUBUNIT_NUM)
    CALL STACK_FROM_PAR%ADD_RTP(CTF_CORRECTION)
    CALL STACK_FROM_PAR%ADD_RTP(CTF_FLIP_ONLY)
    CALL STACK_FROM_PAR%ADD_RTP(WRITE_CTF_STACK)
    CALL STACK_FROM_PAR%ADD_RTP(OUT_OF_PLANE_SELECT)
    CALL STACK_FROM_PAR%ADD_RTP(OUT_OF_PLANE_MIN)
    CALL STACK_FROM_PAR%ADD_RTP(OUT_OF_PLANE_MAX)
    CALL STACK_FROM_PAR%ADD_RTP(MIN_SCORE)
    CALL STACK_FROM_PAR%ADD_RTP(APPLY_ENV_MASK)
    CALL STACK_FROM_PAR%ADD_RTP(APPLY_RECT_MASK)
    CALL STACK_FROM_PAR%ADD_RTP(EXTRACT_SEGMENTS)
    CALL STACK_FROM_PAR%ADD_RTP(EXTRACT_PROJECTIONS)
    CALL STACK_FROM_PAR%ADD_RTP(ROTATE_PARTICLES)
    CALL STACK_FROM_PAR%ADD_RTP(SQUARE_OUTPUT_IMAGES)
    CALL STACK_FROM_PAR%ADD_RTP(UNBEND_SEGMENTS)
    CALL STACK_FROM_PAR%ADD_RTP(STEP_SIZE)
    CALL STACK_FROM_PAR%ADD_RTP(AMPL_SPECTRA)
    CALL STACK_FROM_PAR%ADD_RTP(AMPL_SPECTRA_AVERAGE)
    CALL STACK_FROM_PAR%ADD_RTP(IGNORE_FAL_WPS)
    CALL STACK_FROM_PAR%ADD_RTP(TWIST_PER_SUBUNIT)
    CALL STACK_FROM_PAR%ADD_RTP(RISE_PER_SUBUNIT)
    CALL STACK_FROM_PAR%ADD_RTP(SYMMETRY_AXIAL)
    CALL STACK_FROM_PAR%ADD_RTP(SYMMETRY_PERP)
    CALL STACK_FROM_PAR%ADD_RTP(WPS_ARE_CROSSOVERS)
    CALL STACK_FROM_PAR%ADD_RTP(FAL_WPS_ARE_CROSSOVERS)
    CALL STACK_FROM_PAR%ADD_RTP(AMYLOID_BETA)
    CALL STACK_FROM_PAR%ADD_RTP(HELIX_MAX_RADIUS)
    CALL STACK_FROM_PAR%ADD_RTP(ASYM_UNIT_MW)
    CALL STACK_FROM_PAR%ADD_RTP(NUM_HELICAL_STARTS)
    CALL STACK_FROM_PAR%ADD_RTP(FRAGMENTED_PHI)
    CALL STACK_FROM_PAR%ADD_RTP(FRAGMENTED_SUBUNIT_NUM)
    CALL STACK_FROM_PAR%ADD_RTP(CONSTANT_TWIST)
    CALL STACK_FROM_PAR%ADD_RTP(CONSTANT_RISE_PER_SUBUNIT)
    CALL STACK_FROM_PAR%ADD_RTP(INTERPOL_FUNCTION)
    CALL STACK_FROM_PAR%ADD_RTP(WINDOW_FUNCTION)
    CALL STACK_FROM_PAR%ADD_RTP(WINDOW_WIDTH)

    ! Run the mode
    CALL STACK_FROM_PAR%RUN()
END SUBROUTINE MODE_STACK_FROM_PAR_INIT_AND_RUN


SUBROUTINE MODE_STACK_FROM_PAR_SLAVE
    USE RUNTIME_PARAMETERS
    USE PARSERS
    USE IMAGES
    USE IMAGEFILES
    USE ROTATION_MATRICES
    USE IMAGES_ARITHM
    USE IMAGES_CTF
    USE IMAGES_THREED
    USE FILAMENTS
    USE FREALIX_PARAMETERS
    USE HELICES
    USE IMAGEFILES
    USE IMAGES_HELICES
    USE MASKS
    USE FILAMENTS_PATH
    USE SINGLE_PARTICLES
    USE FREALIGN_PARAMETERS
    USE SPLINES, ONLY : INTERPOL_SPLINE_FIND_ROOTS_OF_MULTIPLE
    use interpolation, only : INTERPOLATION_SCHEME, INTERPOLATION_SCHEME_INIT
    USE FILAMENTS_GEOMETRY, ONLY : WAYPOINTS_PSI_THETA_SMOOTH
    ! Private variables
    INTEGER                             ::  NUM_OF_WPS              ! Number of images defined in parameter file
    TYPE(IMAGE)                         ::  MICROGRAPH              ! Will hold current micrograph in memory
    TYPE(IMAGE)                         ::  BOX                     ! Will hold current box in memory
    INTEGER                             ::  CURRENT_FILAMENT, J
    INTEGER                             ::  CURRENT_WP
    LOGICAL                             ::  NEW_MICROGRAPH          ! Whether current image comes from a new micrograph
    TYPE(IMAGE)                         ::  BOX_AMPL                ! Individual box amplitude spectrum
    TYPE(IMAGE)                         ::  AVE_AMPL                ! Average amplitude spectrum
    INTEGER                             ::  LOC_IN_STACK            ! Current location in output stack file
    INTEGER                             ::  MICROGRAPH_IN_MEMORY    ! Number of the micrograph currently held in memory
    TYPE(IMAGE)                         ::  REC                     ! Will hold the 3D reconstruction in memory
    TYPE(IMAGE)                         ::  PROJ                    ! Will hold the current projection in memory
    TYPE(IMAGE)                         ::  PROJ_MASK, REC_MASK
    TYPE(FILAMENT),         ALLOCATABLE ::  FILS(:)
    LOGICAL                             ::  USE_CTFTILT
    TYPE(HELICAL_SYMMETRY)              ::  HELIX_AVERAGE
    TYPE(IMAGEFILE)                     ::  FILM_FILE, REC_FILE
    TYPE(IMAGEFILE)                     ::  SEGMENTS_FILE, PROJECTIONS_FILE, AMPL_AVE_FILE, AMPL_FILE
    REAL(KIND=8),           ALLOCATABLE ::  TS(:)
    TYPE(INTERPOLATION_SCHEME)          ::  INTERPOL_SCHEME, INTERPOL_LINEAR
    LOGICAL,                PARAMETER   ::  DEBUG   =   .FALSE.
    TYPE(WAYPOINT)                      ::  WP
    TYPE(WAYPOINT),         ALLOCATABLE ::  WPS(:)
    INTEGER                             ::  BOX_SIZE
    REAL                                ::  CURRENT_FILAMENT_LENGTH
    TYPE(SINGLE_PARTICLE)               ::  SP_TMP
    TYPE(SINGLE_PARTICLE),  ALLOCATABLE ::  SPS(:), SPS_TMP(:)
    LOGICAL                             ::  SP_NEAR_EDGE

    !
    ! Some features not yet implemented
    !
    IF (OUT_OF_PLANE_SELECT%VALUE) THEN
        CALL TERMINATE('Out-of-plane selection not yet implemented')
    ENDIF
    IF (MIN_SCORE%VALUE .GT. -1.) THEN
        CALL TERMINATE('Score selection not yet implemented')
    ENDIF
    IF (IGNORE_FAL_WPS%VALUE) THEN
        CALL TERMINATE('FAL WPs exclusion not yet implemented')
    ENDIF
    IF (WRITE_CTF_STACK%VALUE) THEN
        CALL TERMINATE('CTF stack not implemented yet')
    ENDIF
    IF (UNBEND_SEGMENTS%VALUE .AND. CTF_CORRECTION%VALUE) THEN
        WRITE(*,'(A)') '**ERROR(MODE_STACK_FROM_PAR): CTF correction and unbending options are not compatible'
        CALL TERMINATE('Fatal error in MODE_STACK_FROM_PAR')
    ENDIF

    IF (.NOT. SQUARE_OUTPUT_IMAGES%VALUE) THEN
        IF (.NOT. ROTATE_PARTICLES%VALUE) THEN
            WRITE(*,'(A)') '**ERROR(MODE_STACK_FROM_PAR): when not rotating particles, one must output square images'
            CALL TERMINATE('Fatal error in MODE_STACK_FROM_PAR')
        ENDIF
    ENDIF


    !
    ! Are we given CTFFIND files or CTFTILT files?
    !
    USE_CTFTILT = .FALSE.
    IF (STRING_EQUAL(CTFTILT_FILES%VALUE,'none')) THEN
        IF (STRING_EQUAL(CTFFIND_FILES%VALUE,'none')) THEN
            WRITE(*,'(A)') '**ERROR(MODE_STACK_FROM_PAR): neither CTFFIND nor CTFTILT files were supplied'
            CALL TERMINATE('Fatal error in MODE_STACK_FROM_PAR')
        ELSE
            USE_CTFTILT = .FALSE.
        ENDIF
    ELSE
        USE_CTFTILT = .TRUE.
    ENDIF
    ! Diagnostic
    IF (.NOT. USE_CTFTILT) THEN
        PRINT '(A)', '**info(MODE_STACK_FROM_PAR): No CTFTILT files given, will use CTFFIND info'
    ENDIF


    ! PSF safety radius
    PSF_RADIUS_PIXELS = PSF_RADIUS / PIXEL_SIZE%VALUE

    ! The segment length should be at least twice the PSF radius
    IF (SEGMENT_LENGTH%VALUE .LT. 2.0*PSF_RADIUS) THEN
        WRITE(*,'(A,F0.3,A,F0.3,A)') '**WARNING(MODE_STACK_FROM_PAR): segment length (', SEGMENT_LENGTH%VALUE, &
                                     ' A) is less than twice the PSF radius (', PSF_RADIUS, ' A)'
    ENDIF

    ! Initialise the average helical parameters
    CALL HELICAL_SYMMETRY_INIT(HELIX_AVERAGE,RISE_PER_SUBUNIT%VALUE,TWIST_PER_SUBUNIT%VALUE,    &
                                            SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE,  &
                                            NUM_STARTS=NUM_HELICAL_STARTS%VALUE)

    ! Initialise the interpolation scheme
    CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,INTERPOL_FUNCTION%VALUE,WINDOW_FUNCTION%VALUE,WINDOW_WIDTH%VALUE)
	CALL INTERPOLATION_SCHEME_INIT(INTERPOL_LINEAR,'LINEAR','RECT',2)


    !
    ! Read in the frealix parameter file and intialise an array of filament objects
    !
    WRITE(*,'(/A)') '**info(MODE_STACK_FROM_PAR): reading input parameter file and initialising filaments'
    IF (USE_CTFTILT) THEN
        CALL FILAMENTS_FROM_FREALIX_PAR(    FILS,PAR_FILENAME%VALUE,MICROGRAPH_FILENAME_WILD%VALUE, &
                                            PIXEL_SIZE%VALUE,HELIX_AVERAGE,    &
                                            CTFTILT_FILENAME_WILD=CTFTILT_FILES%VALUE,                &
                                            WPS_ARE_CROSSOVERS=WPS_ARE_CROSSOVERS%VALUE,    &
                                            FAL_WPS_ARE_CROSSOVERS=FAL_WPS_ARE_CROSSOVERS%VALUE, &
                                            MAX_RADIUS=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE, &
                                            FRAGMENTED_PHI=FRAGMENTED_PHI%VALUE, &
                                            FRAGMENTED_SUBUNIT_NUM=FRAGMENTED_SUBUNIT_NUM%VALUE,    &
                                            CONSTANT_RISE_PER_SUBUNIT = CONSTANT_RISE_PER_SUBUNIT%VALUE,                    &
                                            CONSTANT_TWIST_PER_LENGTH = CONSTANT_TWIST%VALUE                               )
    ELSE
        CALL FILAMENTS_FROM_FREALIX_PAR(    FILS,PAR_FILENAME%VALUE,MICROGRAPH_FILENAME_WILD%VALUE, &
                                            PIXEL_SIZE%VALUE,HELIX_AVERAGE,    &
                                            CTFFIND_FILENAME_WILD=CTFFIND_FILES%VALUE,                &
                                            WPS_ARE_CROSSOVERS=WPS_ARE_CROSSOVERS%VALUE,    &
                                            FAL_WPS_ARE_CROSSOVERS=FAL_WPS_ARE_CROSSOVERS%VALUE, &
                                            MAX_RADIUS=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE,  &
                                            FRAGMENTED_PHI=FRAGMENTED_PHI%VALUE, &
                                            FRAGMENTED_SUBUNIT_NUM=FRAGMENTED_SUBUNIT_NUM%VALUE,    &
                                            CONSTANT_RISE_PER_SUBUNIT = CONSTANT_RISE_PER_SUBUNIT%VALUE,                    &
                                            CONSTANT_TWIST_PER_LENGTH = CONSTANT_TWIST%VALUE                               )
    ENDIF
    WRITE(*,'(A,I0,A/)') '**info(MODE_STACK_FROM_PAR): ', SIZE(FILS), ' filaments were initialised'



    !
    ! How many filaments & waypoints?
    !
    NUM_OF_WPS = 0
    DO CURRENT_FILAMENT=1,SIZE(FILS)
        NUM_OF_WPS = NUM_OF_WPS + FILS(CURRENT_FILAMENT)%NWP
    ENDDO
    WRITE(*,'(A,I0,A,I0,A)') '**info(STACK_FROM_PAR): ', SIZE(FILS), ' filaments were found in the file, for a total of ', &
                                NUM_OF_WPS, ' waypoints.'


    ! Array of single particles
    IF (ALLOCATED(SPS)) DEALLOCATE(SPS)

    !
    ! Loop over the filaments
    !
    LOC_IN_STACK = 0
    MICROGRAPH_IN_MEMORY = 0
    CURRENT_WP = 0
    DO CURRENT_FILAMENT=1, SIZE(FILS)

        CURRENT_FILAMENT_LENGTH =    FILS(CURRENT_FILAMENT)%T(FILS(CURRENT_FILAMENT)%NWP)  &
                                    -FILS(CURRENT_FILAMENT)%T(1)
        IF ( CURRENT_FILAMENT_LENGTH .LT. SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE ) THEN
            WRITE(*,'(A,I0,2(A,F0.3),A)') '**info(MODE_STACK_FROM_PAR): filament ', FILS(CURRENT_FILAMENT)%ID ,   &
                                    ' is shorter (', CURRENT_FILAMENT_LENGTH ,') than the segment size (', &
                                    SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE ,'), skipping'
            CYCLE
        ENDIF

        WRITE(*,'(A,I0,A,I0,A,I0,A)') '**info(MODE_STACK_FROM_PAR): processing filament ', FILS(CURRENT_FILAMENT)%ID, &
                                        ' (', CURRENT_FILAMENT, ' of ', SIZE(FILS), ')'

        ! Check whether we are dealing with a new micrograph
        IF ( CURRENT_FILAMENT .GT. 1 ) THEN
            IF (FILS(CURRENT_FILAMENT)%FILM%ID .NE. MICROGRAPH_IN_MEMORY) THEN
                NEW_MICROGRAPH = .TRUE.
            ELSE
                NEW_MICROGRAPH = .FALSE.
            ENDIF
        ELSE
            NEW_MICROGRAPH = .TRUE.
        ENDIF

        ! If needed, read micrograph in
        IF ( NEW_MICROGRAPH ) THEN
            CALL FILM_FILE%INIT(FILS(CURRENT_FILAMENT)%FILM%FILENAME)
            CALL FILM_FILE%PrintInfo()
            CALL MICROGRAPH%ReadFromDisk(FILM_FILE)
            CALL FILM_FILE%CLOSE()
            MICROGRAPH_IN_MEMORY = FILS(CURRENT_FILAMENT)%FILM%ID
            ! Apply degridding to micrograph - doesn't work for complex correction yet!
            !CALL CORRECT_3D(MICROGRAPH,INTERPOL_SCHEME%INTERPOLATION_INSTR_FUNCTION,INTERPOL_SCHEME%WINDOWING_INSTR_FUNCTION,INTERPOL_SCHEME%WINDOW_WIDTH)
        ENDIF

        ! We will use the assumption later on that the filament has T(1) = 0.0
        IF (FILS(CURRENT_FILAMENT)%T(1) .NE. 0.0D0) THEN
            WRITE(*,'(A)') '**ERROR(MODE_STACK_FROM_PAR): first T value is not 0.0'
            CALL TERMINATE('Fatal error in MODE_STACK_FROM_PAR')
        ENDIF

        ! Get waypoints where we want to extract segments
        IF (ALLOCATED(WPS)) DEALLOCATE(WPS)
        IF (PHI_ANGLE%SET_BY_USER) THEN
            IF (STEP_SIZE%VALUE .GT. 0.0) THEN
                CALL TERMINATE('Step size parameter not compatible with phi spec')
            ENDIF
            WRITE(*,'(A,F0.1,A)') '**info(MODE_STACK_FROM_PAR): will output one filament segment every ', PHI_PERIOD%VALUE, &
                                    ' degree rotation around the helical axis'
            TS = INTERPOL_SPLINE_FIND_ROOTS_OF_MULTIPLE(    FILS(CURRENT_FILAMENT)%PHI_SPLINE,  &
                                                            DBLE(PHI_PERIOD%VALUE),DBLE(PHI_ANGLE%VALUE))
            DO J=1,SIZE(TS)
                ! Check we are not too close to the ends of the filament
                IF (    TS(J) + (SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE/2.+1.) &
                                                        .GT. FILS(CURRENT_FILAMENT)%T(FILS(CURRENT_FILAMENT)%NWP) .OR.    &
                        TS(J) - (SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE/2.+1.) .LT. FILS(CURRENT_FILAMENT)%T(1)) THEN
                    WRITE(*,'(A,F0.5,A)') '**info(MODE_STACk_FROM_PAR): skipping T (', TS(J), &
                                            ') because it is too close to the ends of the filament'
                    CYCLE
                ENDIF
                CALL ADD_WAYPOINT_TO_PATH(WPS,FILS(CURRENT_FILAMENT),TS(J))
            ENDDO
        ELSE IF (ONE_IMAGE_PER_REPEAT%VALUE) THEN
            WRITE(*,'(A)') '**info(MODE_STACK_FROM_PAR): will output one image per helical repeat'
            CALL FILAMENT_GET_PATH(FILS(CURRENT_FILAMENT),WPS,STEP=1.0D0,STEP_IN_REPEATS=.TRUE.)
        ELSE IF (STEP_SIZE%SET_BY_USER) THEN
            WRITE(*,'(A,F0.1,A)') '**info(MODE_STACK_FROM_PAR): will output one filament segment every ', &
                                                                STEP_SIZE%VALUE, ' Angstroms'
            CALL FILAMENT_GET_PATH_PIXEL_STEPS( FILS(CURRENT_FILAMENT),REAL(STEP_SIZE%VALUE/PIXEL_SIZE%VALUE,KIND=8),WPS,    &
                                                START_T  = DBLE(SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE/2. + 1.0),   &
                                                FINISH_T = DBLE(FILS(CURRENT_FILAMENT)%T(FILS(CURRENT_FILAMENT)%NWP)    &
                                                            - SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE/2. - 1.0),  &
                                                STRICT_START_FINISH_T=.TRUE.)
        ELSE
            WRITE(*,'(A)') '**info(MODE_STACK_FROM_PAR): will output one filament segment for every input waypoint'
            ALLOCATE(WPS(SIZE(FILS(CURRENT_FILAMENT)%WP)))
            WPS = FILS(CURRENT_FILAMENT)%WP
        ENDIF

        ! Ensure that psi and theta are as smooth as possible, even if it
        ! Means breaking the convention of -90<=psi<=90.
        ! This is especially helpful when using Frealign
        CALL WAYPOINTS_PSI_THETA_SMOOTH(WPS)


        IF (DEBUG) THEN
            WRITE(*,'(A,F0.5,A,F0.5)') '**debug(MODE_STACK_FROM_PAR): this filament goes from ', FILS(CURRENT_FILAMENT)%T(1), &
                                        ' to ', FILS(CURRENT_FILAMENT)%T(FILS(CURRENT_FILAMENT)%NWP)
        ENDIF

        ! Loop over points
        DO J=1,SIZE(WPS)

            WP = WPS(J)

            BOX_SIZE = NINT(SEGMENT_LENGTH%VALUE/PIXEL_SIZE%VALUE)
            IF (IS_ODD(BOX_SIZE)) BOX_SIZE = BOX_SIZE+1

            CALL WP_TO_SP(  WP, SP_TMP, CTF=FILS(CURRENT_FILAMENT)%FILM%CTFTILT%CTF,    &
                            SP_IMAGE=BOX, SEGMENT_LENGTH=BOX_SIZE, &
                            FILM_IMAGE=MICROGRAPH, PIXEL_SIZE=PIXEL_SIZE%VALUE, &
                            APPLY_SHIFT_TO_IMAGE=CENTER_IMAGES%VALUE,    &
                            APPLY_CTF_TO_IMAGE=CTF_CORRECTION%VALUE,    &
                            CTF_FLIP_ONLY=CTF_FLIP_ONLY%VALUE, &
                            ROTATE_SP_IMAGE=ROTATE_PARTICLES%VALUE, &
                            UNBEND=UNBEND_SEGMENTS%VALUE, &
                            FIL=FILS(CURRENT_FILAMENT), &
                            IGNORE_SUBUNIT_NUM=IGNORE_SUBUNIT_NUM%VALUE, &
                            APPLY_RECT_MASK=APPLY_RECT_MASK%VALUE,  &
                            TOO_CLOSE_TO_EDGE=SP_NEAR_EDGE, &
                            SQUARE_IMAGE=SQUARE_OUTPUT_IMAGES%VALUE )

            IF(DEBUG) WRITE(*,'(A,2(I0,1X))') '**debug(MODE_STACK_FROM_PAR): segment image dimensions: ', &
                            BOX%logical_dimensions(1:2)

            IF (SP_NEAR_EDGE) THEN
                !
                WRITE(*,'(A,I0,A)') '**info(MODE_STACK_FROM_PAR): skipping waypoint ', J, ' because it is too close to the edge'
            ELSE
                ! We'll add a new image to the output stack
                LOC_IN_STACK = LOC_IN_STACK + 1

                ! Report progress
                WRITE(*,'(A,I0,A,I0,A)') '**info(MODE_STACK_FROM_PAR): processing output location ', LOC_IN_STACK, &
                                            ' (filament ', FILS(CURRENT_FILAMENT)%ID,')'
                CALL WP_PRINT_INFO(WP)

                ! Grow the array of single particles so that it can accomodate this filament's waypoints/single particles
                IF (ALLOCATED(SPS)) THEN
                    SPS_TMP = SPS
                    DEALLOCATE(SPS)
                    ALLOCATE(SPS(LOC_IN_STACK))
                    SPS(1:SIZE(SPS_TMP)) = SPS_TMP
                ELSE
                    ALLOCATE(SPS(1))
                ENDIF

                ! If we're on the first segment, we'll need to open files to write to, prepare masks etc.
                IF (LOC_IN_STACK .EQ. 1) THEN
                    ! Open files
                    IF (EXTRACT_SEGMENTS%VALUE) THEN
                        CALL SEGMENTS_FILE%INIT(STACK_FILENAME%VALUE,  &
                                                DIM_X=BOX%logical_dimensions(1), &
                                                DIM_Y=BOX%logical_dimensions(2), &
                                                DIM_Z=1,PIXEL_SIZE=PIXEL_SIZE%VALUE)
                    ENDIF
                    IF (AMPL_SPECTRA%VALUE) THEN
                        CALL AMPL_FILE%INIT(AMPL_STACK_FILENAME%VALUE, &
                                            DIM_X=BOX%logical_dimensions(1), &
                                            DIM_Y=BOX%logical_dimensions(2), &
                                            DIM_Z=1,PIXEL_SIZE=PIXEL_SIZE%VALUE)
                    ENDIF
                    IF (EXTRACT_PROJECTIONS%VALUE) THEN
                        CALL PROJECTIONS_FILE%INIT(PROJ_FILENAME%VALUE,    &
                                                    DIM_X=BOX%logical_dimensions(1), &
                                                    DIM_Y=BOX%logical_dimensions(2), &
                                                    DIM_Z=1,PIXEL_SIZE=PIXEL_SIZE%VALUE)
                    ENDIF
                    IF (AMPL_SPECTRA_AVERAGE%VALUE) THEN
                        CALL AMPL_AVE_FILE%INIT(AMPL_AVE_FILENAME%VALUE,   &
                                                DIM_X=BOX%logical_dimensions(1), &
                                                DIM_Y=BOX%logical_dimensions(2), &
                                                DIM_Z=1,PIXEL_SIZE=PIXEL_SIZE%VALUE)
                    ENDIF
                    ! If we're going to make projections or calculate masks, we need to read
                    ! the 3D reconstruction in & keep its shifted FT in memory
                    IF (EXTRACT_PROJECTIONS%VALUE .OR. APPLY_ENV_MASK%VALUE) THEN
                        CALL REC_FILE%INIT(RECONSTRUCTION_FILENAME%VALUE)
                        CALL REC_FILE%PrintInfo()
                        CALL REC%ReadFromDisk(REC_FILE,read_volume_not_slice=.true.)
                        CALL REC_FILE%CLOSE()
                        WRITE(*,'(A)') '**info(MODE_STACK_FROM_PAR): resizing 3D reconstruction to match segment width'
                        CALL RESIZE(REC,[BOX%logical_dimensions(1),BOX%logical_dimensions(1),BOX%logical_dimensions(1)])
                        WRITE(*,'(A)') '**info(MODE_STACK_FROM_PAR): applying helical symmetry to input 3D reconstruction'
                        CALL IMAGE_IMPOSE_HELICAL_SYMMETRY_NEW( REC,HELIX_AVERAGE, INTERPOL_SCHEME, &
                                                                MAX_RADIUS=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE)
                        WRITE(*,'(A)') '**info(MODE_STACK_FROM_PAR): applying mask to input 3D reconstruction'
                        CALL IMAGE_HELIX_MASK( REC, HELIX_AVERAGE,ASYM_UNIT_MW%VALUE,PIXEL_SIZE%VALUE,HELIX_MAX_RADIUS%VALUE,   &
                                                HARD_MASK=REC_MASK)
                    ENDIF

                    !
                    IF (EXTRACT_PROJECTIONS%VALUE) THEN
                        PROJ%logical_dimensions = BOX%logical_dimensions
                        CALL IMAGE_ALLOCATE(PROJ)
                        CALL IMAGE_PROJECT_LONG_HELIX(  REC,PROJ,HELIX_AVERAGE,PHI_ANGLE%VALUE,   &
                                                        INTERPOL_SCHEME,    &
                                                        HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE)
                    ENDIF

                    ! Prepare the envelope mask
                    IF (APPLY_ENV_MASK%VALUE) THEN
                        PROJ_MASK%logical_dimensions = BOX%logical_dimensions
                        CALL IMAGE_ALLOCATE(PROJ_MASK)
                        CALL IMAGE_PROJECT_LONG_HELIX(  REC_MASK,PROJ_MASK,HELIX_AVERAGE,PHI_ANGLE%VALUE,   &
                                                        INTERPOL_LINEAR,    &
                                                        HELIX_MAX_RAD=HELIX_MAX_RADIUS%VALUE/PIXEL_SIZE%VALUE)

                        IF (DEBUG) CALL WRITE_TO_DISK(REC_MASK,'dbg_mask_3d.mrc')
                        IF (DEBUG) CALL WRITE_TO_DISK(PROJ_MASK,'dbg_mask_beforemask.mrc')
                        ! Binarise & soften the mask
                        CALL BINARISE(PROJ_MASK,10.*PROJ_MASK%logical_dimensions(1)*TINY(1.))
                        PROJ_MASK%real_values(:,1:7,1) = 0.0
                        PROJ_MASK%real_values(:,PROJ_MASK%logical_dimensions(2)-6:PROJ_MASK%logical_dimensions(2),1) = 0.0
                        CALL MASK_ADD_COSINE_FALLOFF(PROJ_MASK,5.0)
                        !
                        IF (DEBUG) CALL WRITE_TO_DISK(PROJ_MASK,'dbg_mask.mrc')
                    ENDIF
                ENDIF

                ! The film_num property of the single particle is used to keep track of the filament number
                SP_TMP%FILM_NUM = FILS(CURRENT_FILAMENT)%ID

                ! Add this single particle to the output array
                SPS(LOC_IN_STACK) = SP_TMP

                ! Apply envelope mask
                IF (APPLY_ENV_MASK%VALUE) THEN
                    CALL MULTIPLY_IMAGES(BOX,PROJ_MASK,BOX)
                ENDIF

                ! Accumulate average amplitude
                CALL AMPL(BOX,BOX_AMPL)
                IF (LOC_IN_STACK .EQ. 1) CALL IMAGE_ALLOCATE(AVE_AMPL,MOULD=BOX_AMPL)
                AVE_AMPL%real_values = AVE_AMPL%real_values + BOX_AMPL%real_values

                ! Write particle out
                IF (EXTRACT_SEGMENTS%VALUE) THEN
                    call box%WriteToDisk(SEGMENTS_FILE,wanted_location=LOC_IN_STACK)
                ENDIF
                IF (AMPL_SPECTRA%VALUE) THEN
                    call box_ampl%WriteToDisk(AMPL_FILE,wanted_location=loc_in_stack)
                ENDIF
            ENDIF    ! End of test for SP near edge of micrograph
        ENDDO   ! End loop over waypoints in the filament
    ENDDO ! End of loop over the filaments

    IF (.NOT. STRING_EQUAL(OUTPUT_FREALIGN_PAR_FILENAME%VALUE,'none')) THEN
        CALL SP_TO_FREALIGN_PAR(SPS,OUTPUT_FREALIGN_PAR_FILENAME%VALUE,MAGNIFICATION%VALUE,PIXEL_SIZE%VALUE)
    ENDIF

    ! Average amplitude spectra
    IF (AMPL_SPECTRA_AVERAGE%VALUE) THEN
        AVE_AMPL%real_values = AVE_AMPL%real_values / REAL(LOC_IN_STACK)
        call ave_ampl%WriteToDisk(ampl_ave_file)
        CALL AMPL_AVE_FILE%PrintInfo()
        CALL AMPL_AVE_FILE%CLOSE()
    ENDIF

    ! Close output files
    IF (EXTRACT_SEGMENTS%VALUE) THEN
        CALL SEGMENTS_FILE%PrintInfo()
        CALL SEGMENTS_FILE%CLOSE()
    ENDIF
    IF (EXTRACT_PROJECTIONS%VALUE) THEN
        CALL PROJECTIONS_FILE%PrintInfo()
        CALL PROJECTIONS_FILE%CLOSE()
    ENDIF
    IF (AMPL_SPECTRA%VALUE) THEN
        CALL AMPL_FILE%PrintInfo()
        CALL AMPL_FILE%CLOSE()
    ENDIF

END SUBROUTINE MODE_STACK_FROM_PAR_SLAVE

END MODULE MODE_STACK_FROM_PAR
