!>  \brief  Mode to create a synthetic micrograph from a helical 3D model density
MODULE MODE_SYNTH_MICROGRAPH
    ! Modules

    USE MODES
    USE RTPS

    IMPLICIT NONE
    TYPE(MODE) :: SYNTH_MICROGRAPH

    TYPE(RTP_CHAR), TARGET  ::  MODEL_3D_FILENAME  =               &
        RTP_CHAR(   VALUE='model3d.mrc',                  &
                    KEYWORD='MODEL_3D_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  INPUT_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='waypoints.ali',                  &
                    KEYWORD='INPUT_PAR_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFTILT_FILENAMES  =               &
        RTP_CHAR(   VALUE='none',                  &
                    KEYWORD='CTFTILT_FILENAMES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILM_FILENAMES  =               &
        RTP_CHAR(   VALUE='synth_micrograph_?????.mrc',                  &
                    KEYWORD='OUTPUT_FILM_FILENAMES',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_PAR_FILENAME  =               &
        RTP_CHAR(   VALUE='waypoints_out.ali',                  &
                    KEYWORD='OUTPUT_PAR_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  TWIST_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT',           &
                    DESCRIPTION='In degrees')


    TYPE(RTP_REAL), TARGET  ::  RISE_PER_SUBUNIT  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='RISE_PER_SUBUNIT',           &
                    DESCRIPTION='Angstroms')


    TYPE(RTP_REAL), TARGET  ::  HELIX_MAX_RADIUS  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='HELIX_MAX_RADIUS',           &
                    DESCRIPTION='Maximum radius (Angstroms)')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_AXIAL  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_AXIAL',           &
                    DESCRIPTION='')


    TYPE(RTP_INTG), TARGET  ::  SYMMETRY_PERP  =               &
        RTP_INTG(   VALUE=1,                  &
                    KEYWORD='SYMMETRY_PERP',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  ASYM_UNIT_MW  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ASYM_UNIT_MW',           &
                    DESCRIPTION='')


    TYPE(RTP_LOGI), TARGET  ::  STRAIGHTEN_FILAMENT  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='STRAIGHTEN_FILAMENT',           &
                    DESCRIPTION='Whether the filaments should be straightened')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='Pixel size (A)')


    TYPE(RTP_REAL), TARGET  ::  TARGET_RESOLUTION  =               &
        RTP_REAL(   VALUE=10.,                  &
                    KEYWORD='TARGET_RESOLUTION',           &
                    DESCRIPTION='Target resolution (A)')


    TYPE(RTP_REAL), TARGET  ::  SNR  =               &
        RTP_REAL(   VALUE=0.5,                  &
                    KEYWORD='SNR',           &
                    DESCRIPTION='Desired SNR')

    TYPE(RTP_LOGI), TARGET  ::  INDIVIDUAL_SUBUNITS  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='INDIVIDUAL_SUBUNITS',           &
                    DESCRIPTION='Project individual helical subunits rather than segments')

    TYPE(RTP_REAL), TARGET  ::  TWIST_PER_SUBUNIT_SIGMA  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='TWIST_PER_SUBUNIT_SIGMA',           &
                    DESCRIPTION='Standard deviation of the twist per subunit, in degrees. Use this '//&
                                'to add cumulative rotational disorder to the helix.')

    TYPE(RTP_LOGI), TARGET  ::  REAL_SPACE_PROJ  =               &
        RTP_LOGI(   VALUE=.FALSE.,                  &
                    KEYWORD='REAL_SPACE_PROJ',           &
                    DESCRIPTION='Use real-space projection')

    TYPE(RTP_REAL), TARGET  ::  FORCE_WP_TO_WP_DISTANCE  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='FORCE_WP_TO_WP_DISTANCE',           &
                    DESCRIPTION='Each filament''s waypoints will be recalculated so that they are spaced'//&
                                ' by this distance (in Angstroms). If set to 0.0, the waypoints will be left as in input.')

    TYPE(RTP_LOGI), TARGET  ::  APPLY_CTF  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='APPLY_CTF',           &
                    DESCRIPTION='Apply CTF to the synthetic micrograph')

    TYPE(RTP_LOGI), TARGET  ::  NORMALISE_MODEL_3D  =               &
        RTP_LOGI(   VALUE=.TRUE.,                  &
                    KEYWORD='NORMALISE_MODEL_3D',           &
                    DESCRIPTION='Normalise the variance of the input model 3D to 1.0 (its average is unchanged)')
CONTAINS

SUBROUTINE MODE_SYNTH_MICROGRAPH_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL SYNTH_MICROGRAPH%INIT(COMMAND='SYNTH_MICROGRAPH',SUBROUTINE=MODE_SYNTH_MICROGRAPH_SLAVE,   &
                                SHORT_DESCRIPTION='Generate a simulated micrograph given a model 3D density')

    ! Add RTPs to the mode
    CALL SYNTH_MICROGRAPH%ADD_RTP(MODEL_3D_FILENAME)
    CALL SYNTH_MICROGRAPH%ADD_RTP(INPUT_PAR_FILENAME)
    CALL SYNTH_MICROGRAPH%ADD_RTP(CTFTILT_FILENAMES)
    CALL SYNTH_MICROGRAPH%ADD_RTP(OUTPUT_FILM_FILENAMES)
    CALL SYNTH_MICROGRAPH%ADD_RTP(OUTPUT_PAR_FILENAME)
    CALL SYNTH_MICROGRAPH%ADD_RTP(TWIST_PER_SUBUNIT)
    CALL SYNTH_MICROGRAPH%ADD_RTP(RISE_PER_SUBUNIT)
    CALL SYNTH_MICROGRAPH%ADD_RTP(HELIX_MAX_RADIUS)
    CALL SYNTH_MICROGRAPH%ADD_RTP(SYMMETRY_AXIAL)
    CALL SYNTH_MICROGRAPH%ADD_RTP(SYMMETRY_PERP)
    CALL SYNTH_MICROGRAPH%ADD_RTP(ASYM_UNIT_MW)
    CALL SYNTH_MICROGRAPH%ADD_RTP(STRAIGHTEN_FILAMENT)
    CALL SYNTH_MICROGRAPH%ADD_RTP(PIXEL_SIZE)
    CALL SYNTH_MICROGRAPH%ADD_RTP(TARGET_RESOLUTION)
    CALL SYNTH_MICROGRAPH%ADD_RTP(SNR)
    CALL SYNTH_MICROGRAPH%ADD_RTP(INDIVIDUAL_SUBUNITS)
    CALL SYNTH_MICROGRAPH%ADD_RTP(TWIST_PER_SUBUNIT_SIGMA)
    CALL SYNTH_MICROGRAPH%ADD_RTP(REAL_SPACE_PROJ)
    CALL SYNTH_MICROGRAPH%ADD_RTP(FORCE_WP_TO_WP_DISTANCE)
    CALL SYNTH_MICROGRAPH%ADD_RTP(APPLY_CTF)
    CALL SYNTH_MICROGRAPH%ADD_RTP(NORMALISE_MODEL_3D)

    ! Run the mode
    CALL SYNTH_MICROGRAPH%RUN()
END SUBROUTINE MODE_SYNTH_MICROGRAPH_INIT_AND_RUN


SUBROUTINE MODE_SYNTH_MICROGRAPH_SLAVE
    USE UI, ONLY : INTERVAL_PRINT
    USE FILAMENTS_MODEL_UPDATE, ONLY : FILAMENT_UPDATE_MODEL
    USE FILAMENTS_PATH, ONLY : FILAMENT_GET_PATH
    USE FILAMENTS_GEOMETRY, ONLY : FILAMENT_AVERAGE_DEFOCUS
    USE FMCS_EVALUATION, ONLY : FMC_EVAL, FMC_MODEL3D_PREP
    USE FILAMENTS_MODEL_BASE, ONLY : FILAMENT_PRINT_INFO
    USE FILAMENTS_MODEL_MODIFY, ONLY : FILAMENT_MOVE_WPS_TO_NEAREST_SUBUNIT, FILAMENT_STRAIGHTEN
    USE COOS, ONLY : WAYPOINTS_ADD_DISORDER, WAYPOINTS_PRINT_INFO
    !USE FILAMENTS
    USE FREALIX_PARAMETERS
    USE FMCS
    USE FMCS_BASE
    USE IMAGES_COMPARISON
    USE PARSERS
    USE RUNTIME_PARAMETERS
    USE IMAGES_HELICES
    USE INTERPOLATION
    USE IMAGES_THREED
    USE CTFS
    USE IMAGES_CTF
    USE MASKS
    USE FILAMENTS_BASE
    use ImageFiles
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE), TARGET                 ::  MODEL3DVOL
    TYPE(IMAGE), TARGET                 ::  MICROGRAPH
    TYPE(HELICAL_SYMMETRY)              ::  HELIX
    TYPE(INTERPOLATION_SCHEME)          ::  INTERPOL
    TYPE(CTF_PARAMETERS)                ::  LOCAL_CTF
    LOGICAL,                PARAMETER   ::  DEBUG   =   .TRUE.
    TYPE(FILAMENT),     ALLOCATABLE     ::  FILS(:)
    INTEGER                             ::  CURRENT_FIL
    !INTEGER,    PARAMETER               ::  MICROGRAPH_MARGIN = 600 ! In Angstroms
    TYPE(FILAMENT_MODEL_COMPARISON), TARGET     ::  FMC
    LOGICAL,    PARAMETER               ::  REFINE_PARAMS(7) = .FALSE.
    TYPE(WAYPOINT), ALLOCATABLE         ::  PATH(:)
    TYPE(WAYPOINT)                      ::  WP_PHASED
    INTEGER                             ::  CURRENT_WP
    REAL(KIND=8)                        ::  RCOO(3)
    INTEGER                             ::  ICOO(3)
    INTEGER                             ::  PROJ_DIM(3)
    REAL                                ::  RECT_DIM(2)
    REAL,           PARAMETER           ::  SOFT_EDGE = DEFAULT_FALLOFF_PIXELS
    REAL                                ::  DUMMY
    TYPE(IMAGEFILE)                     ::  FILM_FILE
    CHARACTER(LEN=FILENAME_LEN)         ::  FILM_FILENAME
    REAL(KIND=8)                        ::  PROJ_VAR
    REAL(KIND=8)                        ::  Z_SHIFT
    !
    REAL                    ::  AVERAGE_DEFOCUS
    TYPE(CTF_PARAMETERS)    ::  AVERAGE_CTF
    REAL(KIND=8)            ::  PITCH
    INTEGER                 ::  NEW_DIM(3)
    LOGICAL,    PARAMETER   ::  MANUAL_PADDING = .FALSE.
    LOGICAL,    PARAMETER   ::  FORCE_SINGLE_CTF_PER_MICROGRAPH = .TRUE.
    LOGICAL,    PARAMETER   ::  COMPUTE_SCORE = .TRUE.
    LOGICAL,    PARAMETER   ::  ATTEMPT_NOISE_MODELLING = .FALSE.
    REAL,       PARAMETER   ::  INCIDENT_DOSE_PER_PIXEL  =   20.0    !   Average number of electron


    ! Start work


    ! Notify user if APPLY_CTF is false
    IF (.NOT. APPLY_CTF%VALUE) THEN
        WRITE(*,'(A)') '**warning(MODE_SYNTH_MICROGRAPH): CTF will not be applied, debug flag activated'
    ENDIF

    ! Set up interpolation
    CALL INTERPOLATION_SCHEME_INIT(INTERPOL,'LINEAR','RECT',2)

    ! Convert the maximum radius to pixels
    IF (HELIX_MAX_RADIUS%VALUE .NE. 0.0) THEN
        HELIX_MAX_RADIUS%VALUE = HELIX_MAX_RADIUS%VALUE / PIXEL_SIZE%VALUE
    ENDIF

    ! Setup the helical symmetry operator
    CALL HELICAL_SYMMETRY_INIT( HELIX,RISE_PER_SUBUNIT%VALUE,TWIST_PER_SUBUNIT%VALUE,    &
                                SYMMETRY_AXIAL%VALUE,SYMMETRY_PERP%VALUE,PIXEL_SIZE%VALUE)
    PITCH = HELIX_PITCH(HELIX)

    ! Setup the filament object
    CALL FILAMENTS_FROM_FREALIX_PAR(    FILS,INPUT_PAR_FILENAME%VALUE,OUTPUT_FILM_FILENAMES%VALUE,      &
                                        PIXEL_SIZE%VALUE,HELIX,     &
                                        CTFTILT_FILENAME_WILD=CTFTILT_FILENAMES%VALUE,                            &
                                        MAX_RADIUS=HELIX_MAX_RADIUS%VALUE,    &
                                        FILMS_EXIST=.FALSE., SHIFT_FILS_AND_SET_FILMS_DIM=.TRUE.  )

    ! For initial development, we only allow one filament
    IF (SIZE(FILS) .GT. 1) THEN
        WRITE(*,'(A)') '**ERROR(MODE_SYNTH_MICROGRAPH): multiple filaments are not yet supported'
        CALL TERMINATE('Feature not implemented yet')
    ENDIF

    ! Read in the 3D model
    call model3dvol%ReadFromDisk(model_3d_filename%value,read_volume_not_slice=.true.)

    ! Normalise the 3D model
    if (normalise_model_3d%value) then
        call model3dvol%NormaliseVariance()
    endif

    ! Testing: cylindrical average
    !WRITE(*,'(A)') '**debug(MODE_SYNTH_MICROGRAPH): cylindical average of model'
    !CALL IMAGE_CYL_AVERAGE(MODEL3DVOL)


    IF (INDIVIDUAL_SUBUNITS%VALUE) THEN
        WRITE(*,'(A)') '**info(MODE_SYNTH_MICROGAPH): individual helical subunits will be projected'
    ELSE
        IF (TWIST_PER_SUBUNIT_SIGMA%VALUE .NE. 0.0) THEN
            WRITE(*,'(2A)') '**ERROR(MODE_SYNTH_MICROGRAPH): cannot vary the twist per subunit unless',  &
                            ' individual subunits are projected'
            CALL TERMINATE('Fatal error in MODE_SYNTH_MICROGRAPH')
        ENDIF
    ENDIF

    ! Loop over filaments
    DO CURRENT_FIL=1,SIZE(FILS)

        ! Initialise variables for noise stats
        PROJ_VAR = 0.0D0


        ! Print out filament info
        WRITE(*,'(A)') '**debug(MODE_SYNTH_MICROGRAPH): shifted filament info'
        CALL FILAMENT_PRINT_INFO(FILS(CURRENT_FIL))

        ! Straighten if asked for it
        IF (STRAIGHTEN_FILAMENT%VALUE) THEN
            WRITE(*,'(A)') '**info(MODE_SYNTH_MICROGRAPH): straightening filament'
            CALL FILAMENT_STRAIGHTEN(FILS(CURRENT_FIL))
        ENDIF

        IF (FORCE_WP_TO_WP_DISTANCE%VALUE .GT. 0.0) THEN
            WRITE(*,'(A,F0.3)') '**info(MODE_SYTNH_MICROGAPH): resampling waypoints with spacing (A): ',    &
                                 FORCE_WP_TO_WP_DISTANCE%VALUE
            CALL FILAMENT_RESAMPLE_WAYPOINTS(FILS(CURRENT_FIL),REAL(FORCE_WP_TO_WP_DISTANCE%VALUE/PIXEL_SIZE%VALUE,KIND=8))
            !PRINT *, '**DEBUG(): RESAMPLING AT EVERY REPEAT'
            !CALL FILAMENT_RESAMPLE_WAYPOINTS(FILS(CURRENT_FIL),1.0,STEP_IN_REPEATS=.TRUE.)
        ENDIF

        ! Move waypoints to nearest subunit
        WRITE(*,'(A)') '**info(MODE_SYTNH_MICROGAPH): moving waypoints to nearest subunits'
        CALL FILAMENT_MOVE_WPS_TO_NEAREST_SUBUNIT(FILS(CURRENT_FIL))



        ! Prepare the synthetic micrograph image
        MICROGRAPH%logical_dimensions(1:2) = FILS(CURRENT_FIL)%FILM%logical_dimensions(1:2)
        MICROGRAPH%logical_dimensions(3)   = 1
        WRITE(*,'(A,I0,3A,2(I0,1X))')   '**info(MODE_SYNTH_MICROGRAPH): filament ', FILS(CURRENT_FIL)%ID, &
                                        ' will be on micrograph ', TRIM(ADJUSTL(FILS(CURRENT_FIL)%FILM%FILENAME)), &
                                        ', with dimensions: ', MICROGRAPH%logical_dimensions(1:2)
        CALL IMAGE_ALLOCATE(MICROGRAPH)

        ! Print out filament info
        CALL FILAMENT_PRINT_INFO(FILS(CURRENT_FIL))

        ! Initialise the FMC object
        CALL FMC_INIT(  FMC,FILS(CURRENT_FIL),MICROGRAPH,MODEL3DVOL,HELIX,    &
                        TARGET_RESOLUTION%VALUE,INTERPOL,REAL_SPACE_PROJ=INDIVIDUAL_SUBUNITS%VALUE .OR. REAL_SPACE_PROJ%VALUE,&
                        HIGH_RESOLUTION_CUTOFF=MAX(7.0,TARGET_RESOLUTION%VALUE),&
                        ASYM_UNIT_MW=ASYM_UNIT_MW%VALUE    &
                        )
        CALL FMC_PRINT_INFO(FMC)

        ! Prepare the 3D model
        WRITE(*,'(A)') '**info(MODE_SYTNH_MICROGAPH): preparing 3D model'
        CALL FMC_MODEL3D_PREP(FMC)

        ! If we're doing real space projection, let's pad the model 3D in Fourier space
        IF (FMC%REAL_SPACE_PROJ .AND. MANUAL_PADDING) THEN
            WRITE(*,'(A)') '**info(MODE_SYNTH_MICROGRAPH): padding in Fourier space'
            NEW_DIM = FMC%MODEL3DVOL_PRIVATE%logical_dimensions*2
            IF (FMC%MODEL3DVOL_PRIVATE%is_in_real_space) CALL FWD_FT(FMC%MODEL3DVOL_PRIVATE)
            CALL PAD(FMC%MODEL3DVOL_PRIVATE,NEW_DIM)
            IF (.not. FMC%MODEL3DVOL_PRIVATE%is_in_real_space) CALL BWD_FT(FMC%MODEL3DVOL_PRIVATE)
        ENDIF

        ! Dump a file with all the subunits
        !IF (DEBUG) THEN
        !    PATH = FILAMENT_GET_PATH(FMC%FIL,1.0,STEP_IN_REPEATS=.TRUE.)
        !    CALL WAYPOINTS_TO_FREALIX_ALI(PATH,'repeats.ali',PIXEL_SIZE=PIXEL_SIZE%VALUE,FIL_ID=FMC%FIL%ID,FILM_NUM=FMC%FIL%FILM%ID)
        !ENDIF

        ! Get a path along the filament at regular intervals
        DUMMY = FMC%STEP
        FMC%STEP = FMC%STEP
        IF (INDIVIDUAL_SUBUNITS%VALUE) THEN
            CALL FILAMENT_GET_PATH(FMC%FIL,PATH,1.0D0,STEP_IN_REPEATS=.TRUE.)
        ELSE
            CALL FILAMENT_GET_PATH(FMC%FIL,PATH,FMC%STEP,STEP_IN_PIXELS=.TRUE.)
        ENDIF



        ! Perturb the path
        IF (TWIST_PER_SUBUNIT_SIGMA%VALUE .GT. 0.0) THEN
            WRITE(*,'(A,F0.3,A)')   '**info(MODE_SYNTH_MICROGRAPH: perturbing twist-per-subunit by std dev ', &
                                    TWIST_PER_SUBUNIT_SIGMA%VALUE, ' degrees'
            CALL WAYPOINTS_ADD_DISORDER(PATH,   &
                                        CUMULATIVE_DISORDER_STD    =(/0.,0.,0.,TWIST_PER_SUBUNIT_SIGMA%VALUE,0.,0.,0./),   &
                                        NON_CUMULATIVE_DISORDER_STD=(/0.,0.,0.,0.,                           0.,0.,0./))
        ENDIF

        WRITE(*,'(A)') '**debug(MODE_SYNTH_MICROGRAPH): path waypoints follow'
        CALL WAYPOINTS_PRINT_INFO(PATH)

        ! Dimensions for the projection
        PROJ_DIM(1:2) = FMC%BOX_SIZE
        PROJ_DIM(3)   = 1



        ! Loop over the path waypoints
        DO CURRENT_WP=1,SIZE(PATH)

            !
            IF (INTERVAL_PRINT(CURRENT_WP,SIZE(PATH))) THEN
                WRITE(*,'(A,I0,A,I0)') '**info(MODE_SYNTH_MICROGRAPH): on projection ', CURRENT_WP, ' of ', SIZE(PATH)
            ENDIF

            ! Do the phasing (because probably not integer subunit_num)
            WP_PHASED = PATH(CURRENT_WP)
            !CALL FILAMENT_APPLY_PHASING_TO_WP_OLD(FMC%FIL,WP_PHASED,NEG=.FALSE.)
            ! Work out any changes due to the non-integer value of subunit_num
            ! (we may have to change phi and z)
            CALL FILAMENT_MODEL_PHASING_FROM_WP(FILS(CURRENT_FIL),WP_PHASED,MODEL_Z_SHIFT=Z_SHIFT,LOCAL_H_SYM=.TRUE.)

            ! Integer coordinates & remainders
            ICOO(1:2) = NINT(WP_PHASED%COO(1:2))
            ICOO(3) = 1
            RCOO = WP_PHASED%COO - ICOO

            ! Local CTF parameters
            LOCAL_CTF = FMC%FIL%FILM%CTFTILT%CTF
            CALL CTF_ANG_TO_PIX(LOCAL_CTF,FMC%FIL%FILM%PIXEL_SIZE)
            CALL CTF_AT_Z(LOCAL_CTF,WP_PHASED%COO(3))
            CALL CTF_TO_IMG(LOCAL_CTF,FMC%BOX_CTF(1))

            ! When projecting individual subunits, we can't apply the CTF until all subunits have been projected
            CALL PROJECTION_PREPARE(FMC%MODEL3DVOL_PRIVATE,FMC%PROJ(1),PROJ_DIM,WP_PHASED%EULER, &
                                    FMC%BOX_CTF(1),HELIX_MAX_RADIUS%VALUE, &
                                    SHIFT_X=RCOO(1),SHIFT_Y=RCOO(2), &
                                    VOL_SHIFT_Z = Z_SHIFT, &
                                    INTERPOL=FMC%INTERPOL,  &
                                    WIN=FMC%INTERP_WIN_3D(1), &
                                    CTF_SQUARED=.FALSE.,  &
                                    ENV_MASK=FMC%MASK_ENV(1),    &
                                    MASK_WITH_ZEROS=.TRUE.,APPLY_DISC_MASK=.FALSE.,&
                                    REAL_SPACE_PROJ=FMC%REAL_SPACE_PROJ,&
                                    APPLY_CTF=          .NOT. INDIVIDUAL_SUBUNITS%VALUE &
                                                .AND.   .NOT. FORCE_SINGLE_CTF_PER_MICROGRAPH &
                                                .AND. APPLY_CTF%VALUE, &
                                    HELICAL_SUBUNIT_ONLY = INDIVIDUAL_SUBUNITS%VALUE,&
                                    HELIX=HELIX,PITCH=PITCH)

            ! If we were doing real-space projection, we need to undo the effect of the Fourier-space padding
            IF (FMC%REAL_SPACE_PROJ .AND. MANUAL_PADDING) THEN
                IF (FMC%PROJ(1)%is_in_real_space) CALL FWD_FT(FMC%PROJ(1))
                NEW_DIM = FMC%MODEL3DVOL_PRIVATE%logical_dimensions/2
                CALL CROP(FMC%MODEL3DVOL_PRIVATE,NEW_DIM)
                IF (.not. FMC%PROJ(1)%is_in_real_space) CALL BWD_FT(FMC%PROJ(1))
            ENDIF

            ! Apply rectangular mask (not necessary when projecting individual subunits)
            IF (.NOT. INDIVIDUAL_SUBUNITS%VALUE) THEN
                IF (.not. FMC%PROJ(1)%is_in_real_space) CALL BWD_FT(FMC%PROJ(1))
                RECT_DIM = (/ MIN(REAL(FMC%STEP,KIND=4),2.*HELIX_MAX_RADIUS%VALUE)+ &
                            SOFT_EDGE,(HELIX_MAX_RADIUS%VALUE+SOFT_EDGE)*2./)
                RECT_DIM = (/ REAL(FMC%STEP,KIND=4)+SOFT_EDGE,(HELIX_MAX_RADIUS%VALUE+SOFT_EDGE)*2./)
                CALL COSINE_RECT(   FMC%PROJ(1),RECT_DIM,REAL(WP_PHASED%EULER%PSI), &
                                    FALLOFF=SOFT_EDGE,WIPE_INPUT=.FALSE.,MASK_WITH_AVERAGE=.FALSE.,&
                                    SHIFT_X=REAL(RCOO(1)),SHIFT_Y=REAL(RCOO(2)))
            ENDIF

            ! Keep track of the variance
            PROJ_VAR = PROJ_VAR + VARIANCE_REAL(FMC%PROJ(1))

            ! Add the projection to the micrograph
            CALL WINDOW_IMG(MICROGRAPH,FMC%PROJ(1),ICOO,WINDOW_TO_IMG=.TRUE.)

        ENDDO! End of loop over path waypoints


        ! If we're doing real space projection, let's crop the model 3D in Fourier space
        IF (FMC%REAL_SPACE_PROJ .AND. MANUAL_PADDING) THEN
            WRITE(*,'(A)') '**info(MODE_SYNTH_MICROGRAPH): cropping in Fourier space'
            NEW_DIM = FMC%MODEL3DVOL_PRIVATE%logical_dimensions/2
            IF (FMC%MODEL3DVOL_PRIVATE%is_in_real_space) CALL FWD_FT(FMC%MODEL3DVOL_PRIVATE)
            CALL CROP(FMC%MODEL3DVOL_PRIVATE,NEW_DIM)
            IF (.not. FMC%MODEL3DVOL_PRIVATE%is_in_real_space) CALL BWD_FT(FMC%MODEL3DVOL_PRIVATE)
        ENDIF

        ! Debug dump
        if (debug) then
            if (.not. micrograph%is_in_real_space) call micrograph%BackwardFT()
            call micrograph%WriteToDisk('dbg_micrograph_before_ctf.mrc')
        endif

        ! If we projected individual projections, we need to apply the CTF now
        ! As a first approximation, we'll apply a single CTF to the whole micrograph
        IF ((INDIVIDUAL_SUBUNITS%VALUE .OR. FORCE_SINGLE_CTF_PER_MICROGRAPH) .AND. APPLY_CTF%VALUE) THEN
            AVERAGE_DEFOCUS = FILAMENT_AVERAGE_DEFOCUS(FILS(CURRENT_FIL))
            AVERAGE_CTF = FMC%FIL%FILM%CTFTILT%CTF
            CALL CTF_AT_Z(AVERAGE_CTF,REAL(AVERAGE_DEFOCUS,KIND=8),PIXEL_SIZE=PIXEL_SIZE%VALUE)
            IF (MICROGRAPH%is_in_real_space) CALL FWD_FT(MICROGRAPH)
            CALL IMAGE_APPLY_CTF(MICROGRAPH,AVERAGE_CTF)
            IF (.not. MICROGRAPH%is_in_real_space) CALL BWD_FT(MICROGRAPH)
        else
            MICROGRAPH%real_values = MICROGRAPH%real_values * (-1.0)
        ENDIF

        ! Let's run the scoring function to see what happens (we should get 1.0)
        IF (COMPUTE_SCORE) THEN
            FMC%STEP = DUMMY
            DUMMY = FMC_EVAL(FMC,FMC%FIL,DETAILLED_SCORE_INFO=.TRUE.,WRITE_STACKS=.FALSE.)
        ENDIF

        ! Add noise to the micrograph
        IF (ATTEMPT_NOISE_MODELLING) THEN
            CALL IMAGE_ADD_DETECTION_NOISE(MICROGRAPH,DOSE=INCIDENT_DOSE_PER_PIXEL,COUNTING=.FALSE.)
        ELSE
            WRITE(*,'(A,F0.3)') '**info(MODE_SYNTH_MICROGRAPH): adding Gaussian noise, target SNR = ', SNR%VALUE
            PROJ_VAR = PROJ_VAR / SIZE(PATH)
            CALL IMAGE_ADD_GAUSS_NOISE(MICROGRAPH,STDEV_NOISE=SQRT(REAL(PROJ_VAR))/SNR%VALUE)
        ENDIF


        ! Write out micrograph (note: multiple filaments per film not yet supported)
        CALL FILENAME_FROM_WILD(OUTPUT_FILM_FILENAMES%VALUE,FMC%FIL%FILM%ID,FILM_FILENAME)
        CALL FILM_FILE%INIT(FILM_FILENAME,PIXEL_SIZE=PIXEL_SIZE%VALUE)
        CALL MICROGRAPH%WriteToDisk(FILM_FILE)
        CALL FILM_FILE%PrintInfo()
        CALL FILM_FILE%CLOSE()

    ENDDO! End of loop over filaments


    ! Write out parameter file
    CALL FILAMENTS_TO_FREALIX_PAR(FILS,OUTPUT_PAR_FILENAME%VALUE)

    ! Debug: write out every single subunit
    !PATH = FILAMENT_GET_PATH(FMC%FIL,STEP=1.0,STEP_IN_REPEATS=.TRUE.)

    !WRITE(*,'(A)') '**debug(MODE_SYNTH_MICROGRAPH): subunit path waypoints follow'
    !CALL WAYPOINTS_PRINT_INFO(PATH)

END SUBROUTINE MODE_SYNTH_MICROGRAPH_SLAVE

END MODULE MODE_SYNTH_MICROGRAPH
