!>  \brief  Mode to test slice extraction from volume
MODULE MODE_TEST_EXTRACT_SLICE
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: TEST_EXTRACT_SLICE

    TYPE(RTP_CHAR), TARGET  ::  VOLUME_FILENAME  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='VOLUME_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  SLICE_FILENAME  =               &
        RTP_CHAR(   VALUE='test_out',                  &
                    KEYWORD='SLICE_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PHI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PHI',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  THETA  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='THETA',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PSI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PSI',           &
                    DESCRIPTION='')

CONTAINS

SUBROUTINE MODE_TEST_EXTRACT_SLICE_INIT_AND_RUN
    ! Intialise the mode
    CALL TEST_EXTRACT_SLICE%INIT(COMMAND='TEST_EXTRACT_SLICE',SUBROUTINE=MODE_TEST_EXTRACT_SLICE_SLAVE,&
                                SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL TEST_EXTRACT_SLICE%ADD_RTP(VOLUME_FILENAME)
    CALL TEST_EXTRACT_SLICE%ADD_RTP(SLICE_FILENAME)
    CALL TEST_EXTRACT_SLICE%ADD_RTP(PHI)
    CALL TEST_EXTRACT_SLICE%ADD_RTP(THETA)
    CALL TEST_EXTRACT_SLICE%ADD_RTP(PSI)

    ! Run the mode
    CALL TEST_EXTRACT_SLICE%RUN()
END SUBROUTINE MODE_TEST_EXTRACT_SLICE_INIT_AND_RUN


SUBROUTINE MODE_TEST_EXTRACT_SLICE_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    USE IMAGEFILES
    USE IMAGES_THREED
    USE THREEDS
    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)                 ::  VOLUME,SLICE,IMGTMP
    TYPE(WINDOW)                ::  WIN
    TYPE(INTERPOLATION_SCHEME)  ::  INTERPOL_SCHEME


    ! Start work
    CALL READ_FROM_DISK(VOLUME,VOLUME_FILENAME%VALUE)

    CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'SINC','HANN',4)
    CALL CORRECT_3D(VOLUME,INTERPOL_SCHEME%INTERPOLATION_INSTR_FUNCTION,    &
                    INTERPOL_SCHEME%WINDOWING_INSTR_FUNCTION,INTERPOL_SCHEME%WINDOW_WIDTH)




    !CALL VOLUME_PROJECT_LEAN(VOLUME,IMGTMP,DBLE(PHI%VALUE),DBLE(THETA%VALUE),DBLE(PSI%VALUE),INTERPOL_SCHEME)
    CALL FWD_FT(VOLUME)
    CALL SHIFT_REAL_SPACE_ORIGIN(VOLUME)

    CALL ALLOCATE_WINDOW(WIN,INTERPOL_SCHEME%WINDOW_WIDTH,IS_COMP(VOLUME),IS_THREED(VOLUME))
    CALL EXTRACT_SLICE(VOLUME,SLICE,DBLE(PHI%VALUE),DBLE(THETA%VALUE),DBLE(PSI%VALUE),INTERPOL_SCHEME,WIN,Z_SHIFT=0.D0)
    CALL SHIFT_REAL_SPACE_ORIGIN(SLICE)
    CALL BWD_FT(SLICE)
    CALL COSINE_RECT(SLICE,RECT_DIM=[80.,180.],AZIMUTH=20.0,FALLOFF=20.,WIPE_INPUT=.FALSE.,MASK_WITH_AVERAGE=.TRUE.)
    CALL WRITE_TO_DISK(SLICE,'dbg_proj_zshift_rect.mrc')

    CALL EXTRACT_SLICE(VOLUME,SLICE,DBLE(PHI%VALUE),DBLE(THETA%VALUE),DBLE(PSI%VALUE),INTERPOL_SCHEME,WIN)
    CALL SHIFT_REAL_SPACE_ORIGIN(SLICE)
    CALL BWD_FT(SLICE)

    CALL COSINE_RECT(SLICE,RECT_DIM=[20.,60.],AZIMUTH=20.0,FALLOFF=5.,WIPE_INPUT=.TRUE.,MASK_WITH_AVERAGE=.FALSE.)
    CALL WRITE_TO_DISK(SLICE,'dbg_rect.mrc')
    CALL AMPL(SLICE,IMGTMP)
    CALL WRITE_TO_DISK(IMGTMP,'dbg_rect_ampl.mrc')

    CALL COSINE_DISC(SLICE,RAD=20.,FALLOFF=5.,WIPE_INPUT=.TRUE.)
    CALL WRITE_TO_DISK(SLICE,'dbg_disk.mrc')
    CALL AMPL(SLICE,IMGTMP)
    CALL WRITE_TO_DISK(IMGTMP,'dbg_disk_ampl.mrc')


!   WRITE(*,'(/A)') '**debug(MODE_TEST_EXTRACT_SLICE): Slice print out'
!
!   CALL PRINT_IMAGE(SLICE)

    CALL WRITE_TO_DISK(SLICE,SLICE_FILENAME%VALUE)

END SUBROUTINE MODE_TEST_EXTRACT_SLICE_SLAVE

END MODULE MODE_TEST_EXTRACT_SLICE
