!>  \brief  Mode to test slice insertion into volume
MODULE MODE_TEST_INSERT_SLICE
    ! Modules
    USE MODES
    USE RTPS
    USE PARSERS
    IMPLICIT NONE
    TYPE(MODE) :: TEST_INSERT_SLICE

    TYPE(RTP_CHAR), TARGET  ::  VOLUME_FILENAME  =               &
        RTP_CHAR(   VALUE='test_inout',                  &
                    KEYWORD='VOLUME_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  SLICE_FILENAME  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='SLICE_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PHI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PHI',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  THETA  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='THETA',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PSI  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='PSI',           &
                    DESCRIPTION='')
CONTAINS

SUBROUTINE MODE_TEST_INSERT_SLICE_INIT_AND_RUN
    ! Intialise the mode
    CALL TEST_INSERT_SLICE%INIT(COMMAND='TEST_INSERT_SLICE',SUBROUTINE=MODE_TEST_INSERT_SLICE_SLAVE,    &
                                SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL TEST_INSERT_SLICE%ADD_RTP(VOLUME_FILENAME)
    CALL TEST_INSERT_SLICE%ADD_RTP(SLICE_FILENAME)
    CALL TEST_INSERT_SLICE%ADD_RTP(PHI)
    CALL TEST_INSERT_SLICE%ADD_RTP(THETA)
    CALL TEST_INSERT_SLICE%ADD_RTP(PSI)

    ! Run the mode
    CALL TEST_INSERT_SLICE%RUN()
END SUBROUTINE MODE_TEST_INSERT_SLICE_INIT_AND_RUN


SUBROUTINE MODE_TEST_INSERT_SLICE_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    USE IMAGES_THREED
    USE EULERANGLES
    USE ROTATION_MATRICES
    USE PARSERS
    use interpolation, only  : interpolation_scheme, INTERPOLATION_SCHEME_INIT

    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE)                 ::  VOLUME,SLICE
    TYPE(EULER)                 ::  ANGLES
    TYPE(INTERPOLATION_SCHEME)  ::  INTERPOL_SCHEME
    TYPE(ROTATION_MATRIX)       ::  MATRIX                      !   Rotation matrix to go from slice coordinates to 3D coordinates


    ! Start work
    CALL READ_FROM_DISK(SLICE,SLICE_FILENAME%VALUE)
    IF (FILE_EXISTS(VOLUME_FILENAME%VALUE)) THEN
        CALL READ_FROM_DISK(VOLUME,VOLUME_FILENAME%VALUE)
    ELSE
        VOLUME%logical_dimensions = SLICE%logical_dimensions(1)
        CALL IMAGE_ALLOCATE(VOLUME)
    ENDIF

!   WRITE(*,'(/A)') '**debug(MODE_TEST_EXTRACT_SLICE): Volume print out'
!
!   CALL PRINT_IMAGE(VOLUME)

    ! Fill the Eulerangles object
    ANGLES%PHI = PHI%VALUE
    ANGLES%THETA = THETA%VALUE
    ANGLES%PSI = PSI%VALUE

    ! Work out the rotation matrix to transform 2D slice coordinates into 3D reconstruction coordinates
    MATRIX%RMATRIX = EULER_TO_ROTMAT(ANGLES%PHI,ANGLES%THETA,ANGLES%PSI,REVERSE=.TRUE.)

    ! Initialise an interpolation scheme
    !CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'LINEAR','RECT',2)
    CALL INTERPOLATION_SCHEME_INIT(INTERPOL_SCHEME,'SINC','HANN',8)

!   CALL FWD_FT(VOLUME)
!   CALL SHIFT_REAL_SPACE_ORIGIN(VOLUME)
!   CALL FWD_FT(SLICE)
!   CALL SHIFT_REAL_SPACE_ORIGIN(SLICE)
    CALL INSERT_SLICE_PUT(VOLUME,SLICE,MATRIX,INTERPOL_SCHEME)
!   CALL SHIFT_REAL_SPACE_ORIGIN(VOLUME)
!   CALL BWD_FT(VOLUME)


!   WRITE(*,'(/A)') '**debug(MODE_TEST_EXTRACT_SLICE): Slice print out'
!
!   CALL PRINT_IMAGE(SLICE)

    CALL WRITE_TO_DISK(VOLUME,VOLUME_FILENAME%VALUE)

END SUBROUTINE MODE_TEST_INSERT_SLICE_SLAVE

END MODULE MODE_TEST_INSERT_SLICE
