!>  \brief  Mode to test the ROTATE_IMAGE subroutine
MODULE MODE_TEST_ROTATE
    ! Modules
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: TEST_ROTATE

    TYPE(RTP_CHAR), TARGET  ::  INPUT_FILE  =               &
        RTP_CHAR(   VALUE='test_in',                  &
                    KEYWORD='INPUT_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILE  =               &
        RTP_CHAR(   VALUE='test_out',                  &
                    KEYWORD='OUTPUT_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  ROTATION  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ROTATION',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  SHIFT_X  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='SHIFT_X',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  SHIFT_Y  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='SHIFT_Y',           &
                    DESCRIPTION='')

CONTAINS

SUBROUTINE MODE_TEST_ROTATE_INIT_AND_RUN
    ! Intialise the mode
    CALL TEST_ROTATE%INIT(COMMAND='TEST_ROTATE',SUBROUTINE=MODE_TEST_ROTATE_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL TEST_ROTATE%ADD_RTP(INPUT_FILE)
    CALL TEST_ROTATE%ADD_RTP(OUTPUT_FILE)
    CALL TEST_ROTATE%ADD_RTP(ROTATION)
    CALL TEST_ROTATE%ADD_RTP(SHIFT_X)
    CALL TEST_ROTATE%ADD_RTP(SHIFT_Y)

    ! Run the mode
    CALL TEST_ROTATE%RUN()
END SUBROUTINE MODE_TEST_ROTATE_INIT_AND_RUN


SUBROUTINE MODE_TEST_ROTATE_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    USE IMAGES_ARITHM
    USE IMAGES_COMPARISON
    USE ROTATION_MATRICES

    IMPLICIT NONE
    ! Variables
    TYPE(IMAGE) ::  IMGIN,IMGOUT
    REAL    ::  MATRIX1(4,4)!, MATRIX2(4,4)
    !REAL, ALLOCATABLE  ::  SHELLCORR(:)
    !REAL, ALLOCATABLE  ::  IMGIN_AVROT(:), IMGOUT_AVROT(:)


    ! Start work
    CALL READ_FROM_DISK(IMGIN,INPUT_FILE%VALUE)

    ! Allocate output and temp images with same dimensions as input image
    !CALL IMAGE_ALLOCATE(IMGOUT,IMGIN)
    !CALL IMAGE_ALLOCATE(IMGTMP,IMGIN)


    ! Prepare the rotation matrix (ROTATE_IMAGE expects the matrix that transforms the output image into the input image.
    MATRIX1 = ROTMAT(-ROTATION%VALUE,SHIFT_X%VALUE,SHIFT_Y%VALUE,SHIFTS_FIRST=.FALSE.)
    PRINT *, '**info(MODE_TEST_ROTATE): rotation first, shifts second'
    !WRITE(*,*) '**debug: MATRIX1 = '
    !CALL PRINT_ROTMAT(MATRIX1)

    ! Create an image from the interpolation kernel
    !CALL IMAGE_FROM_INTERPOL_FUNCTION(IMGTMP,LINEAR)
    !CALL WRITE_TO_DISK(IMGTMP,'dbg_interpol',FORMAT='I',LUN=99)


    ! Rotate the image
    CALL FWD_FT(IMGIN)
    CALL SHIFT_REAL_SPACE_ORIGIN(IMGIN)
    CALL IMAGE_ALLOCATE(IMGOUT,IMGIN)
!    CALL ROTATE_IMAGE(IMGIN,IMGOUT,MATRIX1)
    CALL SHIFT_REAL_SPACE_ORIGIN(IMGOUT)
    CALL BWD_FT(IMGOUT)
    ! Write output to disk
    CALL WRITE_TO_DISK(IMGOUT,OUTPUT_FILE%VALUE)


    ! Prepare the reverse rotation matrix
    !MATRIX1 = ROTMAT(0.0,-SHIFT_X%VALUE,-SHIFT_Y%VALUE)
    !MATRIX2 = ROTMAT(-ROTATION%VALUE,0.0,0.0)
    !MATRIX1 = MATMUL(MATRIX2,MATRIX1)
    !WRITE(*,*) '**debug: MATRIX1 = '
    !CALL PRINT_ROTMAT(MATRIX1)
    !PRINT *, 'rotation bwd'
    !IMGOUT%TYPE = 'COMP'
    !IMGOUT%complex_values = (0.0,0.0)
    !CALL ROTATE_IMAGE(IMGTMP,IMGOUT,MATRIX1,INTERPOL=LINEAR,WINDOW_WIDTH=2,WINDOW_TYPE=RECT)
    !IMGOUT = IMGTMP
    !IMGOUT%complex_values = IMGTMP%complex_values
    !IMGOUT = IMGIN

    !CALL SHIFT_REAL_SPACE_ORIGIN(IMGOUT)
    !CALL SHIFT_REAL_SPACE_ORIGIN(IMGIN)
    !CALL BWD_FT(IMGOUT)
    !CALL BWD_FT(IMGIN)



    ! Create an image from the interpolation window
    !IMGTMP%TYPE = 'COMP'
    !CALL IMAGE_FROM_WINDOW_FUNCTION(IMGTMP,WINDOW_FUNCTION=HANN,WINDOW_WIDTH=8)
    !CALL SHIFT_REAL_SPACE_ORIGIN(IMGTMP)
    !CALL BWD_FT(IMGTMP)
    !IMGTMP%real_values = ABS(IMGTMP%real_values)
    !IMGTMP%real_values = IMGTMP%real_values
    !CALL WRITE_TO_DISK(IMGTMP,'dbg_window',FORMAT='I',LUN=99)
    ! Divide the output
    !CALL DIVIDE_IMAGES(IMGOUT,IMGTMP,IMGOUT)

    ! Write output to disk
    !CALL WRITE_TO_DISK(IMGOUT,OUTPUT_FILE%VALUE)

    ! Write out profile to terminal
    !WRITE(*,*) '**debug: rotational average of input & output '
    !IMGIN_AVROT  = ROT_AVERAGE(IMGIN)
    !IMGOUT_AVROT = ROT_AVERAGE(IMGOUT)
    !DO I=1, HIGHEST_INDEX_DIM1(IMGIN%logical_dimensions(1))
        !WRITE(*,*) I-1, IMGIN_AVROT(I),IMGOUT_AVROT(I)
    !ENDDO

    ! Do FSC between input and output, printout results
    !CALL FWD_FT(IMGIN)
    !CALL FWD_FT(IMGOUT)
    !WRITE(*,'(/A)') 'FSC of output vs. input image'
    !CALL FSC(IMGIN,IMGOUT,SHELLCORR)
    !CALL PRINT_FSC(SHELLCORR)

END SUBROUTINE MODE_TEST_ROTATE_SLAVE

END MODULE MODE_TEST_ROTATE
