!>  \brief  Synthesise a CTF image and write it to disk. Subroutine for testing purposes mainly.
MODULE MODE_TEST_SPLINE
    USE MODES
    USE RTPS
    IMPLICIT NONE
    TYPE(MODE) :: TEST_SPLINE

    TYPE(RTP_CHAR), TARGET  ::  INPUT_BOX_FILE  =               &
        RTP_CHAR(   VALUE='input.box',                  &
                    KEYWORD='INPUT_BOX_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_BOX_FILE  =               &
        RTP_CHAR(   VALUE='output.box',                  &
                    KEYWORD='OUTPUT_BOX_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  CTFTILT_FILE  =               &
        RTP_CHAR(   VALUE='ctftilt.log',                  &
                    KEYWORD='CTFTILT_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_CHAR), TARGET  ::  FILM_FILE  =               &
        RTP_CHAR(   VALUE='film.mrc',                  &
                    KEYWORD='FILM_FILE',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=9.99,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='')

CONTAINS

SUBROUTINE MODE_TEST_SPLINE_INIT_AND_RUN
    ! Intialise the mode
    CALL TEST_SPLINE%INIT(COMMAND='TEST_SPLINE',SUBROUTINE=MODE_TEST_SPLINE_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL TEST_SPLINE%ADD_RTP(INPUT_BOX_FILE)
    CALL TEST_SPLINE%ADD_RTP(OUTPUT_BOX_FILE)
    CALL TEST_SPLINE%ADD_RTP(CTFTILT_FILE)
    CALL TEST_SPLINE%ADD_RTP(FILM_FILE)
    CALL TEST_SPLINE%ADD_RTP(PIXEL_SIZE)

    ! Run the mode
    CALL TEST_SPLINE%RUN()
END SUBROUTINE MODE_TEST_SPLINE_INIT_AND_RUN


SUBROUTINE MODE_TEST_SPLINE_SLAVE
    USE FILMS, ONLY : FILM, FILM_INIT
    USE FILAMENTS_BASE
    USE FILAMENTS_PATH, ONLY : FILAMENT_GET_PATH
    USE FILAMENTS_MODEL_BASE, ONLY : FILAMENT_PRINT_INFO
    USE COOS
    USE CTFTILTS
    USE FILAMENTS, ONLY : FILAMENT_INIT
    IMPLICIT NONE
    ! Private variables
    TYPE(BOXER_DB)  ::  DBIN,DBOUT
    TYPE(POINT), ALLOCATABLE     ::  TMP_POINTS(:)
    TYPE(WAYPOINT),ALLOCATABLE  ::  WP(:)
    !TYPE(CTFTILT)  ::  CTFT
    TYPE(FILAMENT)  ::  FIL
    TYPE(FILM)      ::  FILM1
    INTEGER ::  I
    INTEGER,    PARAMETER   ::  OUTPUT_BOX_DIM(3) = [10,10,1]

    ! Get variable values from control file


    ! Start work
    ! Set up the film
    !CALL FILM1%INIT(FILM_FILE%VALUE,PIXEL_SIZE%VALUE,CTFTILT_FILENAME=CTFTILT_FILE%VALUE)
    CALL FILM_INIT(FILM1,1,FILM_FILE%VALUE,PIXEL_SIZE%VALUE,CTFTILT_FILENAME=CTFTILT_FILE%VALUE)

    ! Read input coordinates
    !CALL DBIN%READ_FROM_DISK(INPUT_BOX_FILE%VALUE)
    CALL BOXER_DB_READ_FROM_DISK(DBIN,INPUT_BOX_FILE%VALUE)

    ! Initialise a filament - this includes a spline fit - since we're not giving way points (just points),
    ! this will be a parabolic spline, and the PSI and THETA angles will be worked out from it.
    ! Also, since we specify that the points are crossovers, PHI will be set to 0/180
    !CALL FIL%INIT(DBIN%POINT,FILM1,CROSSOVERS=.TRUE.)
    ALLOCATE(TMP_POINTS(DBIN%N))
    DO I=1,DBIN%N
        TMP_POINTS(I) = DBIN%POINT(I)
    ENDDO
    CALL FILAMENT_INIT(FIL,POINTS_IN=TMP_POINTS,FILM_IN=FILM1,CROSSOVERS=.TRUE.)
    ! Print some diagnostics
    !CALL FIL%PRINT_INFO()
    CALL FILAMENT_PRINT_INFO(FIL)
    ! Get path
    CALL FILAMENT_GET_PATH(FIL,WP,10.0D0,STEP_IN_PIXELS=.TRUE.)

    ! Diagnostics
    PRINT *, '**debug(MODE_TEST_SPLINE): interpolated waypoints...'
    CALL WAYPOINTS_PRINT_INFO(WP)
    ! output Boxer db
    !CALL DBOUT%ALLOCATE(SIZE(WP))
    CALL BOXER_DB_ALLOCATE(DBOUT,SIZE(WP))
!#ifdef __INTEL_COMPILER
!    DO CONCURRENT (I=1:SIZE(WP))
!        CALL POINT_ASSIGN(DBOUT%POINT(I),WP(I))
!    ENDDO
!#else
    FORALL (I=1:SIZE(WP))
        DBOUT%POINT(I) = WP(I)
    END FORALL
!#endif
    !CALL DBOUT%SET_BOX_SIZE(OUTPUT_BOX_DIM)
    !CALL DBOUT%WRITE_TO_DISK(OUTPUT_BOX_FILE%VALUE)
    CALL BOXER_DB_SET_BOX_SIZE(DBOUT,OUTPUT_BOX_DIM)
    CALL BOXER_DB_WRITE_TO_DISK(DBOUT,OUTPUT_BOX_FILE%VALUE)

END SUBROUTINE MODE_TEST_SPLINE_SLAVE

END MODULE MODE_TEST_SPLINE
