!>  \brief  Functionality for interactive use
MODULE UI

    INTERFACE GET_RTP_VALUE
        MODULE PROCEDURE GET_RTP_VALUE_LOGI
        MODULE PROCEDURE GET_RTP_VALUE_REAL
        MODULE PROCEDURE GET_RTP_VALUE_INTG
        MODULE PROCEDURE GET_RTP_VALUE_CHAR
    END INTERFACE GET_RTP_VALUE

    INTERFACE UI_GET_VALUE
        MODULE PROCEDURE UI_GET_CHAR
        MODULE PROCEDURE UI_GET_INTG
        MODULE PROCEDURE UI_GET_REAL
        MODULE PROCEDURE UI_GET_LOGI
    END INTERFACE

CONTAINS



    !>  brief   Find out which mode the user wants FREALIX to run by parsing the control file
    SUBROUTINE GET_FREALIX_MODE()
        USE RUNTIME_PARAMETERS
        USE PARSERS
        IMPLICIT NONE
        IF (INTERACTIVE) THEN
            CALL UI_GET_CHAR('Please give FREALIX_MODE', FREALIX_MODE, '(todo: add list of possible modes)')
        ELSE
            CALL FILE_GET_CHAR(CONTROL_FILENAME,'FREALIX_MODE',FREALIX_MODE)
        ENDIF
    END SUBROUTINE GET_FREALIX_MODE


    !>  Get a runtime parameter value, either from the control file or from the user interactively
    SUBROUTINE GET_RTP_VALUE_LOGI(PARAMETER,VALUE,QUESTION,HELP)
        USE RUNTIME_PARAMETERS
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),               INTENT(IN)      ::  PARAMETER   !< Label of parameter
        LOGICAL,                        INTENT(INOUT)   ::  VALUE       !< Value of parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  QUESTION    !< Question to ask user for parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  HELP        !< Help to give user

        ! Private variables
        CHARACTER(LEN=200)  ::  QQUESTION, HHELP

        ! Start work
        IF (PRESENT(QUESTION)) THEN
            QQUESTION = QUESTION
        ELSE
            QQUESTION = 'Please give ' // TRIM(PARAMETER)
        ENDIF
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'Sorry, no help available'
        ENDIF

        IF (INTERACTIVE) THEN
            CALL UI_GET_LOGI(QQUESTION,VALUE,HHELP)
        ELSE
            CALL FILE_GET_LOGI(CONTROL_FILENAME,PARAMETER,VALUE)
        ENDIF

    END SUBROUTINE GET_RTP_VALUE_LOGI

    !>  Get a runtime parameter value, either from the control file or from the user interactively
    SUBROUTINE GET_RTP_VALUE_REAL(PARAMETER,VALUE,QUESTION,HELP)
        USE RUNTIME_PARAMETERS
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),               INTENT(IN)      ::  PARAMETER   !< Label of parameter
        REAL,                           INTENT(INOUT)   ::  VALUE       !< Value of parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  QUESTION    !< Question to ask user for parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  HELP        !< Help to give user

        ! Private variables
        CHARACTER(LEN=200)  ::  QQUESTION, HHELP

        ! Start work
        IF (PRESENT(QUESTION)) THEN
            QQUESTION = QUESTION
        ELSE
            QQUESTION = 'Please give ' // TRIM(PARAMETER)
        ENDIF
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'Sorry, no help available'
        ENDIF

        IF (INTERACTIVE) THEN
            CALL UI_GET_REAL(QQUESTION,VALUE,HHELP)
        ELSE
            CALL FILE_GET_REAL(CONTROL_FILENAME,PARAMETER,VALUE)
        ENDIF

    END SUBROUTINE GET_RTP_VALUE_REAL


    !>  Get a runtime parameter value, either from the control file or from the user interactively
    SUBROUTINE GET_RTP_VALUE_INTG(PARAMETER,VALUE,QUESTION,HELP)
        USE RUNTIME_PARAMETERS
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),               INTENT(IN)      ::  PARAMETER   !< Label of parameter
        INTEGER,                        INTENT(INOUT)   ::  VALUE       !< Value of parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  QUESTION    !< Question to ask user for parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  HELP        !< Help to give user

        ! Private variables
        CHARACTER(LEN=200)  ::  QQUESTION, HHELP

        ! Start work
        IF (PRESENT(QUESTION)) THEN
            QQUESTION = QUESTION
        ELSE
            QQUESTION = 'Please give ' // TRIM(PARAMETER)
        ENDIF
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'Sorry, no help available'
        ENDIF

        !PRINT *, '**debug(GET_RTP_VALUE_INTG): looking for ', TRIM(PARAMETER), ', which has current value of ', VALUE

        IF (INTERACTIVE) THEN
            CALL UI_GET_INTG(QQUESTION,VALUE,HHELP)
        ELSE
            CALL FILE_GET_INTG(CONTROL_FILENAME,PARAMETER,VALUE)
        ENDIF

        !PRINT *, '**debug(GET_RTP_VALUE_INTG): was looking for ', TRIM(PARAMETER), ', which now has value of ', VALUE

    END SUBROUTINE GET_RTP_VALUE_INTG


    !>  Get a runtime parameter value, either from the control file or from the user interactively
    SUBROUTINE GET_RTP_VALUE_CHAR(PARAMETER,VALUE,QUESTION,HELP)
        USE RUNTIME_PARAMETERS
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),               INTENT(IN)      ::  PARAMETER   !< Label of parameter
        CHARACTER(LEN=*),               INTENT(INOUT)   ::  VALUE       !< Value of parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  QUESTION    !< Question to ask user for parameter
        CHARACTER(LEN=*),   OPTIONAL,   INTENT(IN)      ::  HELP        !< Help to give user

        ! Private variables
        CHARACTER(LEN=200)  ::  QQUESTION, HHELP

        ! Start work
        IF (PRESENT(QUESTION)) THEN
            QQUESTION = QUESTION
        ELSE
            QQUESTION = 'Please give ' // TRIM(PARAMETER)
        ENDIF
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'Sorry, no help available'
        ENDIF


        IF (INTERACTIVE) THEN
            CALL UI_GET_CHAR(QQUESTION,VALUE,HHELP)
        ELSE
            CALL FILE_GET_CHAR(CONTROL_FILENAME,PARAMETER,VALUE)
        ENDIF

    END SUBROUTINE GET_RTP_VALUE_CHAR



    !>  brief   Writes the prompt to terminal
    SUBROUTINE PROMPT
        IMPLICIT NONE

        WRITE(*,'(A)',ADVANCE='NO') '<frealix> '
    END SUBROUTINE PROMPT


    SUBROUTINE UI_GET_LOGI(QUESTION,VALUE,HELP)
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),   INTENT(IN)  ::  QUESTION    !<  The question to which the user will reply
        LOGICAL,            INTENT(OUT) ::  VALUE       !<  The value the user provided as an answer
        CHARACTER(LEN=*),   INTENT(IN), OPTIONAL    ::  HELP    !<  Help string given to the user if they type '?'
        ! Private variables
        LOGICAL ::  ANSWERED = .FALSE.
        CHARACTER(LEN=128)  ::  ANSWER, HHELP

        ! We start off not knowing the answer
        ANSWERED = .FALSE.

        ! Start work
        !VALUE = 0.0
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'No help available, sorry'
        ENDIF

        DO WHILE (.NOT. ANSWERED)
            CALL PROMPT()
            WRITE (*,'(2A)',ADVANCE='NO') TRIM(QUESTION), ': '
            READ (*,'(A)') ANSWER
            IF (IS_BLANK(ANSWER)) CYCLE
            IF (STARTS_WITH_QUESTION_MARK(ANSWER)) THEN
                WRITE(*,'(/A/)') HHELP
                CYCLE
            ENDIF
            IF (IS_LOGI(ANSWER)) THEN
                ANSWERED = .TRUE.
                READ(ANSWER,*) VALUE
            ELSE
                CYCLE
            ENDIF
        ENDDO
    END SUBROUTINE UI_GET_LOGI


    SUBROUTINE UI_GET_REAL(QUESTION,VALUE,HELP)
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*), INTENT(IN)    ::  QUESTION    !<  The question to which the user will reply
        REAL,               INTENT(OUT) ::  VALUE       !<  The value the user provided as an answer
        CHARACTER(LEN=*),   INTENT(IN), OPTIONAL    ::  HELP    !<  Help string given to the user if they type '?'
        ! Private variables
        LOGICAL ::  ANSWERED
        CHARACTER(LEN=128)  ::  ANSWER, HHELP

        ! We start off not knowing the answer
        ANSWERED = .FALSE.

        ! Start work
        VALUE = 0.0
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'No help available, sorry'
        ENDIF

        DO WHILE (.NOT. ANSWERED)
            CALL PROMPT()
            WRITE (*,'(2A)',ADVANCE='NO') TRIM(QUESTION), ': '
            READ (*,'(A)') ANSWER
            IF (IS_BLANK(ANSWER)) CYCLE
            IF (STARTS_WITH_QUESTION_MARK(ANSWER)) THEN
                WRITE(*,'(/A/)') HHELP
                CYCLE
            ENDIF
            IF (IS_REAL(ANSWER)) THEN
                ANSWERED = .TRUE.
                READ(ANSWER,*) VALUE
            ELSE
                CYCLE
            ENDIF
        ENDDO
    END SUBROUTINE UI_GET_REAL

    SUBROUTINE UI_GET_INTG(QUESTION,VALUE,HELP)
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*),   INTENT(IN)  ::  QUESTION    !<  The question to which the user will reply
        INTEGER,            INTENT(OUT) ::  VALUE       !<  The value the user provided as an answer
        CHARACTER(LEN=*),   INTENT(IN), OPTIONAL    ::  HELP    !<  Help string given to the user if they type '?'
        ! Private variables
        LOGICAL ::  ANSWERED = .FALSE.
        CHARACTER(LEN=128)  ::  ANSWER, HHELP

        ! We start off not knowing the answer
        ANSWERED = .FALSE.

        ! Start work
        VALUE = 0
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'No help available, sorry'
        ENDIF

        DO WHILE (.NOT. ANSWERED)
            CALL PROMPT()
            WRITE (*,'(2A)',ADVANCE='NO') TRIM(QUESTION), ': '
            READ (*,'(A)') ANSWER
            IF (IS_BLANK(ANSWER)) CYCLE
            IF (STARTS_WITH_QUESTION_MARK(ANSWER)) THEN
                WRITE(*,'(/A/)') HHELP
                CYCLE
            ENDIF
            IF (IS_INTG(ANSWER)) THEN
                ANSWERED = .TRUE.
                READ(ANSWER,*) VALUE
            ELSE
                CYCLE
            ENDIF
        ENDDO
    END SUBROUTINE UI_GET_INTG

    SUBROUTINE UI_GET_CHAR(QUESTION,VALUE,HELP)
        USE PARSERS
        IMPLICIT NONE
        ! Arguments
        CHARACTER(LEN=*), INTENT(IN)    ::  QUESTION    !<  The question to which the user will reply
        CHARACTER(LEN=*),   INTENT(OUT) ::  VALUE       !<  The value the user provided as an answer
        CHARACTER(LEN=*),   INTENT(IN), OPTIONAL    ::  HELP    !<  Help string given to the user if they type '?'
        ! Private variables
        LOGICAL ::  ANSWERED = .FALSE.
        CHARACTER(LEN=128)  ::  ANSWER, HHELP

        ! We start off not knowing the answer
        ANSWERED = .FALSE.

        ! Start work
        VALUE = ' '
        IF (PRESENT(HELP)) THEN
            HHELP = HELP
        ELSE
            HHELP = 'No help available, sorry'
        ENDIF

        DO WHILE (.NOT. ANSWERED)
            CALL PROMPT()
            WRITE (*,'(2A)',ADVANCE='NO') TRIM(QUESTION), ': '
            READ (*,'(A)') ANSWER
            !PRINT *, '**debug (UI_GET_CHAR): got answer ', ANSWER
            IF (IS_BLANK(ANSWER)) CYCLE
            IF (STARTS_WITH_QUESTION_MARK(ANSWER)) THEN
                WRITE(*,'(/A/)') HHELP
                CYCLE
            ENDIF
            ANSWERED = .TRUE.
            READ(ANSWER,'(A)') VALUE
            !PRINT *, '**debug (UI_GET_CHAR): got value ', value
        ENDDO
    END SUBROUTINE UI_GET_CHAR

    !>  \brief  Use this function to work out whether you should print output for iteration CURRENT_ITERATION of a total of NUM_ITERATIONS iterations
    LOGICAL FUNCTION INTERVAL_PRINT(CURRENT_ITERATION,NUM_ITERATIONS)
        IMPLICIT NONE
        ! Arguments
        INTEGER,    INTENT(IN)  ::  CURRENT_ITERATION           !<  Current iteration number
        INTEGER,    INTENT(IN)  ::  NUM_ITERATIONS              !<  Total number of iterations
        ! Private variables
        INTEGER,    PARAMETER   ::  NUM_MESSAGES    =   10      !<  How many messages should be printed over the course of the iterations?
        ! Start work

        !
        ! We want to print on the first iteration, and in total 10 times, at regular intervals
        !
        IF (NUM_ITERATIONS .GE. NUM_MESSAGES) THEN
            IF (MOD(CURRENT_ITERATION,NUM_ITERATIONS/NUM_MESSAGES) .EQ. 0.0 .OR. CURRENT_ITERATION .EQ. 1) THEN
                INTERVAL_PRINT = .TRUE.
            ELSE
                INTERVAL_PRINT = .FALSE.
            ENDIF
        ELSE
            INTERVAL_PRINT = .TRUE.
        ENDIF
    END FUNCTION INTERVAL_PRINT


END MODULE UI
