!>  \brief  Synthesise a CTF image and write it to disk. Subroutine for testing purposes mainly.
MODULE MODE_SYNTH_CTF
    USE MODES
	USE RTPS
	IMPLICIT NONE
    TYPE(MODE) :: SYNTH_CTF

    TYPE(RTP_CHAR), TARGET  ::  OUTPUT_FILENAME  =               &
        RTP_CHAR(   VALUE='ctf_out',                  &
                    KEYWORD='OUTPUT_FILENAME',           &
                    DESCRIPTION='')


    TYPE(RTP_REAL), TARGET  ::  CS  =               &
        RTP_REAL(   VALUE=2.0,                  &
                    KEYWORD='CS',           &
                    DESCRIPTION='Spherical aberration (mm )')


    TYPE(RTP_REAL), TARGET  ::  AKV  =               &
        RTP_REAL(   VALUE=300.0,                  &
                    KEYWORD='AKV',           &
                    DESCRIPTION='Acceleration voltage (kV)')


    TYPE(RTP_REAL), TARGET  ::  AMPL_CONT  =               &
        RTP_REAL(   VALUE=0.1,                  &
                    KEYWORD='AMPL_CONT',           &
                    DESCRIPTION='Fraction of amplitude contrast (from 0. to 1., usually btw .07 and .14, see Mindell 03)')


    TYPE(RTP_REAL), TARGET  ::  DFMID1  =               &
        RTP_REAL(   VALUE=30000,                  &
                    KEYWORD='DFMID1',           &
                    DESCRIPTION='Defocus along first axis (Angstrom)')


    TYPE(RTP_REAL), TARGET  ::  DFMID2  =               &
        RTP_REAL(   VALUE=30000,                  &
                    KEYWORD='DFMID2',           &
                    DESCRIPTION='Defocus along second axis (Angstrom)')


    TYPE(RTP_REAL), TARGET  ::  ANGAST  =               &
        RTP_REAL(   VALUE=0.0,                  &
                    KEYWORD='ANGAST',           &
                    DESCRIPTION='Azimuth of first axis. 0.0 means axis is at 3 o''clock. (deg)')


    TYPE(RTP_REAL), TARGET  ::  PIXEL_SIZE  =               &
        RTP_REAL(   VALUE=1.0,                  &
                    KEYWORD='PIXEL_SIZE',           &
                    DESCRIPTION='Output pixel size (Angstrom)')


    TYPE(RTP_INTG), TARGET  ::  IMAGE_SIZE  =               &
        RTP_INTG(   VALUE=128,                  &
                    KEYWORD='IMAGE_SIZE',           &
                    DESCRIPTION='Size of output image (pixels, image will be square)')

CONTAINS

SUBROUTINE MODE_SYNTH_CTF_INIT_AND_RUN
    IMPLICIT NONE
    ! Intialise the mode
    CALL SYNTH_CTF%INIT(COMMAND='SYNTH_CTF',SUBROUTINE=MODE_SYNTH_CTF_SLAVE,SHORT_DESCRIPTION='Add description here')

    ! Add RTPs to the mode
    CALL SYNTH_CTF%ADD_RTP(OUTPUT_FILENAME)
    CALL SYNTH_CTF%ADD_RTP(CS)
    CALL SYNTH_CTF%ADD_RTP(AKV)
    CALL SYNTH_CTF%ADD_RTP(AMPL_CONT)
    CALL SYNTH_CTF%ADD_RTP(DFMID1)
    CALL SYNTH_CTF%ADD_RTP(DFMID2)
    CALL SYNTH_CTF%ADD_RTP(ANGAST)
    CALL SYNTH_CTF%ADD_RTP(PIXEL_SIZE)
    CALL SYNTH_CTF%ADD_RTP(IMAGE_SIZE)

    ! Run the mode
    CALL SYNTH_CTF%RUN()
END SUBROUTINE MODE_SYNTH_CTF_INIT_AND_RUN

SUBROUTINE MODE_SYNTH_CTF_SLAVE
    USE RUNTIME_PARAMETERS
    USE IMAGES
    USE IMAGES_CTF
    USE IMAGEFILES
    IMPLICIT NONE
    ! Private variables
    TYPE(IMAGE)             ::  IMGOUT
    TYPE(CTF_PARAMETERS)    ::  CTF


    ! Get variable values from control file


    ! Prepare the CTF object
    CTF%PIX_UNITS   = .FALSE.
    CTF%CS          = CS%VALUE
    CTF%WL          = AKV_TO_WL(AKV%VALUE)
    CTF%AMPL_CONT   = AMPL_CONT%VALUE
    CTF%DFMID1      = DFMID1%VALUE
    CTF%DFMID2      = DFMID2%VALUE
    CTF%ANGAST      = ANGAST%VALUE

    ! There are now two ways to proceed. The more efficient way is first, but the alternative (commented out) is left here
    ! for future reference

    !! Preferred way !!

    ! Prepare the CTF image
    IMGOUT%logical_dimensions = (/IMAGE_SIZE%VALUE,IMAGE_SIZE%VALUE,1/)

    ! Calculate the CTF image
    WRITE(*,*) '**debug(SYNTH_CTF): calling CTF_TO_IMG', IMGOUT%logical_dimensions, CTF
    CALL CTF_TO_IMG(CTF,IMGOUT,PIXEL_SIZE%VALUE,ORIGIN_AT_CENTER = .TRUE.)


    !! Alternative way (slightly less efficient) !!

    ! Prepare the CTF image
    !IMGOUT%logical_dimensions = (/IMAGE_SIZE%VALUE/2+1,IMAGE_SIZE%VALUE,1/)

    ! Calculate the CTF image
    !CALL CTF_TO_IMG(CTF,IMGOUT,PIXEL_SIZE%VALUE)

    ! Put origin at image center (the image is now full-size
    !CALL ORIGIN_TO_CENTER(IMGOUT)


    !! End of alternative methods !!


    ! Write out the output image
    CALL WRITE_TO_DISK(IMGOUT,OUTPUT_FILENAME%VALUE)

END SUBROUTINE MODE_SYNTH_CTF_SLAVE

END MODULE MODE_SYNTH_CTF
